#!/usr/local/bin/python3
# coding=utf-8

"""
INFO
*** Paketaufpbau

- gui           GUI-Komponenten (MVC)
- logik         Die zentrale Logik
- helper        Ausgelagerte etwas allgemeinere Verwaltungslogik (befindet sich bezueglich Komplexitaet/Spezialisierung zwischen Logik und Tools)
- tools         Allgemeine Tools
critterbase.py
start.pyw

"""
import random

try:
    import argparse, sys, os, textwrap, getpass, configparser, datetime, requests, re, psutil, subprocess, urllib, ssl, webbrowser, folium, zipfile, pathlib, psycopg2, csv
    from collections import OrderedDict
    from PyQt5.QtWidgets import *
    from gui.controller import Controller
    from gui.model import Model
    from tools.printing import SmartPrinter
    from logik.batchbot import BatchBot
    #from tools.text import TextEncoding
    from tools.fileio import FileTools
    from helper.database import Database
    from tools.text import PrettyText
    from tools.coding import Result,Package
    from tools.multisheet import MultiSheet
    from tools.taxon import Taxon
    from tools.taxa import Taxa
    from tools.ui import ColorGradient, Colors
    from logik.ingestor import Ingestor
    from tools.geocruncher import GeoCruncher
    from tools.taxacruncher import TaxaCruncher
    from distutils.dir_util import copy_tree
    from tools.excel import ExcelTools, ExcelNavigator
    from helper.thread_container import Worker_CheckDois
    import openpyxl, warnings

except ImportError as ex:
    print("Module missing ("+str(__file__)+"): " + ex.name + "\nPlease install.")
    exit(1)

# Main
if __name__ == "__main__":

    # Hilfsmethoden
    def testFileExistsRead(sFile):
        if not os.path.isfile(sFile):
            print("File '"+sFile+"' does not exist ... Abort")
            exit(1)

    def testFileExistsWrite(sFile):
        if os.path.isfile(sFile):
            print("File '"+sFile+"' does exist")
            ein = input("Overwrite the file [yN]? ")
            if ein == "" or ein.lower()[0] not in ('y','j'):
                print("Abort ...")
                exit(1)

    # def testEncoding(sFile):
    #     sFileEndung = os.path.splitext(sFile)[1]
    #     if sFileEndung == ".csv":
    #         print("* Checking type of text encoding")
    #         (sEncoding, fConfidence) = TextEncoding.checkout(sFile)
    #         print("Determined type of encoding: " + sEncoding.upper() + " (Probability: " + str(
    #             round(fConfidence * 100)) + "%)")
    #         if sEncoding.lower() not in ("ascii", "utf-8", "ibm852", "iso-8859-1"):
    #             print("Found a not allowed type of encoding. Abort ...")
    #             exit(1)

    # Model erzeugen

    m = Model()
    m.ui_getSmartprinter().setCliNoPorgress(False)
    sImportTypes = PrettyText.lst2strC(m.subsystem_getMapper().workflow_getAll())

    # Workflowliste
    # Normal
    lstWfInfo = []
    for sWfShort in m.subsystem_getMapper().workflow_getAll():
        sWfLong = m.subsystem_getMapper().workflow_short2long(sWfShort)
        lstWfInfo.append(sWfShort + " (" + sWfLong+")")
    sWf_nativ_liste = PrettyText.lst2strC(lstWfInfo)
    # Converter: Import
    lstWf_Cnv_import = []
    for sWfCnfShort in m.subsystem_getMapper().workflow_converter_import_getAll():
        sWfCnfLong = m.subsystem_getMapper().workflow_converter_import_short2long(sWfCnfShort)
        lstWf_Cnv_import.append(sWfCnfShort + " (" + sWfCnfLong + ")")
    sWf_Cnv_import_liste = PrettyText.lst2strC(lstWf_Cnv_import)
    # Converter: Export
    lstWf_Cnv_export = []
    for sWfCnfShort in m.subsystem_getMapper().workflow_converter_export_getAll():
        sWfCnfLong = m.subsystem_getMapper().workflow_converter_export_short2long(sWfCnfShort)
        lstWf_Cnv_export.append(sWfCnfShort + " (" + sWfCnfLong + ")")
    sWf_Cnv_export_liste = PrettyText.lst2strC(lstWf_Cnv_export)
    # BatchBot Configs
    sBatchBotConfigs = PrettyText.lst2strC(m.subsystem_getBatchBot().getConfig_all())

    # CLI-Parser
        # Kurzinfo zum Programm
    sInfo = m.conf_getInfo("prgname") + " " + m.conf_getInfo("version") + " (" + m.conf_getInfo("dbschema") + ")\n" \
            + m.conf_getInfo("comment").replace("<br>", "\n").replace("<i>", "").replace("</i>", "") + "\n" \
            "Contact: " + m.conf_getInfo("www") + "\n" + \
            m.conf_getInfo("date")
        # Importer/Exporter: Arten des Importes
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=textwrap.dedent(sInfo),
        add_help=False)

    # Import Ingest
    gruppeImportIngest = parser.add_argument_group("Import: Ingest")
    gruppeImportIngest.add_argument("-imp-ing", metavar=("<workflow>", "<file.xlsx>"), help="Import selected ingest tables from Excel file (Workflows: direct: " + sWf_nativ_liste + ", with conversion: " + sWf_Cnv_import_liste, type=str, nargs=2)
    gruppeImportIngest.add_argument("-imp-ing-ext", help="Allow extension of records in database (yes/no)", type=str)
    gruppeImportIngest.add_argument("-imp-ing-ext-react", help="Allow extension of records in database with reaction level if not matching 100 percent", type=str)
    gruppeImportIngest.add_argument("-imp-ing-lup-single", help="Import each lookup on the fly", action="store_true")

    # Import Lookup
    gruppeImportLookup = parser.add_argument_group("Import: Lookup")
    gruppeImportLookup.add_argument("-imp-lup", metavar=("<file.xlsx>", "<table>"), help="Add selected lookup tables from Excel file (Sheets: " + ", ".join(m.subsystem_getMapper().lookup_getExcelSheets()), type=str, nargs='+')
    gruppeImportLookup.add_argument("-imp-lup-in", metavar=("<type>"), help="Type of records to add (types: all,gold,not-gold)", type=str)
    gruppeImportLookup.add_argument("-imp-lup-mode", metavar=("<mode>"), help="Mode of adding data records: (modes: new,update,both)", type=str)
    gruppeImportLookup.add_argument("-imp-lup-update", metavar=("<type>"), help="Type of matching records to update: (types: all,gold,not-gold)", type=str)
    gruppeImportLookup.add_argument("-imp-lup-reuse-react", metavar=("<type>"), help="When update is dewactivated, reaction if existing record is not 100 percent matching with import: (types: error,waring,info)", type=str)
    gruppeImportLookup.add_argument("-imp-lup-goldrush", help="Only allow gold records while import", action="store_true")

    # Import TaxonWizard Data
    gruppeImportTaxon = parser.add_argument_group("Import: TaxonWizard")
    gruppeImportTaxon.add_argument("-imp-tax", metavar=("<file.xlsx>", "<table>"),
                                   help="Add selected lookup tables from Excel file (Sheets: " + ", ".join(
                                        m.subsystem_getMapper().taxon_getExcelSheets()), type=str, nargs='+')
    gruppeImportTaxon.add_argument("-imp-tax-update", help="update existing db records", action="store_true")

    # Esport Subset
    gruppeExportSubset = parser.add_argument_group("Export: Subset")
    gruppeExportSubset.add_argument("-exp-sub-file", metavar=("<file.xlsx>"), help="Export to Excel file - if not set console output is active", type=str)
    gruppeExportSubset.add_argument("-exp-sub-converter", metavar=("<converter>"), help="Use converter for ingest data export: " + sWf_Cnv_export_liste, type=str)
    gruppeExportSubset.add_argument("-exp-sub-selector", help="Show selector ranges", action="store_true")
    gruppeExportSubset.add_argument("-exp-sub-filter-date-start", metavar=("<date start>"), type=str)
    gruppeExportSubset.add_argument("-exp-sub-filter-date-end", metavar=("<date end>"), type=str)
    gruppeExportSubset.add_argument("-exp-sub-filter-ing", metavar=("<ingest id>"), nargs="+")
    gruppeExportSubset.add_argument("-exp-sub-filter-gear", metavar=("<gear>"), nargs="+")
    gruppeExportSubset.add_argument("-exp-sub-filter-cov", metavar=("<taxonomic coverage>"), nargs="+")
    gruppeExportSubset.add_argument("-exp-sub-filter-dat", metavar=("<dataset>"), nargs="+")
    gruppeExportSubset.add_argument("-exp-sub-filter-region", metavar=("<region>"), nargs="+")
    gruppeExportSubset.add_argument("-exp-sub-filter-wf", metavar=("<workflows>"), nargs="+")
    gruppeExportSubset.add_argument("-exp-sub-id", help="Export ids of each record", action="store_true")
    gruppeExportSubset.add_argument("-exp-sub-ing-id", help="Export ingest id of each record", action="store_true")
    gruppeExportSubset.add_argument("-exp-sub-rfriendly", help="Export R-friendly", action="store_true")
    gruppeExportSubset.add_argument("-exp-sub-sheets", metavar=("<sheet>"), nargs="+")
    gruppeExportSubset.add_argument("-exp-sub-stat", help="Only show information an no actual records in console output mode", action="store_true")

    # Export Template
    gruppeExportTemplate = parser.add_argument_group("Export: Template")
    gruppeExportTemplate.add_argument("-exp-template", metavar=("<workflow>"), help="Available workflows: native: " + sWf_nativ_liste + "; for converter import: " + sWf_Cnv_import_liste, type=str)

    # Export Lookup
    gruppeExportLookup = parser.add_argument_group("Export: Lookup")
    gruppeExportLookup.add_argument("-exp-lup-all", help="Export all lookups", action="store_true")
    gruppeExportLookup.add_argument("-exp-lup-ing", metavar=("<ingest id>"), nargs="+", help="Export used lookups for given ingests")
    gruppeExportLookup.add_argument("-exp-lup-id", help="Export ids of each record", action="store_true")
    gruppeExportLookup.add_argument("-exp-lup-sheets", metavar=("<sheet>"), nargs="+", help="Export only given lookups")
    gruppeExportLookup.add_argument("-exp-lup-gold", metavar=("<all/gold/not-gold>"), type=str, help="Export only lookups with the following gold status: all, gold, not-gold lookups")
    gruppeExportLookup.add_argument("-exp-lup-file", metavar=("<file.xlsx>"), help="Export to Excel file - if not set console output is active", type=str)

    # TaxonWizard: Update Taxon List
    gruppeTaxWiz = parser.add_argument_group("TaxonWizard: TaxonList")
    gruppeTaxWiz.add_argument("-tax-update", help="Update the taxon list", action="store_true")

    # Export TaxonWizard
    gruppeExportTaxon = parser.add_argument_group("Export Data: TaxonWizard")
    gruppeExportTaxon.add_argument("-exp-tax", help="Export all taxon system admin tables", action="store_true")
    gruppeExportTaxon.add_argument("-exp-tax-sheets", metavar=("<sheet>"), nargs="+", help="Export only given taxon admin tables")
    gruppeExportTaxon.add_argument("-exp-tax-file", metavar=("<file.xlsx>"),
                                    help="Export to Excel file - if not set console output is active", type=str)
    # Database
    gruppeDB = parser.add_argument_group("Database")
    gruppeDB.add_argument("-db", metavar="<number of account>", help="Display/configure database connection for account number (0-4)", type=str, nargs=1)

    # BatchBot
    gruppeBatch = parser.add_argument_group("Batch")
    gruppeBatch.add_argument("-bot", metavar="<configname>", help="Start the BatchBot (Configs: "+ sBatchBotConfigs+")", type=str, nargs=1)

    # FAO
    gruppeBatch = parser.add_argument_group("FAO")
    gruppeBatch.add_argument("-fao-new", metavar=("<level>"), help="Create new FAO regions (all: all available level, std: major, division, subarea)", type=str, nargs="+")
    gruppeBatch.add_argument("-fao-del", help="Delete mapping (ingest_id: delete mapping for ingest id only, or when nocthing is given delete all)",  metavar="<ingest id>", nargs="?", type=int, const=-1)
    gruppeBatch.add_argument("-fao-update", help="Update mapping", action="store_true")
    gruppeBatch.add_argument("-fao-stat", metavar=("<size>"), help="Show info on mapping (size: short, long <default>)", nargs="?", type=str, const="full")

    # Etc
    gruppeEtc = parser.add_argument_group("Etc")
    gruppeEtc.add_argument("-gui", metavar="Style", help="Start in GUI mode, all other parameters are ignored. (Styles: "+", ".join(QStyleFactory.keys())+")", type=str, nargs='?', const="_system_")
    gruppeEtc.add_argument("-i", metavar=("<sub system>"), help="Shows information on sub systems: all, " + PrettyText.lst2strC(m.subsystem_getSubLst()), type=str, nargs='+')
    gruppeEtc.add_argument("-dry", help="writing database actions are only simulated", action="store_true")
    gruppeEtc.add_argument("-cnp", help="no progress bar in CLI mode", action="store_true")
    gruppeEtc.add_argument("-v", help="verbose", action="store_true")
    gruppeEtc.add_argument("-d", help="debugging", action="store_true")
    gruppeEtc.add_argument("-vs", "--version", help="Version", action="store_true")
    gruppeEtc.add_argument("-update", help="Check for available pdates from server", action="store_true")
    gruppeEtc.add_argument("-h", "--help", help="Shows this help", action="store_true")

    # Testing
    gruppeTesting = parser.add_argument_group("TESTING")
    gruppeTesting.add_argument("-test", help="Generic test area", action="store_true")

    # Parse CLI-Argumente
    try:
        args = parser.parse_args()
    except:
        print("Faulty parameters ... abort")
        exit(1)

    # --- Keine Serververbindung nötig ---

    if len(sys.argv) <= 1:
        # Nichts übergeben -> GUI starten
        del m  # Sauberkeit: loesche das Model, welches hier im CLI-Modus benutzt wird
        Controller()
        exit(1)
    elif args.gui:
        # GUI
        del m # Sauberkeit: loesche das Model, welches hier im CLI-Modus benutzt wird
        sGuiStyle = args.gui
        if args.gui == "_system_":
            sGuiStyle = None
        Controller(sGuiStyle)
        exit(1)
    elif args.help:
        # Hilfe
        parser.print_help()
        exit(1)
    elif args.version:
        # Version
        print(m.conf_getInfo("version") + " (" + m.conf_getInfo("dbschema") + ")")
        exit(1)

    # Veraenderungen (zur eingelesenen Konfigurationsdatei) am Model vornehmen (CLI-Parameter)
        # Geschwaetzigkeit
    m.conf_setVerbosity(args.v, args.d)
        # Dry Run
    m.conf_setDry(args.dry)
        # Logging
    m.conf_setLogging(False) # Std: Logging ausschalten
        # CLI: Kein Progressbar anzeigen
    m.ui_getSmartprinter().setCliNoPorgress(args.cnp)

    # Server-Config veraendern
    if args.db:
        iAccount = int(args.db[0])
        print("* Settings of the database connection")
        print("Account number: " + str(iAccount))
        if iAccount < 0 or iAccount >= m.subsystem_getDb().account_getNumberOfAccounts():
            print("number not valid")
            exit(1)
        m.subsystem_getDb().account_setIndexOfActiveAccount(iAccount)
        if m.subsystem_getDb().account_single_getPart("pw") != "":
            print("Test of the server connection: ", end="")
            resCheck = m.subsystem_getDb().check_isSchemaExisting()
            bOk = resCheck.getData("exist") if resCheck.isOk() else False
            print("ok" if bOk == True else "")
        print("---")
        print("Preset: " + m.subsystem_getDb().account_single_getPart("preset"))
        print("Host:   " + m.subsystem_getDb().account_single_getPart("host"))
        print("Port:   " + m.subsystem_getDb().account_single_getPart("port"))
        print("Name:   " + m.subsystem_getDb().account_single_getPart("name"))
        print("Schema: " + m.subsystem_getDb().account_single_getPart("schema"))
        print("User:   " + m.subsystem_getDb().account_single_getPart("user"))
        if m.subsystem_getDb().account_single_getPart("pw") == "":
            print("Pw:     <not saved>")
        else:
            print("Pw:     <saved>")
        print("---")
        ein = input("Should the data be changed [yN]? ")
        if ein == "" or ein.lower()[0] not in ('y', 'j'):
            exit(1)
        print("* Change of the database connection")
        ein = input("Preset (" + m.subsystem_getDb().account_single_getPart("preset") + "): ")
        if ein != "": m.subsystem_getDb().account_single_setPart("preset", ein)
        ein = input("Server (" + m.subsystem_getDb().account_single_getPart("host") + "): ")
        if ein != "": m.subsystem_getDb().account_single_setPart("host", ein)
        ein = input("Port (" + m.subsystem_getDb().account_single_getPart("port") + "): ")
        if ein != "": m.subsystem_getDb().account_single_setPart("port", ein)
        ein = input("DB-Name (" + m.subsystem_getDb().account_single_getPart("name") + "): ")
        if ein != "": m.subsystem_getDb().account_single_setPart("name", ein)
        ein = input("DB-Schema (" + m.subsystem_getDb().account_single_getPart("schema") + "): ")
        if ein != "": # Check auf Alphanumerik
            if m.subsystem_getDb().account_single_setPart("schema", ein) == False:
                exit(1)
        ein = input("User(" + m.subsystem_getDb().account_single_getPart("user") + "): ")
        if ein != "": m.subsystem_getDb().account_single_setPart("user", ein)
        if m.subsystem_getDb().account_single_getPart("pw") == "":
            sPrompt = "Password (<not saved>): "
        else:
            sPrompt = "Password (<saved>): "
        m.subsystem_getDb().account_single_setPart("pw", getpass.getpass(prompt=sPrompt))
        m.conf_cfgSave()
        exit(1)

    # Teste ob Account leer ist
    if m.subsystem_getDb().account_single_isEmpty():
        print("Configuration of account is empty -> Please configure first")
        exit(1)

    # Initialisiere Workflows (Serververbindung und Passwort)
    if m.subsystem_getDb().account_single_getPart("pw") == "":
        m.subsystem_getDb().account_single_setPart("pw", getpass.getpass(prompt="Password: ")) # Pw holen
    # Programmkonfiguration zeigen
    if args.i:
        if args.i == ["all"]:
            lst = m.subsystem_getSubLst()
        else:
            lst = args.i
        for sSub in lst:
            sInfo = m.subsystem_getSubInfo(sSub)
            if sInfo is not None:
                print(sSub.upper())
                print(sInfo)
                print()
        exit(1)
    # Teste auf Update
    if args.update:
        print("* Checking for available updates")
        res = m.help_updateCheck()
        if res.isErr():
            print("Error:", res.getResult())
        else:
            bUpdate = res.getData("update")
            print(res.getData("update-info"))
            if bUpdate:
                print("Update Level:", res.getData("update-level"))
                ein = input("Do you want to update [yN]? ")
                if ein == "" or ein.lower()[0] not in ('y', 'j'):
                    print("abort ...")
                    exit(0)
                print("* Starting update")
                res = m.help_updateDownload()
                if res.isErr():
                    print("Error:", res.getResult())
                    exit(1)
                print("* Installation")
                sSys = res.getData("sys")
                sSrc = res.getData("src")
                sTarget = res.getData("target")
                print("System: " + sSys)
                print("Source: " + sSrc)
                print("Target: " + sTarget)
                ein = input("Do you want to install the update [yN]? ")
                if ein == "" or ein.lower()[0] not in ('y', 'j'):
                    print("Installation aborted ...")
                    exit(0)
                # Installation ---
                print("Copying files:")
                files = copy_tree(sSrc, sTarget)
                print(PrettyText.lst2strR(files))
                print("Installation is finished")
    elif args.tax_update:
        # Update Taxon list
        res = m.subsystem_getTaxonWizard().update()
        if res.isErr():
            print("\n* Taxon system update failed - nothing is wrtten to database")
            print(res.getData("text"))
            exit(1)
        else:
            print("\n* Taxon system update sucessfully finished")
            print(res.getData("text"))
            exit(0)

    # --- Serververbindung noetig ---
    resDir = m.subsystem_getConnector().reconnect()
    if resDir.isErr():
        print(resDir.getErrWarnInfoString())
        exit(0)

    if args.imp_ing:
        # Import: Ingest
        sWf = args.imp_ing[0]
        sFile = args.imp_ing[1]
        bLupSingle = args.imp_ing_lup_single
        sExcel = args.imp_lup_in
        bGoldrush = args.imp_lup_goldrush
        sMode = args.imp_lup_mode
        sUpdate = args.imp_lup_update
        sExtend = args.imp_ing_ext
        sExtendReact = args.imp_ing_ext_react
        sReuse = args.imp_lup_reuse_react
        if sExtend == "yes":
            bExtend = True
        elif sExtend == "no":
            bExtend = False
            sExtendReact = None
        else:
            bExtend = None
            sExtendReact = None
        bGoldrush = None
        res = m.subsystem_getImporter().startImport(sFile, ingest=True, lookup=True, ingest_wf=sWf, lookup_import_excel=sExcel, lookup_import_mode=sMode, lookup_import_db=sUpdate, lookup_import_goldrush=bGoldrush, ingest_lup_search_single=bLupSingle, tamer=True, extend=bExtend, extend_react=sExtendReact, lookup_reuse=sReuse)
        print(res.getData("log"))
        if res.isAnyMessage():
            print(res.getErrWarnInfoString())
        exit(1)
    elif args.imp_lup is not None:
        # Import: Lookup
        sFile = args.imp_lup.pop(0)
        lstTabs = args.imp_lup
        sExcel = args.imp_lup_in
        bGoldrush = args.imp_lup_goldrush
        sMode = args.imp_lup_mode
        sUpdate = args.imp_lup_update
        sReuse = args.imp_lup_reuse_react
        res = m.subsystem_getImporter().startImport(sFile, lookup=True, lookup_sheets=lstTabs, lookup_import_excel=sExcel, lookup_import_mode=sMode, lookup_import_db=sUpdate, lookup_import_goldrush=bGoldrush, tamer=True, lookup_reuse=sReuse)
        print(res.getData("log"))
        if res.isAnyMessage():
            print(res.getErrWarnInfoString())
            exit(0)
    elif args.imp_tax is not None:
        # Import: Taxon System
        sFile = args.imp_tax.pop(0)
        lstTabs = args.imp_tax
        bUpdate = args.imp_tax_update
        res = m.subsystem_getImporter().startImport(sFile, taxon=True, taxon_sheets=lstTabs, taxon_update=bUpdate, tamer=True)
        print()
        if res.isErr():
            print(res.getResult())
            exit(0)
        print("Occurred errors:", res.getData("err"))
        print("Log:")
        print(res.getData("log"))

    elif args.exp_tax:
        # Export: Taxon System
        sExportFile = args.exp_tax_file
        lstSheets = args.exp_tax_sheets
        res = m.subsystem_getExporter().subsystem_getTaxonWizard(sheets=lstSheets)
        if res.isErr():
            print("Error:", res.getResult())
            exit(1)
        odicTax = res.getData("taxon")
        if sExportFile is None:
            # Console
            if m.conf_isDebug():
                print("* Log")
                print(res.getData("log"))
                print("* Tables")
            for sSheet, dicSheet in odicTax.items():
                lstHeaders = dicSheet["header"]
                lstTab = dicSheet["content"]
                print("--- "+sSheet)
                print(PrettyText.table([lstHeaders] + lstTab))
        else:
            # Excel
            print()
            print("Export to file: "+sExportFile)
            res = m.subsystem_getExporter().writeExcel(taxon=True, taxon_sheets=lstSheets, taxon_data=odicTax, file=sExportFile)
            print()
            if res.isErr():
                print("Error:", res.getResult())
                exit(1)
    elif args.exp_lup_ing or args.exp_lup_all:
        # Export: Lookup
        sExportFile = args.exp_lup_file
        bShowId = args.exp_lup_id
        lstIng = args.exp_lup_ing
        bLupAll = args.exp_lup_all
        lstSheets = args.exp_lup_sheets
        sGold = args.exp_lup_gold
        if sGold not in [None, "all","gold","not-gold"]:
            print("Error: Unknown gold status: ", sGold)
            exit(1)
        dicSelect = {"ingest": lstIng}
        if bLupAll:
            dicSelect = None
        res = m.subsystem_getExporter().getLookup(selector=dicSelect, sheets=lstSheets, showid=bShowId, gold=sGold)
        if res.isErr():
            print("Error:", res.getResult())
            exit(1)
        odicLup = res.getData("lookup")
        if sExportFile is None:
            # Console
            if m.conf_isDebug():
                print("* Log")
                print(res.getData("log"))
                print("* SQL")
                for sSheet, dicSheet in odicLup.items():
                    sSql = dicSheet["sql"]
                    print("--- " + sSheet)
                    print(sSql)
                print("* Tables")
            for sSheet, dicSheet in odicLup.items():
                lstHeaders = dicSheet["header"]
                lstTab = dicSheet["content"]
                print("--- "+sSheet)
                print(PrettyText.table([lstHeaders] + lstTab))
        else:
            # Excel
            print()
            print("Export to file: "+sExportFile)
            res = m.subsystem_getExporter().writeExcel(lookup_sheets=lstSheets, showid=bShowId, rfriendly=b, lookup_data=odicLup, lookup=True, file=sExportFile)
            if res.isErr():
                print("Error:", res.getResult())
                exit(1)
    elif args.exp_sub_selector:
        # Export: Subset - Zeige Selector Ranges
        sel = m.ui_selector_getSelector()
        sel.reset()
        print("* Selector ranges")
        print(sel.infoConfigShort2())
        exit(0)
    elif any((args.exp_sub_filter_date_start, args.exp_sub_filter_date_end, args.exp_sub_filter_ing, args.exp_sub_filter_gear, args.exp_sub_filter_cov, args.exp_sub_filter_dat, args.exp_sub_filter_region, args.exp_sub_filter_wf)) :
        # Export: Subset

        # Daten
        bLookup = True
        odicLup = None

        # Parameter
        lstExportSheets = args.exp_sub_sheets
        sExportFile = args.exp_sub_file
        bExportId = args.exp_sub_id
        bExportIngestId = args.exp_sub_ing_id
        bRfriendly = args.exp_sub_rfriendly
        bInfoOnly = args.exp_sub_stat
        sDateStart = args.exp_sub_filter_date_start
        sDateEnd = args.exp_sub_filter_date_end
        lstIng = args.exp_sub_filter_ing
        lstGea = args.exp_sub_filter_gear
        lstCov = args.exp_sub_filter_cov
        lstDat = args.exp_sub_filter_dat
        lstReg = args.exp_sub_filter_region
        lstWf = args.exp_sub_filter_wf
        sConverter = args.exp_sub_converter # Converter
        # Auswerten
        dicSelect = dict()
        if lstIng is not None:
            dicSelect["ingest"] = lstIng
        if sDateStart is not None:
            dicSelect["sample_date_start"] = sDateStart
        if sDateEnd is not None:
            dicSelect["sample_date_end"] = sDateEnd
        if lstGea is not None:
            dicSelect["gear"] = lstGea
        if lstCov is not None:
            dicSelect["cover"] = lstCov
        if lstDat is not None:
            dicSelect["dataset"] = lstDat
        if lstReg is not None:
            dicSelect["region"] = lstReg
        if lstWf is not None:
            dicSelect["workflow"] = lstWf

        # Kein Export von lookups wenn der PANGAEA Exporter benutzt wird
        if sConverter == 'pangaea':
            bLookup = False

        # Ingest
        res = m.subsystem_getExporter().getSubset(selector=dicSelect, showid=bExportId, showingestid=bExportIngestId, sheets=lstExportSheets, converter=sConverter, rfriendly=bRfriendly)
        if res.isErr():
            print("Error:", res.getErrWarnInfoString(short=True))
            exit(1)
        msIng = res.getData("multisheet")
        sLogIng = res.getData("log")
        sWf = res.getData("wf")

        # Lookup
        if bLookup:
            res = m.subsystem_getExporter().getLookup(selector=dicSelect)
            if res.isErr():
                print("Error:", res.getErrWarnInfoString(short=True))
                exit(1)
            odicLup = res.getData("lookup")
            sLogLookup = res.getData("log")

        if sExportFile is None:
            print("")
            # --- Console Output
            if m.conf_isVerbose():
                print("* Console output")
            # Ingest
            print(">>> Ingest")
            sLogIngest = res.getData("log")
            if bInfoOnly:
                print(msIng.getInfo(full=False))
            else:
                print(msIng.getInfo(full=True))
            print(">>> Lookup")
            if m.conf_isVerbose():
                print("* Tables")
            for sSheet, dicSheet in odicLup.items():
                lstHeaders = dicSheet["header"]
                lstTab = dicSheet["content"]
                iRows = dicSheet["rows"]
                if bInfoOnly:
                    print(sSheet + ":", iRows)
                else:
                    print("--- " + sSheet)
                    print(PrettyText.table([lstHeaders] + lstTab))
            # Log
            sLog = "*** LOG\n>>> Ingest\n" + sLogIngest
            if bLookup:
                sLog += "\n>>> Lookup\n" + sLogLookup
            if m.conf_isVerbose():
                print(sLog)
        else:
            # Excel
            print("Export to file: " + sExportFile)
            res = m.subsystem_getExporter().writeExcel(ingest=True, ingest_data=msIng, ingest_wf=sWf, rfriendly=bRfriendly, ingest_sheets=lstExportSheets, lookup_data=odicLup, lookup=bLookup, converter=sConverter, file=sExportFile)
            if res.isErr():
                print("Error:", res.getErrWarnInfoString(short=True))
                exit(1)
    elif args.exp_template:
        # Export: Template
        sWf = args.exp_template
        sFile = None
        if m.subsystem_getMapper().workflow_converter_import_isExisting(sWf):
            # Converter
            sWf_long = m.subsystem_getMapper().workflow_converter_import_short2long(sWf)
            sFile = PrettyText.clean_2filename(sWf_long) + "_template.xlsx" if sWf_long is not None else None
            res = m.subsystem_getConverter().write_converter_import_template(sWf, sFile)
            if res.isErr():
                print("Error:", res.getErrWarnInfoString(short=True))
                exit(1)
            print("Converter template written:", sFile)
        elif m.subsystem_getMapper().workflow_isExisting(sWf):
            # Normal
            sWf_long = m.subsystem_getMapper().workflow_short2long(sWf)
            sFile = PrettyText.clean_2filename(sWf_long) + "_template.xlsx" if sWf_long is not None else None
            res = m.subsystem_getExporter().writeExcel(ingest_wf=sWf, file=sFile, ingest=True, lookup=True)
            if res.isErr():
                print("Error:", res.getErrWarnInfoString(short=True))
                exit(1)
            print("Template written:", sFile)
        else:
            print("Unknown workflow:", sWf)
    elif args.bot:
        # BatchBot
        sConf = args.bot[0]
        bat = BatchBot(m)
        res = bat.startBatch(sConf)
        if res.isErr():
            print("Error:", res.getErrWarnInfoString(short=True))
            exit(1)
        else:
            pass
            print(res.getData("log"))
    elif args.fao_new:
        # FAO: New
        lstLevel = args.fao_new
        level = None
        if len(lstLevel) ==1 and lstLevel[0] == "std":
            level = ["major", "division", "subarea"]
            sInfoLevel = PrettyText.lst2strC(level)
        elif len(lstLevel) ==1 and lstLevel[0] == "all":
            sInfoLevel = "<all available>"
            level = "*"
        else:
            level = lstLevel
            sInfoLevel = PrettyText.lst2strC(level)
        ein = input("New FAO regions: "+sInfoLevel+" [yN]? ")
        if ein == "" or ein.lower()[0] not in ('y', 'j'):
            print("Abort ...")
            exit(1)
        res = m.subsystem_getDb().fao_regions_create(level=level)
        if res.isErr():
            print("Error:", res.getErrWarnInfoString(short=True))
            exit(1)
        else:
            print(res.getData("log"))
    elif args.fao_del:
        # FAO: Del mapping
        iIngId = args.fao_del
        if iIngId == -1:
            iIngId = None
            ein = input("Del whole FAO mapping [yN]? ")
        else:
            ein = input("Del FAO mapping for ingest "+str(iIngId) + " [yN]? ")
        if ein == "" or ein.lower()[0] not in ('y', 'j'):
            print("Abort ...")
            exit(1)
        res = m.subsystem_getDb().fao_map_clean(ing=iIngId)
        if res.isErr():
            print("Error:", res.getErrWarnInfoString(short=True))
            exit(1)
        else:
            iDel = res.getData("del")
            if iDel is not None:
                print("Removed samples from mapping: "+ str(iDel))
    elif args.fao_stat:
        bShort = True if args.fao_stat == "short" else False
        # FAO: Info
        res = m.subsystem_getDb().fao_map_getData(short=bShort)
        if res.isErr():
            print("Error:", res.getErrWarnInfoString(short=True))
            exit(1)
        else:
            print(res.getData("log"))
    elif args.fao_update:
        # FAO: Update
        res = m.subsystem_getDb().fao_map_update()
        if res.isErr():
            print("Error:", res.getErrWarnInfoString(short=True))
            exit(1)
        else:
            print(res.getData("log"))
    elif args.test:
        # Generic test
        print("*** Test area")
        resPANGAEA = m.subsystem_getMyPangaea().puffer_load()
        #resPANGAEA = m.subsystem_getMyPangaea().puffer_update()
        resPANGAEA = m.subsystem_getMyPangaea().puffer_createStats()
        resPANGAEA = m.subsystem_getMyPangaea().puffer_getStatusString()

        print(resPANGAEA)
        quit()
#         map = m.subsystem_getMapper()
#         sSchema = "critterbase"
#         sWsTable = "ws_inf_data"
#         sSql = \
# f"""
# DROP TABLE IF EXISTS {sSchema}.{sWsTable};
# CREATE TABLE {sSchema}.{sWsTable} (
# "table" text NOT NULL,
# "attr" text NOT NULL,
# "sheet" text NOT NULL,
# "col" text NOT NULL,
# "pk" boolean NOT NULL,
# "fk" boolean NOT NULL,
# "lup" boolean NOT NULL,
# "obl" boolean NOT NULL,
# "name" text,
# "cmt" text,
# "unit" text
# );"""
#         for sTable in map.ingest_getDbTables():
#             for sAttrib in map.ingest_getDbAttributes(sTable):
#                 sSheet, sCol = map.ingest_db2excel(sTable, sAttrib)
#                 bObl = map.ingest_isObl(sSheet, sCol)
#                 bPk = map.ingest_isPk(sSheet, sCol)
#                 bFk = map.ingest_isFk(sSheet, sCol)
#                 bLup = map.ingest_isLookup(sSheet, sCol)
#                 resComment = m.conf_getDbSchema().getDbSchemaInfo("ingest", sSheet, sCol)
#                 if resComment.isOk():
#                     sComName = resComment.getData("comment-part-name")
#                     sComUnit = resComment.getData("comment-part-unit")
#                     sComComment = resComment.getData("comment-part-comment")
#                 else:
#                     sComName = sComUnit = sComComment = None
#                 sSql += \
# f"""
# INSERT INTO {sSchema}.{sWsTable} ("table", "attr", "sheet", "col", "pk", "fk", "lup", "obl", "name", "cmt", "unit")
# VALUES ('{sTable}', '{sAttrib}', '{sSheet}', '{sCol}', {bPk}, {bFk}, {bLup}, {bObl}, '{sComName}', '{sComComment}', '{sComUnit}');"""
#         print(sSql)
#         quit()

        # sDir = "/Users/pkloss/_temp/REEF_ANNO/annotations"
        # wb = openpyxl.Workbook()
        # ws = wb.active
        # ws.title = "reef anno"
        # ws.append(["file name", "part1", "part2", "part3", "part4", "part5", "ps x", "ps y", "taxon"])
        # for filename in sorted(os.listdir(sDir)):
        #     with open(os.path.join(sDir, filename), 'r') as f:
        #         csvReader = csv.reader(f, delimiter='\t', quotechar='"')
        #         for lstRow in csvReader:
        #             sFileName = filename
        #             sFileName2 = sFileName.replace("grid.txt","")
        #             # sFileName2 = sFileName2.replace(" ", "_")
        #             lstFileName2 = sFileName2.split("_")
        #             sPsX = lstRow[0]
        #             sPsY = lstRow[1]
        #             sTaxon = lstRow[2]
        #             sA = lstFileName2[0]
        #             sB = lstFileName2[1]
        #             sC = lstFileName2[2]
        #             sD = lstFileName2[3]
        #             try:
        #                 sE = lstFileName2[4]
        #             except:
        #                 sE = ""
        #             lstExcelRow = [sFileName, sA, sB, sC, sD, sE, sPsX, sPsY, sTaxon]
        #             ws.append(lstExcelRow)
        # wb.save(filename="/Users/pkloss/_temp/REEF_ANNO/reef_anno2.xlsx")
        # quit()
        # map = folium.Map(control_scale=True, tiles=None)
        # style = lambda sCol: lambda feature: dict(weight=1)  # color=sCol, fillColor=sCol
        # lstRegions = m.subsystem_getRegions().regions_getAll()
        # iParts = len(lstRegions)
        # colGradient = ColorGradient(Colors.getColor("yellow"), Colors.getColor("purple"), iParts)
        #
        # """
        # for iNr, sRegion in enumerate(lstRegions):
        #     sCol = colGradient.getHtml(iNr)
        #     shape, iSize = m.subsystem_getRegions().regions_getShape(sRegion)
        #     if sRegion != "non-FAO":
        #         print(sRegion)
        #         folium.GeoJson(shape, name="Region: " + sRegion, tooltip=sRegion, style_function=style(sCol)).add_to(map)
        # """
        # sConString = "connect_timeout=4 dbname='critterbase' user='paul' host='localhost' port='5432' password='polly'"
        # sqlCmd_fao = "select ST_AsGeoJSON(geometry) from release.region where id=100;"
        # sqlCmd_ices_cb = "select ST_AsGeoJSON(geometry) from test.region where id=1;"
        # sqlCmd_ices_direct = 'select ST_AsGeoJSON(geom) from "ices" where gid=1;'
        # conn = psycopg2.connect(sConString)
        # cursor = conn.cursor()
        # cursor.execute(sqlCmd_fao)
        # lstContent = cursor.fetchall()
        # shape_fao = lstContent[0][0]
        # cursor.execute(sqlCmd_ices_direct)
        # lstContent = cursor.fetchall()
        # shape_ices = lstContent[0][0]
        # #print(shape_fao)
        # print(shape_ices)
        # folium.GeoJson(shape_ices).add_to(map)
        #
        # map.save("/Users/pkloss/_temp/map.html")
        # quit()
        # from decimal import *
        # ms = MultiSheet(name="Test Workbench")
        # odic = OrderedDict([('Cruise', {'header': ['Cruise', 'RefSystem', 'DateStart', 'DateEnd', 'ShipName', 'SciCruiseLeader', 'Region', 'PositionSystem', 'Remark'], 'content': [('ST0667', 'WGS84', datetime.date(1967, 6, 5), datetime.date(1967, 6, 5), 'FK Victor Hensen', 'kstripp', 'German Bight', None, 'this Cruisecode incorporates all Data from Stripp, taken in 1966 & 1967')]}), ('Station', {'header': ['Cruise', 'Station', 'ProjectLeader', 'DateStart', 'DateEnd', 'TimeStart', 'TimeEnd', 'LatTarget', 'LonTarget', 'LatStart', 'LonStart', 'LatEnd', 'LonEnd', 'Precision', 'DepthStart', 'DepthEnd', 'Area', 'NumberReplicates', 'TempAir', 'TempBot', 'TempSurf', 'SalinitySurf', 'SalinityBot', 'WaterOxyBot', 'WindSpeed', 'WindDirection', 'Remark'], 'content': [('ST0667', 'V', None, datetime.date(1967, 6, 5), None, None, None, 54.375, 7.61666666666667, 54.375, 7.61666666666667, 54.375, None, 3, 26.0, None, 'Fsd', 1, None, None, None, None, None, None, None, None, None), ('ST0667', '0071', None, datetime.date(1966, 7, 28), None, None, None, 54.05, 8.08333333333333, 54.05, 8.08333333333333, 54.05, None, 3, 26.0, None, 'H1', 1, None, None, None, None, None, None, None, None, None), ('ST0667', '0090', None, datetime.date(1966, 10, 6), None, None, None, 54.0333333333333, 7.83333333333333, 54.0333333333333, 7.83333333333333, 54.0333333333333, None, 3, 42.0, None, 'P12_area', 1, None, None, None, None, None, None, None, None, 'nahe P12 aber tief (und Oktober!)')]}), ('Sample', {'header': ['Cruise', 'Station', 'Sample', 'Gear', 'LatStart', 'LonStart', 'LatEnd', 'LonEnd', 'Precision', 'DateStart', 'DateStartError', 'DateEnd', 'TimeStart', 'TimeEnd', 'DepthStart', 'DepthEnd', 'SampledArea', 'SampledVolume', 'ReferenceArea', 'ReferenceVolume', 'Replicates', 'SampleSize', 'SubSampleShare', 'SubSampleMass', 'SamplingDistance', 'SamplingSpeed', 'DetermBiomass', 'DetermDry', 'DetermAFDW', 'Conservation', 'TempSurfStart', 'TempSurfEnd', 'SalSurfStart', 'SalSurfEnd', 'Remark', 'RespScientist', 'SedimentProperties', 'SedimentSmell', 'GrabPenetrationDepth'], 'content': [('ST0667', 'V', '1', 'vV0.1', 54.375, 7.61666666666667, None, None, 3, datetime.date(1967, 6, 5), 0, None, None, None, 26.0, None, 0.1, None, None, None, 1, 0.0, 2.0, None, None, None, None, None, None, '0', None, None, None, None, None, 'kstripp', 'Sediment k.A.  Fsd', None, None), ('ST0667', '0071', '1', 'vV0.1', 54.05, 8.08333333333333, None, None, 3, datetime.date(1966, 7, 28), 0, None, None, None, 26.0, None, 0.1, None, None, None, 1, 0.0, 2.0, None, None, None, None, None, None, '0', None, None, None, None, None, 'kstripp', '"zS"= zäher Schlick', None, None), ('ST0667', '0090', '1', 'vV0.1', 54.0333333333333, 7.83333333333333, None, None, 3, datetime.date(1966, 10, 6), 0, None, None, None, 42.0, None, 0.1, None, None, None, 1, 0.0, 2.0, None, None, None, None, None, None, '0', None, None, None, None, None, 'kstripp', '"sSd"= schlickiger Sand', None, None)]}), ('SubSample', {'header': ['Cruise', 'Station', 'Sample', 'SubSample', 'Ratio'], 'content': [('ST0667', 'V', '1', '1', None), ('ST0667', '0071', '1', '1', None), ('ST0667', '0090', '1', '1', None)]}), ('Subset', {'header': ['Cruise', 'Station', 'Sample', 'SubSample', 'Subset', 'Project', 'TaxonomicCoverage', 'TaxonomicResolution', 'SpatialLevel'], 'content': [('ST0667', 'V', '1', '1', '1', 'Benthos monitoring German Bight 1967', 'Macrozoobenthos', 'species', 0), ('ST0667', '0071', '1', '1', '1', 'Benthos monitoring German Bight 1967', 'Macrozoobenthos', 'species', 0), ('ST0667', '0090', '1', '1', '1', 'Benthos monitoring German Bight 1967', 'Macrozoobenthos', 'species', 0)]}), ('Biota', {'header': ['Cruise', 'Station', 'Sample', 'SubSample', 'Subset', 'Biota', 'AphiaId', 'ScientificName', 'Number', 'Abundance', 'PresenceAbsence', 'SumWet', 'SumDry', 'SumAFDW', 'Stage', 'SieveFraction', 'SubTotalFactor', 'Specification', 'Image', 'Remark'], 'content': [('ST0667', 'V', '1', '1', '1', '66', 124913, 'Ophiura albida', Decimal('8'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '65', 130494, 'Ophelia limacina', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '64', 130359, 'Nephtys hombergii', Decimal('3'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '63', 130355, 'Nephtys caeca', Decimal('2'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '62', 140539, 'Euspira pulchella', Decimal('3'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '61', 100730, 'Edwardsia', Decimal('5'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '60', 124392, 'Echinocardium cordatum', Decimal('2'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '59', 117452, 'Corymorpha nutans', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '58', 181373, 'Cochlodesma praetenue', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '57', 141907, 'Chamelea gallina', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '56', 123776, 'Asterias rubens', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '55', 138802, 'Arctica islandica', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '54', 129840, 'Aphrodita aculeata', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '53', 1135, 'Amphipoda', Decimal('5'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '52', 141436, 'Abra prismatica', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '51', 141435, 'Abra nitida', Decimal('2'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '71', 141587, 'Tellina fabula', Decimal('6'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '70', 140302, 'Spisula subtruncata', Decimal('2'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '69', 130537, 'Scoloplos armiger', Decimal('2'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '68', 140737, 'Phaxas pellucidus', Decimal('4'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', 'V', '1', '1', '1', '67', 130544, 'Owenia fusiformis', Decimal('14'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0071', '1', '1', '1', '1', 141433, 'Abra alba', Decimal('2'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0071', '1', '1', '1', '10', 130359, 'Nephtys hombergii', Decimal('15'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0071', '1', '1', '1', '11', 129898, 'Notomastus latericeus', Decimal('4'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0071', '1', '1', '1', '12', 140589, 'Nucula nitidosa', Decimal('73'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0071', '1', '1', '1', '13', 124913, 'Ophiura albida', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0071', '1', '1', '1', '14', 234099, 'Pectinaria koreni', Decimal('2'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0071', '1', '1', '1', '15', 334506, 'Phyllodoce groenlandica', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0071', '1', '1', '1', '16', 130980, 'Scalibregma inflatum', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0071', '1', '1', '1', '2', 141435, 'Abra nitida', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0071', '1', '1', '1', '3', 283798, 'Cerianthus lloydii', Decimal('2'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0071', '1', '1', '1', '4', 139410, 'Corbula gibba', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0071', '1', '1', '1', '5', 110487, 'Diastylis rathkei', Decimal('100'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0071', '1', '1', '1', '6', 110377, 'Echiurus echiurus', Decimal('2'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0071', '1', '1', '1', '7', 152313, 'Gattyana cirrosa', Decimal('2'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0071', '1', '1', '1', '8', 130140, 'Goniada maculata', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0071', '1', '1', '1', '9', 140380, 'Mysella bidentata', Decimal('3'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '38', 129898, 'Notomastus latericeus', Decimal('3'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '39', 140589, 'Nucula nitidosa', Decimal('48'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '40', 139336, 'Oenopota turricula', Decimal('2'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '41', 130500, 'Ophelina acuminata', Decimal('5'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '42', 124913, 'Ophiura albida', Decimal('47'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '43', 130544, 'Owenia fusiformis', Decimal('85'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '44', 234099, 'Pectinaria koreni', Decimal('12'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '45', 140737, 'Phaxas pellucidus', Decimal('10'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '46', 130599, 'Pholoe baltica', Decimal('5'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '47', 128545, 'Phoronis', Decimal('35'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '48', 334506, 'Phyllodoce groenlandica', Decimal('4'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '49', 130980, 'Scalibregma inflatum', Decimal('5'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '50', 152394, 'Sphaerodorum flavum', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '37', 130359, 'Nephtys hombergii', Decimal('15'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '36', 130355, 'Nephtys caeca', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '35', 246812, 'Nemertini', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '34', 140380, 'Mysella bidentata', Decimal('21'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '33', 140431, 'Mya truncata', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, 'genaue Anzahl fraglich, Sum(Seltene Arten)=8'), ('ST0667', '0090', '1', '1', '1', '32', 140371, 'Montacuta ferruginosa', Decimal('2'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '31', 107388, 'Liocarcinus holsatus', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, 'genaue Anzahl fraglich, Sum(Seltene Arten)=8'), ('ST0667', '0090', '1', '1', '1', '30', 131495, 'Lanice conchilega', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '29', 130140, 'Goniada maculata', Decimal('19'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '28', 140539, 'Euspira pulchella', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '27', 130738, 'Enipo kinbergi', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, 'genaue Anzahl fraglich, Sum(Seltene Arten)=8'), ('ST0667', '0090', '1', '1', '1', '26', 100730, 'Edwardsia', Decimal('4'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '25', 124392, 'Echinocardium cordatum', Decimal('4'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '24', 110487, 'Diastylis rathkei', Decimal('9'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '23', 139476, 'Cylichna cylindracea', Decimal('1'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '22', 139410, 'Corbula gibba', Decimal('3'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '21', 125080, 'Amphiura filiformis', Decimal('5'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '20', 129778, 'Ampharete finmarchica', Decimal('2'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '19', 129775, 'Ampharete acutifrons', Decimal('14'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '18', 101891, 'Ampelisca brevicornis', Decimal('27'), None, None, None, None, None, None, 1000, None, None, None, None), ('ST0667', '0090', '1', '1', '1', '17', 141435, 'Abra nitida', Decimal('47'), None, None, None, None, None, None, 1000, None, None, None, None)]}), ('Sediment', {'header': ['Cruise', 'Station', 'Sample', 'SubSample', 'Subset', 'Type', 'GrainSize1', 'MeshSize1', 'GrainSize2', 'MeshSize2', 'GrainSize3', 'MeshSize3', 'GrainSize4', 'MeshSize4', 'GrainSize5', 'MeshSize5', 'GrainSize6', 'MeshSize6', 'GrainSize7', 'MeshSize7', 'GrainSize8', 'MeshSize8', 'GrainSize9', 'MeshSize9', 'Remark', 'SampleWeight', 'Median', 'GSD', 'GrainSizeMethod', 'SedTemp', 'COrg', 'CTotal', 'LossOnIgnition', 'Nitrogen', 'Sulfur', 'OxyLayer', 'Silt', 'SandVF', 'SandF', 'SandM', 'SandC', 'SandVC', 'Gravel', 'Stones', 'Schill', 'Klei'], 'content': [('ST0667', 'V', '1', '1', '1', None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, 'Sediment k.A.  Fsd', None, None, None, None, None, None, None, None, None, None, None, None, None, 3, None, None, None, None, None, None, None), ('ST0667', '0071', '1', '1', '1', None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, 'zS= zäher Schlick', None, None, None, None, None, None, None, None, None, None, None, 3, None, None, None, None, None, None, None, None, None), ('ST0667', '0090', '1', '1', '1', None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, 'sSd= schlickiger Sand', None, None, None, None, None, None, None, None, None, None, None, 2, 2, 3, None, None, None, None, None, None, None)]}), ('IndividualMeasure', {'header': ['Cruise', 'Station', 'Sample', 'SubSample', 'Subset', 'Biota', 'Sex', 'LengthTotal', 'LengthSt', 'WeightTotal', 'WeightSh', 'Remark'], 'content': []})])
        # for sSheet, dic in odic.items():
        #     lstHeader = dic.get("header")
        #     lstContent = dic.get("content")
        #     ms.addSheet(sSheet, lstHeader)
        #     for lstRow in lstContent:
        #         lstRow = ["Huhu"]
        #         lstRow = ["Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu","Huhu"]
        #         ms.addRow(sSheet, {"Cruise": "Paul"})
        # ms.setIndex("Cruise", ["Cruise"])
        # ms.setIndex("Station", ["Cruise", "Station"])
        # ms.setIndex("Sample", ["Cruise", "Station", "Sample"])
        # ms.setIndex("SubSample", ["Cruise", "Station", "SubSample"])
        # ms.setIndex("Subset", ["Cruise", "Station", "Sample", "SubSample", "Subset"])
        # ms.setIndex("Biota", ["Cruise", "Station", "Sample", "SubSample", "Subset", "Biota"])
        # ms.setIndex("Sediment", ["Cruise", "Station", "Sample", "SubSample", "Subset"])
        # ms.setIndex("IndividualMeasure", ["Cruise", "Station", "Sample", "SubSample", "Subset", "Biota"])
        # ms.finalize()
        # # dic = {"Cruise": "Paul", "RefSystem": "WGS"}
        # # dic = {"Cruise": "Paul", "RefSystem": "WGS"}
        # # bool = ms.addRow("Cruise", dic)
        # # print()
        # # print(ms.getRow("Cruise", row=0, mode="list"))
        # # print(ms.getValue("Cruise", "RefSystem", row=0))
        # print(ms)
        # # print(ms.getValue("Biota", "ScientificName", index=["ST0667", "V", "1", "1", "1", "67"]))
        # #ms.setValue("Biota", "ScientificName", "Knut", index=["ST0667", "V", "1", "1", "1", "67"])
        # #resVal = ms.getValue("Biota", "ScientificName", row=100000)
        # resVal = ms.getValue("Biota", "ScientificName", index=["ST0667", "V", "1", "1", "1", "67x"])
        # val = resVal.getData("value")
        # if resVal.isErr():
        #     print(resVal)
        # else:
        #     print(val)
        # print(ms.getValue("Biota", "ScientificName", row=0))
        # ms.setValue("Biota", "ScientificName", "Yürgen", row=0)
        # ms.setValue("Biota", "Number", 10, row=0)
        #print(ms.getInfo(full=True))

        # ms.setOpenSheetName("Biota")
        # for lstRow in ms():
        #      print(lstRow)
        #quit()
        # sDir = "/Users/paul/git/awi/arctic_ckraan/"
        # sDir = "/Users/paul/git/awi/antarctic_kteschke/matrix/"
        # lstFiles = [f for f in os.listdir(sDir) if os.path.isfile(os.path.join(sDir, f))]
        # iNr = 1
        # iRows_total = 0
        # wb2 = openpyxl.Workbook()
        # s2 = wb2.active
        # for sFile in lstFiles:
        #     if sFile.endswith(".xlsx") and sFile not in ["nahidik.xlsx", "sejr.xlsx"]:
        #         wb = openpyxl.load_workbook(sDir+sFile, read_only=True)
        #         sheet = wb["Subset"]
        #         iRows = sheet.max_row-1
        #         print(iNr, sFile, iRows)
        #         s2.append([iNr, sFile, iRows])
        #         iRows_total += iRows
        #         iNr += 1
        # print("Gesamt: ", iRows_total)
        # s2.append(["", "Gesamt", iRows_total])
        # wb2.save('/Users/paul/_temp/sum.xlsx')
        # quit()
        # # sUrl = 'https://critterbase.awi.de/update/version_app.txt'
        # # r = requests.get(sUrl, allow_redirects=True, timeout=3, auth=('critter', 'P0lych4eta'))
        # # if r.status_code == 200 and ('text/plain' in r.headers['Content-Type']):
        # #     print(r.text)
        # # print(r)
        # # #print(pd["Sample"].describe())
        # map = m.subsystem_getMapper()
        # # Hier fehlt
        # bBool = map.ingest_getFk_HMMMM("SubSample", "Station")
        # print(bBool)
        # quit()

        # dicSelect = {"ingest": [160]} # 12
        # bRfriendly = False
        # res = m.subsystem_getExporter().getSubset(selector=dicSelect, converter="mar_trans")
        # #res = m.subsystem_getExporter().getSubset(selector=dicSelect, rfriendly=bRfriendly)
        # if res.isErr():
        #     print("Error:", res.getErrWarnInfoString(short=True))
        #     exit(1)
        # ms = res.getData("multisheet")
        # sLog = res.getData("log")
        # sWf = res.getData("wf")
        # print(ms.getInfo(full=True))
        # sExportFile = "/Users/paul/_temp/subset.xlsx"
        # res = m.subsystem_getExporter().writeExcel(ingest=True, ingest_data=ms, converter="mar_trans", ingest_wf=sWf, lookup=True, glossary=False, rfriendly=bRfriendly, file=sExportFile)
        # if res.isErr():
        #     print("Error:", res.getErrWarnInfoString(short=True))
        #     exit(1)
        # #print(sLog)
        # quit()
        # lstData = [
        #     ["Paul",10],
        #     ["Otto", 4],
        #     ["Otto", 4],
        #     ["Uschi", 17],
        #     ["Gnulf", 8],
        #     ["Sabine", 3]
        # ]
        # sBarChart = PrettyText.barchart(lstData, header=False, maxbarsize=20)
        # print(sBarChart)
        # quit()
        # p = Package("Pauls Daten")
        # p.add(["lookup","log"], "Pups1")
        # p.add(["lookup","data"], "Pups2")
        # print(p)
        # print(p.get(["lookup","log"]))
        # quit()
        # dir = m.subsystem_getConnector()
        # print("------------------")
        # dir.status_reload(system="ingest")
        # quit()
        # print(dir)
        #print(dir.status_isOk("lookup"))
        #print(dir.status_getLog("lookup"))
        #print(dir.status_getData("lookup"))

        # reg = m.subsystem_getRegions()
        # db = m.subsystem_getDb()
        # #print(reg.map_getRegions(leftover=False))
        # print(reg.info_map(short=True))
        # # res = reg.reload()
        # # print(reg.infoRegions_all())
        # # print(reg.infoMap())
        #
        # #print(reg.getRegions_all_nr(leftover=False), reg.getRegions_all(leftover=False))
        #
        # # for sRegion in reg.getRegions():
        # #     print(sRegion)
        # #     shape, iSize = reg.getShape(sRegion)
        # quit()
        # # map = folium.Map(control_scale=True, tiles=None)
        # folium.TileLayer('openstreetmap', name="Open Street Map").add_to(map)
        # folium.TileLayer(tiles='https://server.arcgisonline.com/ArcGIS/rest/services/NatGeo_World_Map/MapServer/tile/{z}/{y}/{x}',
        #                  name="Esri.NatGeoWorldMap",
        #                  maxZoom=16,
        #                  attr='Tiles &copy; Esri &mdash; National Geographic, Esri, DeLorme, NAVTEQ, UNEP-WCMC, USGS, NASA, ESA, METI, NRCAN, GEBCO, NOAA, iPC'
        #                  ).add_to(map)
        # folium.TileLayer(tiles='https://server.arcgisonline.com/ArcGIS/rest/services/Ocean_Basemap/MapServer/tile/{z}/{y}/{x}',
        #                  name="Esri.OceanBasemap",
        #                  attr='Tiles &copy; Esri &mdash; Sources: GEBCO, NOAA, CHS, OSU, UNH, CSUMB, National Geographic, DeLorme, NAVTEQ, and Esri',
        #                  maxZoom= 13
        #                  ).add_to(map)
        # folium.TileLayer(tiles='https://basemap.nationalmap.gov/arcgis/rest/services/USGSTopo/MapServer/tile/{z}/{y}/{x}',
        #                  name="USGS.USTopo",
        #                  attr='Tiles courtesy of the <a href="https://usgs.gov/">U.S. Geological Survey</a>',
        #                  maxZoom=16
        #                  ).add_to(map)
        #
        # resReg = m.subsystem_getDb().fao_getRegions()
        # odicRegions = resReg.getData("regions")
        #
        # lstRegions = list(odicRegions.keys())
        # lstRegions = lstRegions[0:70]
        # iParts = len(lstRegions)
        # colGradient = ColorGradient(Colors.getColor("yellow"), Colors.getColor("purple"), iParts)
        # style  = lambda sCol : lambda feature: dict(fillColor=sCol, weight=1)
        # #lstCol = ["blue", Colors.getColor("awi_blue_dark"), Colors.getColor("awi_blue_mid"), Colors.getColor("awi_blue_light")]
        # for iNr, sReg in enumerate(lstRegions):
        #     sCol = colGradient.getHtml(iNr)
        #     resReg = m.subsystem_getDb().fao_getShape(sReg)
        #     shape = resReg.getData("shape")
        #     if shape is not None:
        #         #sCol = random.choice(lstCol)
        #         #folium.GeoJson(shape, name=sReg, tooltip=sReg, style_function=style(sCol)).add_to(map)
        #         folium.GeoJson(shape, name=sReg, tooltip=sReg, style_function=style(sCol)).add_to(map)
        # folium.LayerControl().add_to(map)
        # map.save("/Users/paul/_temp/region.html")
        # m.subsystem_getConverter().init("bsh")
        # sOut = m.subsystem_getConverter().info("bsh", html=True)
        # file = open("/Users/paul/_temp/test.html", "w")
        # file.write(sOut)
        #res = m.subsystem_getDb().fao_mapping_clean(schema="test", ing=62)
        #print(res)
        #quit()
        #res = m.subsystem_getDb().fao_mapping_clean(schema="test")
        #quit()
        #print(res)
        #res = m.subsystem_getDb().fao_create(schema="test", level="*")
        #print(res)
        #quit()
        # res = m.subsystem_getDb().fao_getRegions()
        # for k, (id, code, level, points) in res.getData("regions").items():
        #     print(id, code, level, k, points)
        #res = m.subsystem_getDb().fao_mapping_update(schema="test")
        #print(res)
        # res = m.subsystem_getDb().fao_mapping_stats(schema="test", short=False)
        # print(res)
        # quit()
        #res = m.subsystem_getDb().fao_mapping_update(schema="test")
        #sLog = res.getData("log")
        #print(sLog)
        #quit()
        # print(res)

        # res = m.subsystem_getDb().fao_mapping_stats(schema="test", short=False)
        # sLog = res.getData("log")
        # print(sLog)
        #quit()

        # res = m.subsystem_getDb().fao_getShape("Arctic Sea")
        # print(res.getData("shape-size"))
        #print(res.getData("shape"))
        # taxpol = m.subsystem_getVirtualAssistant().getTaxaPolishing()
        # res = taxpol.start_polishing("/Users/paul/_temp/tax_polish.xlsx",
        #                                                             "/Users/paul/_temp/tax_polish_final.xlsx",
        #                                                             fuzzy=True,
        #                                                             like=True,
        #                                                             marine_only=False,
        #                                                             puffer=True,
        #                                                             sheet_biota="Taxa",
        #                                                             col_aid="AphiaID_given",
        #                                                             col_scientificname="ScientificName_given")
        # if res.isErr():
        #     print(res.getErrWarnInfoString(short=True))
        # else:
        #     sLog = res.getData("text")
        #     print(sLog)


        # taxrep = m.subsystem_getVirtualAssistant().getTaxaReport()
        # res = taxrep.start_report()
        # if res.isErr():
        #     print(res.getErrWarnInfoString(short=True))
        # else:
        #     sLog = res.getData("text")
        # quit()

        # bat = BatchBot(m)
        # # res = bat.config_getFilenames_all()
        # # res = bat.config_getData("bat_release.txt")
        # # res = bat.config_renameFile("batch.txt", "batch_release.txt")
        # print(res)

        # res = bat.config_getFilenames_all()

        # res = bat.config_getData("bat_release.txt")
        # res = bat.config_renameFile("batch.txt", "batch_release.txt")

        # import os.path
        # import sys
        #
        # from PyQt5.QtWidgets import QApplication

        # class Interceptor(QWebEngineUrlRequestInterceptor):
        #     def interceptRequest(self, info):
        #         info.setHttpHeader(b"Accept-Language", b"en-US,en;q=0.9,es;q=0.8,de;q=0.7")
        #
        # filename = m.conf_getMap()
        # print(filename)
        # app = QApplication(sys.argv)
        # browser = QWebEngineView()
        #
        # #interceptor = Interceptor()
        # #browser.page().profile().setUrlRequestInterceptor(interceptor)
        #
        # browser.load(QUrl.fromLocalFile(filename))
        # print(filename)
        # browser.show()
        # sys.exit(app.exec_())
        #
        # for iId in lstIdsAccessRestricted:
        #     print(dicNames[iId])

        # resDoi = m.subsystem_getDb().lookup_dataset_checkDois()
        # sLog = resDoi.getData("log")
        # print(resDoi)
        # sFile = "/Users/paul/_temp/doi_check_2.html"
        # #print(webbrowser.get())
        # webbrowser.open("file:///"+sFile, new=0)
        #file = open("/Users/paul/_temp/doi_check.html", "w")
        #file.write(sLog)
        # app = QApplication(sys.argv)
        # browser = QWebEngineView()
        # browser.load(QUrl.fromLocalFile(sFile))
        # browser.show()
        # sys.exit(app.exec_())

        #resStatus = m.subsystem_getDb().lookup_getStatus("dataset")
        #print(resStatus)



        # dicAll = resDataset.getData("all-names")
        # dicFiles = resDataset.getData("dataset-files")
        # lstTab = []
        # for iId, sName in dicAll.items():
        #     if iId in dicFiles:
        #         lstFiles = dicFiles[iId]
        #         sFiles = PrettyText.lst2strC(lstFiles)
        #         lstTab.append([sName, len(lstFiles), sFiles])
        # lstTab.sort(key=lambda x: str(x[0]).lower())  # index 1 means second element
        # lstTab.insert(0, ["Dataset", "Usage", "Files"])
        # print(PrettyText.table(lstTab))
        # quit()
        # iGroups = resGear.getData("gear-double-groups")
        # setGroups = resGear.getData("gear-double-groups-set")
        # lstDoubleHeader = resGear.getData("gear-double-header")
        # dicDouble = resGear.getData("gear-double-dic")
        # iDouble = resGear.getData("gear-double")
        # lstDoubleIds = resGear.getData("gear-double-ids")
        # tpGroupHeader = lstDoubleHeader[0]
        # tpConfHeader = lstDoubleHeader[1]
        # sInfo = ""
        # for tpGroup, dicConf in dicDouble.items():
        #     lstGroup = []
        #     for k,v in dict(zip(tpGroupHeader, tpGroup)).items():
        #         lstGroup.append(str(k)+"="+str(v)+"")
        #     sInfo += "* Group: " + PrettyText.lst2strC(lstGroup) + "\n"
        #     for tpConf, lstNames in dicConf.items():
        #         sInfo += ""+str(len(lstNames))+"x: " + PrettyText.lst2strC(lstNames) + "\n"
        #         lstTab = [tpConfHeader]
        #         lstTmp = [it if it is not None else "" for it in tpConf]
        #         lstTab.append(lstTmp)
        #         sInfo += PrettyText.table(lstTab) + "\n\n"
        # sInfo += "* Groups ("+str(iGroups)+"x)\n"
        # print(setGroups)
        # lstTab = [tpGroupHeader]
        # lstTab+=list(setGroups)
        # sInfo += PrettyText.table(lstTab) + "\n\n"
        # print(sInfo)

        #print(m.subsystem_getMapper().lookup_getAllUsages_ingest())
        #print(m.subsystem_getMapper().lookup_getAllUsages_lookup())
        #sCmd="open -a /Applications/critterbase.app/"
        #subprocess.run(sCmd.split(), close_fds=True)
        #print("fertig")
        # process = psutil.Process(os.getpid())
        # print(process.cpu_percent())
        # iMemUse = process.memory_info().rss / (1024 * 1024)
        # iMemUsePercent = process.memory_percent("rss")
        # iCpuUsePercent = process.cpu_percent()
        # sMem = "RAM: "+str("{:.0f}".format(iMemUse))+"MB ("+str("{:.2f}".format(iMemUsePercent)) +"%) CPU: "+str("{:.0f}".format(iCpuUsePercent))+"%"
        # resUpdateLeafTest = m.getDb().check_isLeafExisting(
        #     "HE0185",
        #     "0425",
        #     "1",
        #     "1")
        # dir = m.subsystem_getDirector()
        # #print(dir)
        # print(dir.getIngDepNeed())
        # print(dir.getIngDepBase())
        #res = m.subsystem_getDb().table_isEntryInTable("wormsbase","aid","131344")

        #print(res)
        # resUpdateLeafTest = m.subsystem_getDb().check_isLeafExisting(
        #     cruise="HE0185",
        #     station="0425",
        #     sample="1",
        #     subset="1")
        # resUpdateLeafTest = m.subsystem_getDb().check_isLeafExisting(
        #     cruise="HE0185",
        #     station="0425",
        #     sample="1",
        #     subset="1")
        # print(resUpdateLeafTest)
        # res = m.subsystem_getDb().table_getWholeTable("wormsbase", order="scientificname", gold=True)
        # lstHeaders = res.getData("header")
        # lstContent = res.getData("content")
        # print(res)
        # lstTable = []
        # lstTable.append(lstHeaders)
        # lstTable += lstContent
        # print(PrettyText.table(lstTable))
        #lstAid = [138474, 633169, 102756, 122651, 796572, 149619, 416485, 210835]
        lstAid = [131072, 131073, 2, 147459, 131074, 5, 131077, 139272, 131083, 147467, 131089, 147474, 147475, 131096,
                  131100, 131106, 131107, 131114, 131116, 131117, 131118, 131119, 131121, 51, 131127, 131128, 131129,
                  59, 131131, 1040445, 131134, 131135, 131138, 131141, 131143, 139336, 131144, 131147, 933964, 131149,
                  155727, 131153, 131157, 131159, 131160, 139354, 131164, 131166, 876640, 131169, 131170, 131171, 101,
                  102, 131173, 131174, 105, 139370, 139371, 131177, 131178, 131180, 491631, 131182, 491633, 131183,
                  131184, 131185, 131187, 131188, 131193, 540794, 123, 131201, 491650, 131205, 135, 131209, 114830,
                  139410, 131218, 162, 131237, 131238, 106670, 175, 106673, 106674, 139442, 189, 155839, 192, 193,
                  131266, 131267, 123080, 123082, 123083, 204, 123084, 139474, 211, 139476, 214, 131290, 1073374,
                  139486, 139488, 225, 139490, 123106, 229, 230, 233, 235, 106733, 123117, 458994, 106738, 243, 377084,
                  131327, 131328, 139523, 106758, 106760, 106763, 205077, 114965, 876821, 123160, 876825, 106782,
                  131362, 106788, 139557, 131366, 106791, 131368, 123177, 106800, 327985, 139580, 131388, 123200,
                  934208, 123206, 131398, 131402, 131403, 131405, 139602, 123219, 139604, 106834, 131411, 131415,
                  106854, 131435, 254316, 131436, 139631, 254319, 745846, 139640, 131452, 106877, 139645, 139647,
                  106880, 246148, 139655, 106888, 106889, 131471, 139664, 123281, 131474, 106903, 131480, 139673,
                  131482, 131489, 131490, 139686, 131495, 106922, 131499, 131500, 106925, 131502, 410031, 131504,
                  410032, 131506, 156083, 131507, 131508, 836020, 836033, 131525, 139718, 139719, 139720, 557512,
                  238026, 238027, 131531, 106964, 131540, 131543, 139736, 131554, 131568, 139763, 139765, 139766,
                  131573, 139768, 131574, 131575, 139771, 131576, 123389, 107007, 107010, 107011, 369158, 238104,
                  139800, 1040924, 123426, 107044, 558, 559, 107054, 139832, 737849, 123449, 737852, 107072, 107079,
                  123479, 238180, 336485, 238192, 238194, 238196, 836212, 836215, 238200, 467577, 238201, 139903,
                  131715, 139908, 1450632, 107148, 107150, 107152, 107154, 107155, 254615, 139940, 139942, 107188,
                  123574, 107190, 475838, 107199, 107205, 139975, 107218, 156376, 139993, 139994, 123613, 828127,
                  107232, 156384, 107235, 238310, 107240, 123626, 107253, 107254, 819958, 140029, 140032, 140033,
                  107273, 107275, 107276, 107277, 107278, 107281, 254745, 793, 794, 107292, 107294, 799, 802, 107298,
                  107299, 107301, 107302, 475938, 140072, 238384, 107313, 115513, 107322, 107323, 830, 107333, 140103,
                  140104, 107338, 140108, 107341, 107344, 107345, 852, 107350, 140125, 156510, 140126, 140127, 140129,
                  140132, 107379, 883, 107381, 238454, 238455, 889, 107387, 107388, 107390, 123776, 140161, 107392,
                  107393, 107394, 107395, 140166, 107397, 140168, 140169, 140170, 107398, 140172, 140173, 902, 107400,
                  913, 914, 916, 107414, 919, 476056, 921, 107418, 923, 927, 928, 929, 931, 935, 140199, 140201, 939,
                  943, 107441, 946, 948, 950, 952, 953, 132026, 107451, 140218, 956, 107458, 140227, 140229, 140230,
                  1516487, 140231, 967, 123851, 1516493, 238542, 975, 123856, 976, 107473, 980, 981, 148438, 982, 985,
                  123867, 988, 989, 107486, 992, 993, 140260, 140261, 140262, 140263, 140264, 132072, 140266, 107499,
                  181228, 156655, 107504, 140271, 107506, 107507, 148463, 107513, 140282, 140283, 107518, 140287,
                  181248, 181249, 140295, 140296, 140297, 156681, 140299, 336908, 107533, 140300, 140301, 140302,
                  123922, 246812, 1053, 107551, 107552, 107557, 1066, 156714, 107562, 107564, 148529, 1076, 1080,
                  148537, 468026, 132159, 140352, 123970, 1090, 148550, 123974, 140365, 140366, 148560, 140371, 123990,
                  140374, 140377, 140380, 410717, 107613, 107614, 107615, 107616, 181343, 410724, 148582, 1128, 132200,
                  1130, 148586, 1131, 132201, 1133, 1135, 148592, 1137, 148594, 140403, 181364, 410742, 132216, 132219,
                  148603, 181373, 181372, 107647, 132224, 107649, 410749, 107651, 402566, 132233, 140429, 140430,
                  140431, 140432, 238740, 148629, 140440, 132251, 238751, 107682, 607396, 173223, 107688, 107689,
                  607402, 1195, 140460, 140461, 107690, 140464, 140467, 132277, 1205, 1207, 140471, 525497, 140472,
                  148667, 140474, 140480, 345281, 140482, 148680, 107729, 107731, 156887, 107739, 148701, 132317,
                  107741, 1248, 1033441, 107742, 132324, 132326, 141905, 140521, 148715, 181484, 140528, 1267, 1268,
                  140533, 140536, 124154, 140539, 410876, 181512, 1292, 1296, 181523, 1302, 140571, 836896, 140577,
                  156961, 140579, 140584, 140588, 140589, 140590, 181555, 181556, 181557, 181558, 181559, 1337, 181563,
                  181564, 1341, 181566, 140605, 156991, 181569, 181575, 140616, 181580, 181581, 181582, 1358, 1360,
                  140627, 140629, 181590, 140640, 140643, 140656, 140657, 124273, 140658, 124287, 1410, 394627, 140685,
                  140687, 124316, 124319, 124321, 140710, 140712, 140728, 140730, 140731, 140732, 140733, 140735,
                  140737, 140743, 140744, 140747, 1484, 124373, 140758, 140761, 140767, 140768, 140770, 140771, 124392,
                  124394, 132588, 124402, 124418, 1295878, 861702, 992781, 132627, 1562, 140830, 140834, 140838, 124455,
                  124462, 124465, 157234, 124467, 124470, 140855, 1594, 1599, 1603, 140868, 140869, 140870, 1606, 1607,
                  140873, 1611, 1614, 992858, 607846, 1648, 140916, 140918, 992890, 730747, 747133, 132736, 140929,
                  140942, 132761, 132762, 108207, 140976, 738997, 558775, 140989, 124611, 124612, 353995, 141010,
                  141014, 845528, 124635, 845535, 141025, 1762, 141031, 141033, 132852, 141052, 157437, 1789, 124670,
                  124681, 1806, 141072, 132883, 124694, 1818, 157477, 1839, 993072, 993077, 108343, 108344, 108345,
                  141123, 141134, 141138, 345943, 395099, 395103, 157561, 157566, 370562, 878470, 157576, 157583,
                  141203, 157589, 157590, 157593, 559007, 1509279, 1509285, 124850, 370616, 411579, 247753, 10194,
                  624602, 165853, 1066975, 264171, 141291, 124911, 2032, 124913, 2036, 2040, 2041, 141308, 124929,
                  419843, 124934, 141320, 1001482, 124940, 141334, 146952, 2081, 141359, 141365, 141372, 141374, 133205,
                  370776, 10331, 731235, 247907, 125027, 141424, 141433, 141435, 141436, 157822, 141439, 157828, 141444,
                  125064, 141452, 141454, 125073, 125080, 149668, 845991, 133293, 125125, 125131, 321741, 116956,
                  141539, 141541, 141546, 157933, 125166, 141555, 141556, 116980, 141560, 116986, 157947, 239867,
                  534781, 141565, 231681, 231683, 116998, 141577, 231689, 231690, 141579, 141580, 338192, 157970,
                  141586, 141587, 117015, 231706, 141595, 117025, 117029, 117030, 141607, 117033, 117034, 100653,
                  141615, 141617, 141618, 141621, 100665, 141627, 141632, 141633, 141638, 231751, 1337670, 100681,
                  141639, 141641, 141644, 117070, 141649, 141650, 141651, 125269, 100694, 141662, 125284, 117093,
                  141672, 141673, 117103, 100730, 141690, 117119, 158080, 149904, 141715, 125333, 117149, 182693,
                  182695, 100776, 100777, 141738, 182699, 182701, 100782, 141744, 182708, 248248, 248249, 231870,
                  117183, 100803, 100805, 248261, 231879, 141765, 141767, 248263, 248264, 141782, 100833, 158178,
                  100834, 141799, 117225, 141801, 117234, 100872, 117258, 117260, 100880, 100882, 182807, 125464,
                  182808, 141850, 125469, 182816, 158251, 100910, 141872, 141874, 141875, 100916, 100917, 100927,
                  117320, 117321, 125516, 117324, 117327, 117328, 117329, 117330, 117331, 141907, 141908, 141909,
                  100951, 117336, 141911, 141912, 1501784, 117340, 141916, 117342, 125537, 117348, 141929, 117354,
                  141932, 141934, 117364, 117365, 100982, 117367, 117368, 100986, 100987, 100989, 100991, 117377,
                  117378, 117379, 117380, 100994, 117382, 117383, 100999, 117385, 117386, 125579, 117388, 117389,
                  101002, 101004, 117399, 101016, 117401, 117402, 125598, 117412, 117413, 125606, 731818, 117426,
                  117428, 101053, 101063, 117452, 166605, 150225, 158417, 117469, 117471, 821985, 428773, 117483,
                  117491, 133882, 2824, 158482, 494362, 125723, 117535, 117538, 101156, 2853, 117544, 158504, 133928,
                  101160, 117548, 117554, 117555, 101172, 117561, 1059646, 117591, 117605, 117606, 117607, 379749,
                  1059688, 117631, 183173, 117644, 863123, 117654, 822172, 560033, 822178, 117674, 175026, 175027,
                  535477, 150457, 117690, 175043, 117702, 183242, 125909, 125911, 863197, 117736, 134121, 125930,
                  117743, 101361, 150514, 134131, 101364, 101365, 101368, 150521, 150520, 117755, 207871, 101376,
                  150531, 150534, 117768, 101388, 101389, 101395, 101397, 101400, 322588, 117791, 101409, 101417,
                  117804, 150575, 125999, 150577, 134194, 101430, 117815, 134204, 150590, 117824, 101445, 101450,
                  371788, 117842, 101459, 101464, 117848, 101469, 101470, 101471, 879714, 150630, 150637, 117870,
                  150638, 101489, 117877, 150646, 101493, 150648, 101497, 117883, 150654, 478336, 117888, 117890,
                  101509, 134285, 117903, 150672, 117906, 117907, 445590, 117912, 117913, 150683, 101537, 117925,
                  183466, 240817, 101563, 101567, 101571, 494791, 101580, 126160, 101586, 117980, 117982, 117988,
                  126180, 117990, 593127, 593128, 167144, 117994, 117995, 593131, 167151, 167160, 101633, 421139,
                  126227, 101665, 101669, 240939, 101679, 150842, 101694, 167235, 101702, 101704, 126281, 290131,
                  118100, 101716, 150868, 150874, 290145, 290147, 118119, 101736, 150890, 150892, 290156, 101742,
                  994671, 290157, 118127, 118128, 150894, 118133, 257397, 118137, 118139, 118146, 118148, 118149,
                  216452, 101764, 118152, 118153, 101770, 118154, 118156, 118158, 880017, 150930, 118168, 11676, 101789,
                  101796, 118182, 126375, 339374, 118196, 11707, 101820, 118205, 134591, 134592, 396735, 101830, 134599,
                  118216, 118218, 134603, 126413, 126415, 101839, 126417, 118226, 126421, 101846, 11734, 126425, 126426,
                  101851, 118235, 101853, 118236, 142817, 101857, 101858, 126436, 126437, 126438, 101864, 126441,
                  126442, 142827, 126445, 126446, 101871, 126448, 126449, 126450, 134643, 142835, 142837, 142838,
                  126458, 126461, 118273, 101891, 118277, 101896, 101898, 142861, 134674, 101908, 126484, 134687,
                  134688, 101926, 101928, 126505, 101930, 101933, 142894, 724527, 716336, 134705, 716334, 716341,
                  716343, 716344, 716345, 716353, 134723, 134725, 101960, 134730, 101963, 101966, 101967, 101968,
                  134741, 134744, 101977, 101979, 126555, 101981, 101982, 102000, 102002, 102012, 118399, 118401,
                  102018, 102021, 102032, 102033, 102036, 134805, 1044116, 102039, 102040, 102043, 102048, 102053,
                  102057, 102061, 102064, 118454, 102075, 102082, 102085, 102087, 102089, 118474, 478923, 102092,
                  102096, 102100, 102101, 102111, 339681, 102115, 102119, 151275, 102126, 102127, 102131, 102132,
                  102135, 102136, 102139, 1068800, 143106, 110345, 102160, 126736, 102163, 102164, 102167, 102168,
                  102172, 126751, 126752, 102178, 126754, 126755, 126756, 110377, 110378, 110379, 110380, 110383,
                  110387, 110393, 110398, 143167, 151365, 126792, 126793, 126795, 110412, 110427, 290652, 126822,
                  110440, 102251, 110444, 110445, 102253, 102261, 274294, 102263, 102267, 110462, 274304, 110465,
                  110466, 102274, 102275, 102276, 102277, 110471, 110472, 102279, 110474, 102281, 110467, 102285,
                  110481, 110483, 126868, 102292, 110487, 110488, 102296, 110490, 102299, 110492, 126878, 110494,
                  110497, 126892, 102317, 126898, 110514, 110516, 118715, 110524, 126916, 118726, 110535, 110536,
                  102345, 102347, 118732, 126925, 110542, 110543, 126928, 126930, 577495, 479195, 102364, 102365,
                  102367, 102371, 118757, 110567, 102377, 102379, 102380, 102383, 102387, 126964, 126965, 126975,
                  102401, 102403, 102408, 126985, 126986, 102412, 118801, 126996, 110618, 127007, 102431, 102432,
                  102433, 110626, 110627, 102437, 110628, 102439, 102440, 102443, 151596, 102447, 127023, 135220,
                  233525, 798772, 118839, 847930, 118843, 102460, 151612, 102462, 135226, 151616, 102466, 102467,
                  118852, 1044546, 102470, 397383, 118859, 151628, 110671, 1151064, 102497, 110698, 127082, 127084,
                  102513, 102516, 381057, 118917, 151687, 135304, 143495, 135306, 151689, 110733, 135312, 127123,
                  127126, 135322, 127136, 127137, 127138, 127139, 127140, 151717, 127141, 127143, 389288, 118953,
                  766122, 102570, 127147, 151718, 118958, 127149, 127150, 127153, 127151, 110775, 127160, 143546,
                  110783, 102598, 102599, 143558, 143564, 118990, 118994, 757970, 127186, 127190, 143574, 143576,
                  119000, 102620, 102624, 250081, 127203, 127204, 127214, 119024, 127219, 127220, 110839, 872696,
                  266489, 119034, 119038, 119039, 471295, 471297, 119042, 119043, 119044, 119046, 119047, 119048,
                  102665, 119050, 250126, 102671, 102672, 160017, 102673, 160019, 127262, 143647, 143650, 102690,
                  102691, 102696, 151860, 110903, 110904, 110911, 102720, 102736, 151894, 102742, 102748, 102758,
                  102771, 102779, 102783, 110975, 102787, 102788, 102789, 176517, 225673, 102794, 102795, 135566,
                  102798, 110993, 127379, 127380, 102805, 102807, 168344, 127385, 102811, 127387, 127389, 233888,
                  127393, 111010, 233891, 111016, 151982, 900, 102837, 901, 102839, 111032, 102840, 102841, 102843,
                  102844, 274877, 151997, 143808, 152002, 127427, 143814, 102860, 143825, 143834, 111067, 102882,
                  111079, 143853, 102897, 233970, 111099, 102915, 102916, 102917, 102918, 152074, 102924, 1015302,
                  102928, 102929, 1372692, 111125, 225814, 102932, 234025, 111147, 111152, 111153, 102960, 102963,
                  111158, 111159, 102968, 234043, 102972, 234047, 102975, 102989, 111186, 111187, 234067, 111190,
                  234073, 111193, 234074, 111196, 111197, 234077, 111199, 234079, 103007, 103008, 234093, 234098,
                  234099, 234102, 234103, 103031, 111228, 111230, 103042, 152196, 103044, 152198, 168587, 103058,
                  103059, 103060, 152213, 742036, 742037, 103064, 152217, 103066, 111250, 103068, 152221, 111254,
                  103073, 103076, 111268, 152229, 152231, 103077, 103079, 152234, 111285, 152249, 152250, 152252,
                  160446, 152259, 103110, 111304, 152267, 103116, 152269, 103118, 111313, 111314, 152276, 152279,
                  111320, 152283, 103132, 152285, 152287, 152288, 234208, 152292, 152293, 144102, 111337, 152302,
                  111343, 152304, 111350, 111351, 111352, 152313, 111353, 111354, 111355, 324346, 103166, 152316,
                  152320, 111361, 144129, 152322, 152324, 111365, 160514, 111367, 152321, 103169, 103175, 152331,
                  111374, 111379, 176919, 152345, 160539, 152352, 103202, 152356, 111397, 152357, 111399, 103208,
                  152365, 152367, 103217, 152370, 111411, 103220, 881461, 111418, 152378, 103226, 111421, 103228,
                  152383, 103233, 103235, 152389, 152391, 152392, 152394, 152395, 103251, 136021, 103256, 103259,
                  152414, 152420, 136041, 152426, 152428, 136044, 152431, 136049, 152435, 594808, 152441, 152443,
                  111484, 136060, 136061, 136063, 152448, 332672, 111495, 111496, 136073, 111498, 111500, 111509,
                  955291, 144293, 144294, 111527, 144296, 955309, 111538, 111539, 111540, 111547, 111558, 136135,
                  111573, 136164, 111589, 111594, 111596, 111597, 111600, 111601, 111602, 824307, 111603, 111604,
                  111606, 111607, 152569, 111611, 111614, 422916, 111621, 111622, 111624, 152587, 119822, 111631,
                  111632, 103443, 111635, 234519, 103448, 816154, 103450, 111644, 111652, 1037349, 111653, 1381415,
                  111665, 594993, 111671, 111673, 144442, 103483, 103484, 111676, 119870, 226375, 111690, 283723,
                  111695, 111696, 103509, 111706, 111707, 119898, 119905, 119906, 111719, 103528, 111721, 111730,
                  103552, 119936, 111745, 119939, 144521, 210060, 111757, 119950, 111760, 103570, 136340, 111765,
                  283798, 144538, 103579, 111796, 332984, 111800, 111802, 111803, 119997, 111806, 111807, 472256,
                  111808, 152774, 120019, 120020, 120025, 120026, 103647, 111839, 120039, 103656, 13551, 13552, 120052,
                  103670, 103674, 1324283, 152828, 152829, 120062, 1324287, 1324288, 431365, 120072, 136457, 136465,
                  136466, 382226, 152852, 103702, 423191, 120087, 120089, 472353, 103717, 103718, 103719, 136486,
                  120113, 103732, 136503, 136505, 136506, 103743, 734530, 734532, 120136, 144714, 103758, 103759,
                  120144, 103764, 103765, 120155, 103775, 234848, 103776, 234850, 234851, 136546, 333160, 103788,
                  382316, 103791, 103792, 120177, 120178, 120180, 472439, 103807, 177538, 120203, 120204, 120206,
                  292242, 152986, 1488284, 103862, 153027, 103882, 103894, 505319, 103926, 103929, 128506, 103930,
                  488957, 488966, 128539, 1381919, 128545, 128546, 128548, 128549, 128550, 128551, 128552, 144937,
                  153131, 177726, 423507, 751247, 112299, 104152, 14107, 153375, 153380, 481067, 5953, 325483, 341892,
                  128900, 128901, 153507, 333735, 587704, 153552, 153558, 22496, 284644, 284645, 210916, 333844, 710680,
                  145440, 178212, 129103, 481374, 22626, 153717, 153728, 137344, 153730, 129155, 333954, 145541, 153734,
                  145545, 1398927, 145551, 153764, 489646, 137390, 137399, 137402, 129211, 137404, 129218, 1071300,
                  137413, 129220, 137416, 137422, 137424, 596180, 129240, 129242, 596189, 137438, 129249, 145650,
                  137459, 129266, 129270, 153847, 719102, 137471, 137477, 137478, 137480, 137481, 137482, 137484,
                  137485, 129293, 137486, 129296, 506128, 137487, 129307, 129309, 129313, 235809, 137507, 235811,
                  129337, 145721, 129341, 137543, 137547, 137551, 137553, 137556, 137559, 137561, 129370, 137562,
                  137563, 129373, 137565, 137568, 137569, 129378, 129379, 137570, 137571, 137577, 137582, 481647,
                  252278, 334212, 129413, 129414, 137607, 137609, 424332, 129420, 153996, 153998, 129425, 129426,
                  129427, 129430, 154006, 154008, 129437, 137630, 129439, 129443, 129445, 129446, 129451, 129455,
                  1382835, 571832, 833984, 129472, 104906, 137675, 129487, 129488, 178639, 834002, 137683, 129491,
                  743898, 834010, 129499, 834018, 834020, 137701, 137704, 129514, 129525, 424437, 834039, 424440,
                  154107, 129534, 137732, 137739, 129549, 424463, 424465, 129555, 129572, 129582, 23091, 129594, 129595,
                  145984, 129605, 547398, 129614, 137807, 129615, 334415, 334417, 129619, 129620, 129621, 129623,
                  129625, 129626, 137820, 236129, 236130, 129646, 129654, 162422, 490104, 293496, 129659, 236157,
                  416382, 137867, 465551, 129679, 129680, 285333, 1038999, 817816, 416409, 490139, 170654, 129697,
                  416417, 334506, 334508, 424622, 334510, 424624, 334512, 334514, 129710, 129715, 129718, 367297,
                  408266, 137943, 146142, 137954, 178915, 416482, 637672, 129769, 129775, 129776, 129777, 129778,
                  129781, 129784, 129785, 293624, 129789, 416510, 129792, 416513, 416514, 293632, 416516, 129797,
                  129804, 137997, 154385, 162581, 604950, 129819, 170782, 236327, 129834, 465707, 138027, 129837,
                  129840, 129844, 416570, 129851, 416573, 817982, 129856, 129867, 129868, 138063, 129875, 129876,
                  129877, 138068, 129879, 154453, 129884, 138081, 129892, 334692, 154473, 129898, 457588, 129912,
                  129914, 129924, 162695, 138120, 416648, 482186, 129937, 129938, 334741, 138135, 129944, 138138,
                  138139, 129948, 129951, 555935, 129955, 129959, 138154, 138157, 236462, 129967, 138158, 138159,
                  334772, 129984, 105408, 105410, 129989, 129992, 236488, 236495, 367568, 105440, 138211, 130020,
                  105443, 130023, 105450, 138223, 130031, 138225, 146420, 130041, 236541, 138239, 162819, 236548,
                  138262, 130099, 130100, 146486, 130103, 146492, 130110, 130113, 130116, 130118, 130119, 130122,
                  138315, 130123, 130126, 130127, 130128, 130130, 130131, 146517, 146518, 130136, 154712, 130140,
                  138333, 130141, 130143, 425058, 138339, 146532, 334946, 130156, 130157, 744567, 490616, 130168,
                  130171, 130172, 130174, 130176, 236678, 130185, 130187, 138388, 130197, 130198, 834720, 138401,
                  867492, 130219, 236715, 138413, 236719, 113843, 138421, 130236, 163005, 130238, 138432, 130244,
                  138438, 744647, 130248, 138445, 138451, 130261, 130263, 138456, 130266, 130268, 130269, 130270,
                  130271, 130272, 130274, 138474, 138477, 130305, 130311, 130314, 130319, 163088, 130326, 130327,
                  130329, 130330, 130331, 138533, 154918, 130343, 130345, 146733, 138543, 130352, 130353, 146738,
                  130355, 130356, 130357, 138549, 130359, 130362, 130363, 130364, 130365, 130366, 130367, 130374,
                  130375, 146768, 130387, 105814, 130391, 130392, 105815, 105820, 138590, 769375, 130404, 130407,
                  195953, 146802, 146803, 130417, 130426, 130437, 105865, 138636, 195980, 146832, 138648, 105883,
                  105887, 196002, 130467, 105891, 155049, 138681, 130491, 138686, 130494, 130495, 990657, 130496,
                  138691, 130500, 122309, 105923, 122312, 122313, 122314, 163275, 237004, 130506, 130507, 146895,
                  122319, 130512, 146900, 138709, 130518, 138711, 146905, 130522, 146907, 130523, 122329, 146912,
                  712167, 130537, 146922, 122348, 138734, 130542, 130543, 146928, 130544, 122356, 146932, 122358,
                  146933, 130554, 130555, 138748, 138749, 138751, 138752, 130564, 1015301, 138758, 146950, 138760,
                  130569, 130570, 138763, 130572, 122379, 1015304, 1015306, 1015303, 1015307, 130578, 155155, 122388,
                  130581, 130585, 130589, 130590, 155173, 138789, 138791, 146984, 130598, 130599, 130601, 130603,
                  155181, 106033, 138802, 130613, 130614, 146999, 130616, 130617, 130618, 155194, 138812, 138814,
                  147006, 130624, 130625, 138818, 147008, 138821, 138823, 138824, 106057, 130631, 147021, 147022,
                  130639, 130640, 327249, 130641, 130642, 130643, 130644, 138831, 147027, 147026, 130649, 147032,
                  532050, 130655, 130661, 122471, 130666, 130668, 122478, 130670, 130673, 130683, 130684, 138878,
                  851584, 851589, 130696, 851593, 106122, 851599, 122511, 147087, 138899, 130707, 138903, 130711,
                  130712, 130714, 106142, 122528, 147104, 130722, 147109, 122534, 122536, 138920, 130732, 130735,
                  122543, 130737, 130738, 147123, 130745, 122555, 130748, 130749, 130754, 130756, 138952, 130760,
                  130762, 130763, 130769, 130770, 138963, 155360, 106209, 130785, 868065, 122596, 106213, 106210,
                  106215, 138984, 122599, 106218, 106219, 138990, 138992, 130801, 122610, 138998, 138999, 106231,
                  130809, 130810, 130811, 139004, 130813, 147195, 130815, 130816, 130817, 139008, 139009, 139010,
                  605957, 605958, 139011, 139012, 122626, 122632, 122637, 122641, 106257, 130837, 196375, 122651,
                  196385, 130852, 122666, 130859, 139054, 130866, 130867, 130871, 1040184, 130878, 130888, 130891,
                  851788, 155476, 122710, 130906, 122717, 130913, 139106, 130915, 573286, 130921, 106346, 376686,
                  130926, 327537, 122739, 122747, 139134, 1040258, 122759, 589704, 122764, 122768, 106386, 130963,
                  155542, 130967, 130969, 122778, 360352, 106401, 130977, 130979, 130980, 122791, 1073066, 139178,
                  130988, 565166, 122806, 131009, 122817, 122819, 122822, 122823, 761800, 139211, 139217, 1040337,
                  131027, 139229, 16352, 16353, 245736, 131051, 131066, 131069, 162579]
        lstAid_small= [332984, 130270, 131159, 794, 246812, 131027, 139229, 16352, 16353, 245736, 131051, 131066, 131069, 162579]
        lstAid_mid = [131072, 131073, 2, 147459, 131074, 5, 131077, 139272, 131083, 147467, 131089, 147474, 147475, 131096,
         131100, 131106, 131107, 131114, 131116, 131117, 131118, 131119, 131121, 51, 131127, 131128, 131129,
         59, 131131, 1040445, 131134, 131135, 131138, 131141, 131143, 139336, 131144, 131147, 933964, 131149,
         155727, 131153, 131157, 131159, 131160, 139354, 131164, 131166, 876640, 131169, 131170, 131171, 101,
         102, 131173, 131174, 105, 139370, 139371, 131177, 131178, 131180, 491631, 131182, 491633, 131183,
         131184, 131185, 131187, 131188, 131193, 540794, 123, 131201, 491650, 131205, 135, 131209, 114830,
         139410, 131218, 162, 131237, 131238, 106670, 175, 106673, 106674, 139442, 189, 155839, 192, 193,
         131266, 131267, 123080, 123082, 123083, 204, 123084, 139474, 211, 139476, 214, 131290, 1073374,
         139486, 139488, 225, 139490, 123106, 229, 230, 233, 235, 106733, 123117, 458994, 106738, 243, 377084,
         131327, 131328, 139523, 106758, 106760, 106763, 205077, 114965, 876821, 123160, 876825, 106782,
         131362, 106788, 139557, 131366, 106791, 131368, 123177, 106800, 327985, 139580, 131388, 123200,
         934208, 123206, 131398, 131402, 131403, 131405, 139602, 123219, 139604, 106834, 131411, 131415,
         106854, 131435, 254316, 131436, 139631, 254319, 745846, 139640, 131452, 106877, 139645, 139647,
         106880, 246148, 139655, 106888, 106889, 131471, 139664, 123281, 131474, 106903, 131480, 139673,
         131482, 131489, 131490, 139686, 131495, 106922, 131499, 131500, 106925, 131502, 410031, 131504]
        tc = TaxaCruncher(m)
        tc.reset()
        for iAid in lstAid_mid:
            tc.addTaxon(iAid, None, None)
        res = tc.crunch(html=True)
        taxa = res.getData("taxa")
        sStat_html = taxa.getStatistic(html=True)
        sStat_ascii = taxa.getStatistic(html=False)
        print(sStat_ascii)

        sLog_html = PrettyText.html_txt2html(sStat_html)
        with open("/Users/pkloss/_temp/tc.html", "w") as f:
            f.write(sLog_html)
        quit()

        print(res)


        #lstAid = [138474, 633169, 102756, 122651, 796572, 149619, 961694, 416485, 210835, 950, 935]
        #lstAid = [122313]
        # for iNr, iAid in enumerate(lstAid):
        #     t1_res=m.subsystem_getMyWorms().getTaxon(iAid, deage_force=True)
        #     t1 = t1_res.getData("taxon")
        #     print(iNr, len(lstAid), t1)
        # # print(t1.signature(debug=True))
        # # print(t1.isLeadingToAnotherTaxon())
        # # print(t1.isLeadingToAnotherTaxon_accepted())
        #     m.subsystem_getMyWorms().puffer_save()
        # quit()

        # t1_res = m.subsystem_getMyWorms().getTaxon(246812, deage_force=False)
        # t1 = t1_res.getData("taxon")
        # print(t1.getAphiaId_accepted())
        # print(t1.isRedirected())
        # print(t1.isRedirected_2_accepted())
        # print(t1.isRedirected_2_accepted())
        # print(t1.getAphiaId_accepted())
        # print(t1)

        # lstTaxa = []
        # taxa = Taxa(m)
        # sInfo_html = "<br><br><br><br>"
        # for iAid in  lstAid:
        #     resMyWorms = m.subsystem_getMyWorms().getTaxon(iAid)
        #     #print(resMyWorms)
        #     if resMyWorms.isOk() and resMyWorms.getData("found"):
        #         t=resMyWorms.getData("taxon")
        #         lstTaxa.append(t.signature(html=True))
        #         taxa.addTaxon(t)
        # # m.subsystem_getMyWorms().puffer_save()
        # # for sTaxon in lstTaxa:
        # #     sInfo_html += sTaxon +"<br>"
        # sInfo_html = taxa.getStatistic(True, accepted=False)
        # sInfo_site  = PrettyText.html_txt2html(sInfo_html)
        # with open("/Users/paul/_temp/t.html", "w") as f:
        #     f.write(sInfo_site)
        # sName = m.getDb().ingest_getIngestFilenames(124)
        # print(sName)
        # quit()


        # for sTable, sSql in resUpdateLeafTest.getData("sql").items():
        #     print(sTable, sSql)
        # print(sMem)
        # res0 = Result()
        # res = Result()
        # res0.addErr("Test",
        #            add="Paul")
        # res.addResult(res0, msg_only=True,  sheet="Biota", row=1,kat="DB", sys="Ingestor")
        # print(res)
        # res.addErr("Biota with tuple: AphiaId/LifeStage/Sieve/Specification not uniq per subset",
        #            add="subset: Paul", sheet="Biota", row=2,
        #            kat="Biota Check", sys="Ingestor")
        # res.addErr("Biota with tuple: AphiaId/LifeStage/Sieve/Specification not uniq per subset",
        #            add="subset: Otto", sheet="Biota", row=1,
        #            kat="Biota Check", sys="Ingestor")
        # res.addErr("Biota with tuple: AphiaId/LifeStage/Sieve/Specification not uniq per subset",
        #            add="subset: Otto", sheet="Biota", row=2,
        #            kat="Biota Check", sys="Ingestor")
        # print(res)
        #res = m.getMyWorms().queryByName("Molpadia", puffer=True, fuzzy=False)
        #res = m.getMyWorms().queryByName("Abra alba", puffer=False, fuzzy=False)
        #res = m.checkUpdateApp()
        #print(res)
        # print(result)
        #print(res)
        #res = m.checkUpdateApp()
        #print(res)
        # db = m.getDbSchema()
        # map = m.getMapper()
        # for sTable in map.ingest_getDbTables():
        #     for sAtr in map.ingest_getDbAttributes(sTable):
        #         sSheet, sCol = map.ingest_db2excel(sTable,sAtr)
        #         #print(sTable,sAtr,sSheet,sCol)
        #         bLup = map.ingest_isLookup(sSheet, sCol)
        #         if bLup:
        #             # Lup
        #             sLupTable = map.ingest_getLookup(sSheet, sCol)
        #             sLupTableKey = map.lookup_getKey(sLupTable)
        #             lstLupTableCols = map.lookup_getDbAttributes(sLupTable)
        #             for sLupAtr in lstLupTableCols:
        #                 sLupSheet, sLupCol = map.lookup_db2excel(sLupTable, sLupAtr)
        #                 if sLupAtr != "gold":
        #                     print(sLupTable+"."+sLupAtr+" AS \""+sSheet+"."+sLupSheet+"."+sLupCol+"\",")
        #                 bLupLup = map.lookup_moreLookupNeeded_isAttributeNeeding(sLupTable, sLupAtr)
        #                 if bLupLup:
        #                     # Lup 2
        #                     sLupLupTable = map.lookup_moreLookupNeeded_getNeededLookup(sLupTable,sLupAtr)
        #                     sLupLupTableKey = map.lookup_getKey(sLupLupTable)
        #                     lstLupLupTableCols = map.lookup_getDbAttributes(sLupLupTable)
        #                     for sLupLupAtr in lstLupLupTableCols:
        #                         sLupLupSheet, sLupLupCol = map.lookup_db2excel(sLupLupTable, sLupLupAtr)
        #                         print("--", sLupLupTable, sLupLupAtr, sLupLupSheet, sLupLupCol)
        #             print()
        # print()
        # # Maping Comments
        # print("Db.Table","Db.Attribute","EXCEL.Sheet", "Excel.Column", "Comment.JSON", sep="\t")
        # for sTable in map.ingest_getDbTables():
        #     for sAtr in map.ingest_getDbAttributes(sTable):
        #         sSheet, sCol = map.ingest_db2excel(sTable,sAtr)
        #         res = db.getDbSchemaInfo("ingest",sSheet,sCol)
        #         sComJson = res.getData("comment-json")
        #         print(sTable,sAtr,sSheet,sCol,sComJson,sep="\t")
        #     print("","","","","",sep="\t")
        #
        # print("Db.Table","Db.Attribute","EXCEL.Sheet", "Excel.Column", "Comment.JSON", sep="\t")
        # for sTable in map.lookup_getDbTables():
        #     for sAtr in map.lookup_getDbAttributes(sTable):
        #         sSheet, sCol = map.lookup_db2excel(sTable,sAtr)
        #         res = db.getDbSchemaInfo("lookup",sSheet,sCol)
        #         sComJson = res.getData("comment-json")
        #         print(sTable,sAtr,sSheet,sCol,sComJson,sep="\t")
        #     print("","","","","",sep="\t")

        #t = Taxon({"aid":-2})
        # t0 = Taxon(blueprint={"aid":-1, "scientificname": "Hallo"})
        # t2 = Taxon(123506)
        # t3 = Taxon(935)
        # t3 = Taxon(557488)

        # dicTaxonWorms  = {'AphiaID': 119036, 'url': 'http://www.marinespecies.org/aphia.php?p=taxdetails&id=119036',
        #  'scientificname': 'Saduria sabini', 'authority': '(Krøyer, 1849)', 'status': 'accepted',
        #  'unacceptreason': None, 'taxonRankID': 220, 'rank': 'Species', 'valid_AphiaID': 119037,
        #  'valid_name': 'Saduria sabini', 'valid_authority': '(Krøyer, 1849)', 'parentNameUsageID': 118451,
        #  'kingdom': 'Animalia', 'phylum': 'Arthropoda', 'class': 'Malacostraca', 'order': 'Isopoda',
        #  'family': 'Chaetiliidae', 'genus': 'Saduria',
        #  'citation': 'Boyko, C.B.; Bruce, N.L.; Hadfield, K.A.; Merrin, K.L.; Ota, Y.; Poore, G.C.B.; Taiti, S.; Schotte, M.; Wilson, G.D.F. (Eds) (2008 onwards). World Marine, Freshwater and Terrestrial Isopod Crustaceans database. Saduria sabini (Krøyer, 1849). Accessed through: World Register of Marine Species at: http://www.marinespecies.org/aphia.php?p=taxdetails&id=119036 on 2021-01-05',
        #  'lsid': 'urn:lsid:marinespecies.org:taxname:119036', 'isMarine': 1, 'isBrackish': 0, 'isFreshwater': 0,
        #  'isTerrestrial': 0, 'isExtinct': 0, 'match_type': 'exact', 'modified': '2019-07-22T19:06:47.557Z'}
        #
        #
        # dicTaxonWorms = {'aid': 124861, 'url': None, 'scientificname': None, 'authority': None, 'status': 'deleted', 'unacceptreason': 'AphiaID resurrection', 'taxonRankID': None, 'rank': None, 'vaid': 124860, 'valid_name': 'Ophiocten sericeum', 'valid_authority': '(Forbes, 1852)', 'parentNameUsageID': 123561, 'kingdom': None, 'phylum': None, 'class': None, 'order': None, 'family': None, 'genus': None, 'citation': None, 'lsid': 'urn:lsid:marinespecies.org:taxname:124861', 'ismarine': None, 'isbrackish': None, 'isfreshwater': None, 'isterrestrial': None, 'isextinct': None, 'match_type': 'exact', 'modified': '2012-05-30T11:00:00Z'}
        #
        # dicTaxon = {}
        # for k, v in dicTaxonWorms.items():
        #     k = Taxon.getMapHeaderWorms2Intern()[k] if k in Taxon.getMapHeaderWorms2Intern() else k
        #     dicTaxon[k] = v
        # t1 = Taxon(blueprint=dicTaxon)
        # t3 = Taxon(177684)
        #
        # print(t3.info())
        #print(dicTaxonWorms)
        #print(t2.hasProblems())
        # print(t2)
        #print(t3,t3.hasProblems())

        ##m.setDebug(True)
        # m.getLookup().update("person")
        # ant = m.getLookup().getLookupCacheAll("person", reverse=True)
        # print(ant)
        # print(ant.getData("lookup"))
        # quit()

        #res = m.getDirector().reconnect()
        #print(m.getDirector())
        #print(res.getErrWarnInfoString(short=True))

        # wb = MultiSheet(name="Test Workbench")
        # res = wb.setSheet("Sample", ["x", "y", "z"], [])

        #res = wb.setSheet("Sample", ["x","y","z"], [[1,2,3],[3,4,6]])
        #res = wb.setSheet("Paul", ["x", "y", "z"], [[1, 2, 3], [3, 4, 6]])

        # if res.isErr():
        #     print(res.getErrWarnInfoString())
        #     quit()

        # wb.setOpenSheetIdx(0)
        # print(wb.isOk())
        # print(wb.getOpenSheetIdx())

        #dicTaxon = {'is_colony': True, 'class': 'Magnoliopsida', 'order': 'Asterales', 'authority': 'Fernald & H.St.John', 'valid_name': 'Pilosella verruculata subsp. verruculata', 'ismarine': 0, 'phylum': 'Tracheophyta', 'isextinct': None, 'aaid_way': [1234567, 1151064], 'aid': 1151064, 'kingdom': 'Plantae', 'isterrestrial': 1, 'valid_authority': '(Link) Soják', 'aaid': 1151062, 'url': 'http://www.marinespecies.org/aphia.php?p=taxdetails&id=1234567', 'isbrackish': 0, 'unacceptreason': None, 'lsid': 'urn:lsid:marinespecies.org:taxname:1234567', 'status': 'unaccepted', 'family': 'Compositae', 'rank': 'Variety', 'modified': '2013-09-23T06:42:42Z', 'match_type': 'exact', 'citation': 'Flann, C. Global Compositae Checklist. Hieracium scabrum var. leucocaule Fernald & H.St.John. Accessed through: World Register of Marine Species at: http://www.marinespecies.org/aphia.php?p=taxdetails&id=1234567 on 2019-07-25', 'vaid': 1151064, 'scientificname': 'Hieracium scabrum var. leucocaule', 'aname': 'Pilosella verruculata subsp. verruculata', 'isfreshwater': 0, 'genus': 'Hieracium'}
        #t1 = Taxon(123506)
        #t3 = Taxon(446035)
        #t2 = Taxon(blueprint=dicTaxon)
        #t1.overide("is_colony", True)
       # print(t1)
        #print(t3.info())
        #print(t1.cmp(t2))
        #print(Taxon.checkCompatibility2Worms().isOk())
        #ant = Taxon.askWormsByScientificname("abr", True)
        #print(len(ant), ant)
        # import openpyxl
        # sFile = ""
        # warnings.simplefilter("ignore")  # Ignoriere Warnungen
        # self.__inputExcelDatei = openpyxl.load_workbook(sFile,data_only=True)  # Keine Formeln sondern letzter gespeicherter Wert
        # warnings.simplefilter("default")  # Warnungen wieder aktivieren
        # lstBar = [('Age Group', 'Taxa'), ('88 days', 18), ('87 days', 28), ('86 days', 16), ('85 days', 15), ('84 days', 26),
        #  ('83 days', 22), ('82 days', 26), ('81 days', 40), ('80 days', 30), ('79 days', 25), ('78 days', 37),
        #  ('77 days', 19), ('76 days', 28), ('75 days', 18), ('74 days', 27), ('73 days', 30), ('72 days', 20),
        #  ('71 days', 26), ('70 days', 35), ('69 days', 28), ('68 days', 27), ('67 days', 31), ('66 days', 18),
        #  ('65 days', 30), ('64 days', 19), ('63 days', 24), ('62 days', 34), ('61 days', 25), ('60 days', 29),
        #  ('59 days', 32), ('58 days', 27), ('57 days', 24), ('56 days', 34), ('55 days', 27), ('54 days', 24),
        #  ('53 days', 32), ('52 days', 32), ('51 days', 36), ('50 days', 25), ('49 days', 23), ('48 days', 23),
        #  ('47 days', 23), ('46 days', 21), ('45 days', 28), ('44 days', 28), ('43 days', 26), ('42 days', 28),
        #  ('41 days', 22), ('40 days', 20), ('39 days', 28), ('38 days', 25), ('37 days', 32), ('36 days', 33),
        #  ('35 days', 33), ('34 days', 23), ('33 days', 27), ('32 days', 24), ('31 days', 17), ('30 days', 19),
        #  ('29 days', 25), ('28 days', 33), ('27 days', 30), ('26 days', 24), ('25 days', 22), ('24 days', 23),
        #  ('23 days', 21), ('22 days', 29), ('21 days', 23), ('20 days', 20), ('19 days', 24), ('18 days', 29),
        #  ('17 days', 28), ('16 days', 23), ('15 days', 23), ('14 days', 19), ('13 days', 28), ('12 days', 23),
        #  ('11 days', 27), ('10 days', 25), ('9 days', 26), ('8 days', 27), ('7 days', 31), ('6 days', 19),
        #  ('5 days', 23), ('4 days', 24), ('3 days', 23), ('2 days', 34), ('1 days', 27), ('0 days', 75)]
        #
        # print(PrettyText.barchart(lstBar, maxbarsize=25))

        # res = Result("Res1")
        # res2 = Result("Res2")
        # res3 = Result("Res3")
        # res4 = Result("Res4")
        #
        # sKat1 = "Probleme mit Aphia Ids"
        # sKat2 = "Probleme mit Gears"
        #
        # sErr1 = "Doppelte Aphia Ids"
        # sErr2 = "Verkorkste Aphia Ids"
        # sErr3 = "Doofe Aphia Ids"
        # sErr4 = "Gear Area ist kaputt"
        # sInfo = "Tolle Information"
        # sWarn = "Achtung bitte Dings!"
        #
        # res2.addErr(sErr1, kat=sKat1, sys="Eigenes System")
        # res2.addErr("Pupsfehler", add="Knoedel")
        # res2.addErr(sErr1, kat=sKat1, sheet="sample")
        # res2.addErr(sErr2, kat=sKat1, sheet="sample")
        # res2.addErr(sErr1, kat=sKat1, sheet="person")
        # res2.addErr(sErr1, kat=sKat1, sheet="sample", row="17")
        # res2.addErr(sErr1, kat=sKat1, sheet="sample", row="18")
        # res2.addWarn(sErr1, kat=sKat2, sheet="cruise", row="18", sys="Eigenes Warnungs System")
        # res2.addErr(sErr1, kat=sKat1, row=7, col="name", sheet="sample", sys="Eigenes System")
        #
        # res.addWarn(sErr1, kat=sKat2, sheet="cruise", row="18", sys="Eigenes Warnungs System")
        #
        # print(res.isAnyMessage())
        #print(PrettyText.lst2strC(res2.getWarnLst(short=True)))
        # quit()
        # print("Sys:  ", res2.getErrSysNo())
        # print("Kat:  ", res2.getErrKatNo())
        # print("Kat u:", res2.getErrKatNo(uniq=True))
        # print("Msg:  ",res2.getErrMsgNo())
        # print("Msg u:", res2.getErrMsgNo(uniq=True))
        # print("Add  :", res2.getErrMsgAddNo())
        #
        # print("---")
        # for sSys in res2.getErrSysLst():
        #     print("*",sSys, res2.getErrKatNo(sys=sSys))
        #     for sKat in res2.getErrKatLst(sSys):
        #         print("-",sKat, res2.getErrMsgNo(sys=sSys, kat=sKat))
        #         for sMsg in res2.getErrMsgLst(sSys, sKat):
        #             print(sMsg, res2.getErrMsgAddNo(sys=sSys, kat=sKat, msg=sMsg))
        # #quit()
        # print("---")
        # print(res2.getErr())
        # quit()




        # res2.addErr(sErr1, kat=sKat1, col="name", sheet="sample")
        # res2.addErr(sErr1, kat=sKat1, col="name", sheet="sample")
        # res2.addErr(sErr1, kat=sKat1, col="name", sheet="sample")
        # res2.addErr(sErr1, kat=sKat1, col="name", sheet="station")
        # res2.addErr(sErr1, kat=sKat1, row=8, col="name", sheet="sample")
        # res2.addErr(sErr1, kat=sKat1, row=10, col="name", sheet="sample")
        # res2.addErr(sErr1, kat=sKat1, row=8, col="name", sheet="station")
        # res2.addErr(sErr1, kat=sKat1, row=9, col="name", sheet="station")
        # res2.addErr(sErr1, kat=sKat1, row=8, col="name", sheet="cruise")
        #
        # res.addErr(sErr2, kat=sKat1, row=2, sheet="cruise", col="possys")
        # res.addErr(sErr2, kat=sKat1, row=1, sheet="cruise", col="possys")
        #
        # res.addErr(sErr3, kat=sKat1, sheet="sample", row="17", add="-10")
        # res.addErr(sErr3, kat=sKat1, sheet="sample", row="18", add="-6")
        # res.addErr(sErr3, kat=sKat1, sheet="cruise", row="18", add="-1")
        #
        # res.addErr(sErr3, kat=sKat1, add="-9")
        # res.addErr(sErr3, kat=sKat1, add="-2")
        # res.addErr(sErr3, kat=sKat1, add="-2")
        # res.addErr(sErr3, kat=sKat1, add="-22")
        #
        # res.addErr(sErr3, kat=sKat1, sheet="sample", add="-9")
        # res.addErr(sErr3, kat=sKat1, sheet="sample", add="-2")
        # res.addErr(sErr3, kat=sKat1, sheet="sample", add="-2")
        # res.addErr(sErr3, kat=sKat1, sheet="station", add="-22")
        #
        # res.addErr(sErr3, kat=sKat1, col="name", sheet="sample", add="-9")
        # res.addErr(sErr3, kat=sKat1, col="name", sheet="sample", add="-2")
        # res.addErr(sErr3, kat=sKat1, col="name", sheet="sample", add="-2")
        # res.addErr(sErr3, kat=sKat1, col="name", sheet="station", add="-22")
        #
        # res.addErr(sErr3, kat=sKat1, row=10, sheet="station", col="person")
        # res.addErr(sErr3, kat=sKat1, row=11, sheet="station", col="person")
        # res.addErr(sErr3, kat=sKat1, row=12, sheet="station", col="person")
        #
        # res.addErr(sErr3, kat=sKat1, row=10, sheet="station", col="person", add="-2")
        # res.addErr(sErr3, kat=sKat1, row=11, sheet="station", col="person", add="-2")
        # res.addErr(sErr3, kat=sKat1, row=12, sheet="station", col="person", add="Jürgen")
        #
        # res3.addErr(sErr4, kat=sKat2, row=10, sheet="biota", col="aid")
        # res3.addErr(sErr4, kat=sKat2, row=11, sheet="biota", col="aid")
        # res3.addErr(sErr4, kat=sKat2, row=12, sheet="biota", col="aid")
        # res3.addErr(sErr4, kat=sKat2, row=13, sheet="biota", col="aid")
        #
        # res3.addWarn(sWarn, kat="Lasche Warnung")
        # res3.addInfo(sInfo, kat="Kann man beachten")
        # res3.addData("name", "Paul")
        #
        # res4.addResult(res)
        # res4.addResult(res2)
        # res4.addResult(res3)
        #
        # #print(res4.getInfoMsgNo())
        #
        # # print(res)
        # # print(res2)
        # # print(res3)
        # print(res4.getErrWarnInfoStringHeader())
        # print(res4.getErrWarnInfoString())
        # #print(res2.getErrWarnInfoStringFooter())

        # res = m.downloadUpdateApp()
        # if res.isErr():
        #     print(res)

        # cursor = m.getDb().getCursor(sConn)
        # print(m.getDb().getVersionConnectedDatenmodell(cursor, "marlin_science"))

        #m.evalPgTools()
        #geoCrunch = GeoCruncher()
        #sGeoIn = "49° 44.0\' W"
        #sGeoIn = "180"
        #res = geoCrunch.crunch(sGeoIn, "lon")
        #print(res)
        # if res.isErr():
        #     print(geoCrunch.getInfo())

        #res = m.getDb().checkLeafExists("HE0263", "0188", 1)
        #res = m.getDb().getIngestDependencies()
        #print(res)
        #exit()

        # FileTools.prepareSaveFile("~/_temp/test.txt")
        # quit()
        # m.getImporterExporter().saveImportTemplate("bsh","/scratch/users/pkloss/_temp/empty_bsh.xlsx")
        # quit()
        # dicInput = {
        #     # "date_start": {datetime.date(1900, 1, 1): True},
        #     # "date_end": {datetime.date(2000, 1, 1): True},
        #     #"gear": {"video": True, "grab": True, "trawl": False}
        #     "ingest": {"20": True}
        # }
        # sel = m.getSelector()
        # sel.reset()
        # sel.deactivateAllEntries("ingest")
        # sel.update(dicInput)
        # if sel.isErr():
        #     print("* Error Selector")
        #     print(sel.getErr())
        # m.setDebug(False)
        #print(sel.infoConfigShort())
        #res = m.getImporterExporter().startExport("bsh", "/scratch/users/pkloss/_temp/ms_out.xlsx")

        # res = m.getImporterExporter().startImport("bsh", "/scratch/users/pkloss/_temp/bsh_in.xlsx", "/scratch/users/pkloss/_temp/ms_out.xlsx")
        # if res.isErr():
        #     print("* Error Exporter")
        #     print(res.getErr())
        #     quit()



        # dic = {'name': '1', 'lon_start': '-12.718333333333334', 'lat_end': '-71.1', 'lat_start': '-71.10166666666667', 'lon_end': '-12.72', 'depth_start': '576', 'date_start': '2000-04-06', 'gear_id': '74', 'station_id': '13281', 'id': '20084', 'depth_end': '587'}
        #
        # cursor = m.getDb().getCursor()
        # res = m.getDb().checkRecordExistsInTable(cursor, "sample","id", dic, True)
        # print("search:      ", dic)
        # print("answer-full: ", res.getData("answer-full"))
        # print("answer:      ", res.getData("answer"))
        # print("found:       ", res.getData("found"))
        # print("equal:       ", res.getData("equal"))
        # print("diff:        ", res.getData("diff"))
        # sFile = "/Users/paul/_temp"
        # lstTabs = None
        # m.getComments().update()
        # res = m.getExporter().__addExcel_subset(None, 11)
        # if res.isErr():
        #     print("Error:", res.getErr())
        #     exit(1)
        # else:
        #     odicAnt = res.getData("data")
        #     print("*** Overview")
        #     for sSheet, dicSheet in odicAnt.items():
        #         if sSheet != "Selection":
        #             print(sSheet, ":", dicSheet["rows"])
        #     print("*** SQL")
        #     for sSheet, dicSheet in odicAnt.items():
        #         print("---",sSheet)
        #         print(dicSheet["sql"])
        #     print("*** Tables")
        #     for sSheet, dicSheet in odicAnt.items():
        #         if sSheet != "Selection":
        #             print("---",sSheet)
        #             print(PrettyText.table([dicSheet["header"]]+dicSheet["content"], True))
        # Excel
        # try:
        #     sFile = "/Users/paul/_temp/test2.xlsx"
        #     warnings.simplefilter("ignore")  # Ignoriere Warnungen
        #     wb = openpyxl.load_workbook(sFile, data_only=True)  # Keine Formeln sondern letzter gespeicherter Wert
        #     warnings.simplefilter("default")  # Warnungen wieder aktivieren
        # except Exception as ex:
        #     print(ex)
        #     quit()
        # wbNav = ExcelNavigator(wb)
        # print(wbNav)
        # quit()
    else:
        # Nichts ausgewaehlt
        print("Nothing to do ...")
