--
-- PostgreSQL database dump
--

-- Dumped from database version 14.8
-- Dumped by pg_dump version 14.8

SET statement_timeout = 0;
SET lock_timeout = 0;
SET idle_in_transaction_session_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = on;
SELECT pg_catalog.set_config('search_path', '', false);
SET check_function_bodies = false;
SET xmloption = content;
SET client_min_messages = warning;
SET row_security = off;

--
-- Name: cb_empty; Type: SCHEMA; Schema: -; Owner: -
--

CREATE SCHEMA cb_empty;


--
-- Name: antwort; Type: TYPE; Schema: cb_empty; Owner: -
--

CREATE TYPE cb_empty.antwort AS (
	err text,
	warn text,
	info text
);


--
-- Name: is_sample_ok(text, integer, boolean); Type: FUNCTION; Schema: cb_empty; Owner: -
--

CREATE FUNCTION cb_empty.is_sample_ok(s_schema text DEFAULT NULL::text, i_sample_id integer DEFAULT NULL::integer, b_debug boolean DEFAULT false) RETURNS text[]
    LANGUAGE plpgsql
    AS $$DECLARE
	f_replicates float; -- Ausgerechnete Replikatanzahl
	f_gsa float; -- GSA: Gear Sampled Area wurde ausgerechnet
	b_gsa boolean; -- GSA: Gear Sampled Area ist berechenbar, wurde aber nicht ausgerechnet
	b_schema_existing boolean; 
	rec_sample record; -- Sample
	rec_gear record; -- Gear
	arr_antwort text ARRAY[3]; -- Antwort String-Array:0:Error/1:Warnung/2:Info
BEGIN	
	
	b_gsa = false;
	
	-- ----------------------------------------------------------------------
	-- HIER WIRD NUR AUF DIREKTE, DEN PROZESS UNTERBRECHENDE FEHLER GETESTET!
	-- WELCHES DATENSZENARIO VORLIEGT WIRD NICHT GETESTET
	-------------------------------------------------------------------------
	
	-- Teste ob Schema existiert
	EXECUTE format('SELECT TRUE FROM information_schema.schemata WHERE schema_name=%L', s_schema) INTO b_schema_existing;
	IF b_schema_existing is null THEN
		arr_antwort[0] = 'Schema not existing: ' ||  s_schema;
		return arr_antwort ;
	END IF;

	-- Hole das gesuchte Sample
	EXECUTE format('SELECT * from %s.sample smp WHERE smp.id=%s',  s_schema, i_sample_id) INTO rec_sample;
	IF rec_sample is null THEN
		arr_antwort[0] = 'Sample not existing: i_sample_id=' || i_sample_id;
		return arr_antwort;
	END IF;

	-- Hole gear data
	EXECUTE format('SELECT name, category, width, area FROM %s.gear gea WHERE gea.id=%s', s_schema, rec_sample.gear_id) INTO rec_gear;
	
	-- ------------------------------------------------------------------------------------------
	-- CHECK GRAB_PENETRATION_DEPTH: Teste ob bei gegebener das Gear auch wirklich ein Grab ist 
	-- ------------------------------------------------------------------------------------------
	
	IF rec_sample.grab_penetration_depth is not NULL and rec_gear.category <> 'grab' THEN
		arr_antwort[0] = 'Grab penetration depth is set (' || rec_sample.grab_penetration_depth || ') and gear ('|| rec_gear.name ||') type is not grab ';
		return arr_antwort;
	END IF;
	
	-- ------------------------------------------------------------------------------------------
	-- CHECK AREA: Teste ob die Gear Area Eingabe konsistent ist (keine Eingabe ist auch korrekt)
	-- ------------------------------------------------------------------------------------------
	
	-- ------------------------------------
	-- Eingangstests (fuer alle Gear-Typen)
	-- ------------------------------------
	
	-- sample.sampled_area und sample.reference_area duerfen nicht gleichzeitig gesetzt sein
	IF rec_sample.sampled_area is not NULL and rec_sample.reference_area is not NULL THEN
		arr_antwort[0] = 'Sampling area SA (' || rec_sample.sampled_area || ') and reference area RA ('|| rec_sample.reference_area ||') are set simultaniously';
		return arr_antwort;
	END IF;
	
	-- -------------------------------------
	-- Haupttests (getrennt nach Gear-Typen)
	-- -------------------------------------
	
	IF rec_gear.category='trawl' THEN
		-- *** Trawl
		-- Sampling Distance darf nicht mehrfach berechenbar sein: sample.sampling_distance <=> sample.sampling_speed + sample.time_start/end) 
		IF rec_sample.sampling_distance is not NULL and (rec_sample.sampling_speed is not NULL and rec_sample.time_start is not NULL and rec_sample.time_end is not NULL) THEN
			arr_antwort[0] = 'Trawling distance could be calculated twice (sampling_distance and sampling_speed with time_start/end are set together)';
			return arr_antwort;
		END IF;
		-- Teste ob GSA berechenbar ist 
		if (rec_sample.sampling_distance is not NULL) or
			(rec_sample.sampling_speed is not NULL and rec_sample.time_start is not NULL and rec_sample.time_end is not NULL) or 
			(rec_sample.lat_start is not NULL and rec_sample.lat_end is not NULL and rec_sample.lon_start is not NULL and rec_sample.lon_end is not NULL) and
			rec_gear.width is not NULL THEN
				-- Ja... GSA ist berechenbar
				b_gsa = true;
		END IF;
	ELSE
		-- ### Grab, Sensor, Video, Scrape, Rocks (Rest)
		-- Area ist in Gear-Tabelle definiert?
		IF (rec_gear.area is not NULL and rec_gear.area <> 0) THEN
			-- Ja... GSA ist berechenbar und wird berechnet
			f_gsa = rec_gear.area * rec_sample.replicates; 
			b_gsa = true;
			-- Teste ob GSA von expliziter Angabe SA (sample.sampled_area) abweicht
			IF (rec_sample.sampled_area is not NULL and rec_sample.sampled_area <> f_gsa) THEN
				arr_antwort[0] = 'Gear (' || rec_gear.name || ') sampling area GSA (sample.replicates*gear.area -> '|| rec_sample.replicates || '*' || rec_gear.area || '=' || f_gsa ||') differing from given sampled area SA in sample (' || rec_sample.sampled_area || ')';
				return arr_antwort; 
			END IF;
		END IF;
	END IF;
	
	-- -------------------------------------
	-- Abschlussbericht
	-- -------------------------------------
	-- Debug Info 
	IF b_debug is true THEN
		RAISE NOTICE '%','* Information: area sample';
		RAISE NOTICE 'schema: %', s_schema;
		RAISE NOTICE 'sample.id: %', i_sample_id;
		RAISE NOTICE 'sample.sampled_area SA: %', rec_sample.sampled_area;
		RAISE NOTICE 'sample.reference_area RA: %', rec_sample.reference_area;
		RAISE NOTICE 'sample.lat_start: %', rec_sample.lat_start;
		RAISE NOTICE 'sample.lon_start: %', rec_sample.lon_start;
		RAISE NOTICE 'sample.lat_end: %', rec_sample.lat_end;
		RAISE NOTICE 'sample.lon_end: %', rec_sample.lon_end;
		RAISE NOTICE 'sample.time_start: %', rec_sample.time_start;
		RAISE NOTICE 'sample.time_end: %', rec_sample.time_end;
		RAISE NOTICE 'sample.sampling_distance: %', rec_sample.sampling_distance;
		RAISE NOTICE 'sample.sampling_speed: %', rec_sample.sampling_speed;
		RAISE NOTICE 'sample.replicates: %', rec_sample.replicates;
		RAISE NOTICE 'gear.name: %', rec_gear.name;
		RAISE NOTICE 'gear.category: %', rec_gear.category;
		RAISE NOTICE 'gear.width: %', rec_gear.width;
		RAISE NOTICE 'gear.area: %', rec_gear.area;
		IF b_GSA = true then
			RAISE NOTICE '%', 'GSA (calculated via gear): area is calcuable';
		ELSE
			RAISE NOTICE 'GSA (calculated via gear): %', f_gsa;
		END IF;
	END IF;
	
	return arr_antwort;
END;$$;


--
-- Name: is_subset_biota(text, integer, boolean); Type: FUNCTION; Schema: cb_empty; Owner: -
--

CREATE FUNCTION cb_empty.is_subset_biota(s_schema_name text DEFAULT NULL::text, i_subset_id integer DEFAULT NULL::integer, b_debug boolean DEFAULT false) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$declare
	b_schema_existing boolean; 
	b_subset_is_biota boolean;
	rec_subset record;
begin
	-- Der gesamte Test ist inzwischen 
	--- hinfaellig, da wir taxcover und taxres obligat gemacht haben (Grund: Probleme beim SQL-Selector-Statement für Export)
	-- CB ist eine Biotische Datenbank, da brauchen wir immer taxcover und taxres
	
	-- Test: alle Daten angegeben
	if s_schema_name is null or i_subset_id is null then
		raise exception 'missing data: schema=% subset=%', s_schema_name, i_subset_id;
	end if;

	-- Test: Schema existiert
	execute format('select true from information_schema.schemata where schema_name=%L', s_schema_name) into b_schema_existing;
	if b_schema_existing is null then
		raise exception 'schema not existing: %', s_schema_name;
	end if;
	
	-- Test: Subset biota existiert
	execute format('select taxcover_id,taxres from %1$s.subset where id=%2$L', s_schema_name, i_subset_id) into rec_subset;
	IF (rec_subset.taxcover_id is Null or rec_subset.taxres is Null) THEN
		b_subset_is_biota = false;
	ELSE
		b_subset_is_biota = true;
	END IF;
	
	-- Debug Info 
	if b_debug is true then
		raise notice '* Information: subset is biota';
		raise notice 'schema: %', s_schema_name;
		raise notice 'taxcover_id: %', rec_subset.taxcover_id;
		raise notice 'taxres: %', rec_subset.taxres;
		raise notice 'spatiallevel_bottom: %', rec_subset.spatiallevel_bottom;
		raise notice 'spatiallevel_top: %', rec_subset.spatiallevel_top;
		if b_subset_is_biota then 
			raise notice 'subset is biota: yes';
		else
			raise notice 'subset is biota: no';
		end if;
	end if;

	-- Antwort
	return b_subset_is_biota;
end;
$_$;


--
-- Name: is_subset_triplet_uniq(text, integer, integer, text, integer, integer, boolean); Type: FUNCTION; Schema: cb_empty; Owner: -
--

CREATE FUNCTION cb_empty.is_subset_triplet_uniq(s_schema_name text DEFAULT NULL::text, i_subsample_id integer DEFAULT NULL::integer, i_taxcover_id integer DEFAULT NULL::integer, s_taxres text DEFAULT NULL::text, i_spatiallevel_bottom integer DEFAULT NULL::integer, i_spatiallevel_top integer DEFAULT NULL::integer, b_debug boolean DEFAULT false) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$
declare
	b_schema_existing boolean; 
	i_count_triplets_overlap integer;
begin
	-- INFO: Das Triplet ist inzwischen ein Quadrupel geworden ;)
	
	-- Teste ob alle Daten angegeben wurden
	--if s_schema_name is null or i_subsample_id is null or i_taxcover_id is null or s_taxres is null or i_spatiallevel_bottom is null or i_spatiallevel_top is null then
	--	raise exception 'missing data: schema=% sample=% taxcover=% taxres=% spatiallevel_bottom=% spatiallevel_top=%', s_schema_name, i_subsample_id, i_taxcover_id, s_taxres, i_spatiallevel_bottom, i_spatiallevel_top;
	--end if;

	-- Teste ob Schema existiert
	execute format('SELECT TRUE FROM information_schema.schemata WHERE schema_name=%L', s_schema_name) into b_schema_existing;
	if b_schema_existing is null then
		raise exception 'schema not existing: %', s_schema_name;
	end if;
	
	-- Test Einmaligkeit
	-- neuer Bereich mit den Grenzen: top_val und bottom_val
	-- es sind nur zwei Varianten fuer alle anderen Bereiche, die bereits in der DB sind ok
	-- 1) spatiallevel_bottom<=top_val und sub.spatiallevel_top<=top_val
	-- und 
	-- 2) spatiallevel_bottom>=bottom_val und spatiallevel_top>=bottom_val
	-- d.h. alle anderen bereiche liegen links oder rechts vom neuen bereich
	-- die negation ergibt die anderen bereiche, die nicht erlaubt sind (schneiden sich mit neuem bereich)
	-- ausserdem muss gelten ...
	-- 3) in DB darf es kein bereits vorhandenes entsprechendes Subset geben mit spatiallevel_bottom und sub.spatiallevel_top nicht NULL sein 
	-- das würde ja bedeuten, dass der gesamte Bereich bereits belegt wäre
	-- Sonderfall) Wenn der neue Bereich NULL-NULL ist, d.h. der gesamte Bereich, darf dieser nur eingetragen werden wenn noch keine Daten in DB sind 

	-- Check: Sonderfall (NULL-NULL-Eintrag)
	if (i_spatiallevel_bottom is null and i_spatiallevel_top is null) then
		execute format('select count(id) from %1$s.subset sub where sub.subsample_id=%2$L and sub.taxcover_id=%3$L and sub.taxres=%4$L', s_schema_name, i_subsample_id, i_taxcover_id, s_taxres) into i_count_triplets_overlap; -- Hole alle Bereiche (das waeren dann alles ueberlappende Bereiche)
	else
		-- Check: 1-2-3 (Hole alle ueberlappenden Bereiche)
		execute format('select count(id) from %1$s.subset sub where sub.subsample_id=%2$L and sub.taxcover_id=%3$L and sub.taxres=%4$L and (not((sub.spatiallevel_bottom<=%6$L and sub.spatiallevel_top<=%6$L) or (sub.spatiallevel_bottom>=%5$L and sub.spatiallevel_top>=%5$L)) or (sub.spatiallevel_bottom is null and sub.spatiallevel_top is null))', s_schema_name, i_subsample_id, i_taxcover_id, s_taxres, i_spatiallevel_bottom, i_spatiallevel_top) into i_count_triplets_overlap;
	end if;
	
	-- Debug Info 
	if b_debug is true then
		raise notice '* Information: subset triplet uniq';
		raise notice 'schema: %', s_schema_name;
		raise notice 'subsample_id: %', i_subsample_id;
		raise notice 'taxcover_id: %', i_taxcover_id;
		raise notice 'taxres: %', s_taxres;
		raise notice 'spatiallevel_bottom: %', i_spatiallevel_top;
		raise notice 'spatiallevel_top: %', i_spatiallevel_bottom;
		raise notice 'found subsets_range_overlap: %', i_count_triplets_overlap;
	end if;

	-- Antwort
	if i_count_triplets_overlap = 0 then return true; else return false; end if;
end;
$_$;


--
-- Name: new_biota_check_consistency(); Type: FUNCTION; Schema: cb_empty; Owner: -
--

CREATE FUNCTION cb_empty.new_biota_check_consistency() RETURNS trigger
    LANGUAGE plpgsql
    AS $_$DECLARE
	
	-- Debug
	b_debug boolean; -- Debug
	
	-- Basis
	i_subset_id integer; -- Subset Id
	i_sample_id integer; -- Sample Id
	i_subsample_id integer; -- SubSample Id
	rec_subsample record; -- SubSample
	rec_sample record; -- Sample
	rec_gear record; -- Gear
	rec_branch record; -- Vollständiger Cruise-Station-Sample-Subset-Ast des Tests 
	
	-- BioDjin
	f_gsa float; -- GSA: Sampled Area berechnet aus Gear und evtl. anderen Quellen
	f_gsa_tmp float; -- GSA Temp
	b_gsa boolean; -- GSA: Sampled Area ist berechenbar aus Gear und evtl. anderen Quellen
	b_subset_is_biota boolean; -- Antwort des Subset-Biota-Test 
	s_szenario_grab text; -- Code Grab-Datenszenario
	
	-- Answer
	s_error text; -- Fehlertext
	s_warn text; -- Warnungstext
	s_branch text;
	s_branch_long text;
	
BEGIN

	b_debug = false;
	b_gsa = false;
	
	--------------------------------------------------------------------------------------------
	-- Vorarbeiten 
	--------------------------------------------------------------------------------------------
	
	-- Subset Id
	i_subset_id = NEW.subset_id;
	
	-- SubSample Id
	EXECUTE format('select subsample_id from %1$s.subset where id=%2$s;',TG_TABLE_SCHEMA, i_subset_id) into i_subsample_id;
	
	-- Hole SubSample
	EXECUTE format('select * from %1$s.subsample where id=%2$s;',TG_TABLE_SCHEMA, i_subsample_id) into rec_subsample;
	
	-- Sample Id
	EXECUTE format('select sample_id from %1$s.subsample where id=%2$s;',TG_TABLE_SCHEMA, i_subsample_id) into i_sample_id;
	
	-- Hole Sample
	EXECUTE format('select * from %1$s.sample where id=%2$s;',TG_TABLE_SCHEMA, i_sample_id) into rec_sample;
	
	-- Hole gear data
	EXECUTE format('SELECT name, category, width, area, smpunit FROM %s.gear gea WHERE gea.id=%s',  TG_TABLE_SCHEMA, rec_sample.gear_id) INTO rec_gear;

	-- Teste ob Subset Biotisches Subset ist (Tax-Cover/Resolution sind gesetzt) (ist nicht mehr nötig, da beides obligat ist)
	--EXECUTE format('select %1$s."is_subset_biota"(%1$L,%2$s,False)', TG_TABLE_SCHEMA, i_subset_id) INTO b_subset_is_biota;
	--IF b_subset_is_biota IS FALSE THEN
	--	s_error = 'You want to attach an biota to a subset that is not a biota subset';
	--END IF;
	
	-- Teste ob in number eine Fließkommazahl erlaubt ist (es muss eine Angabe in sample.subsample_share oder biota.subtotal_factor exisiteren)
	if NEW.number is not null and ((trunc(NEW.number) != NEW.number) and rec_sample.subsample_share is NULL and NEW.subtotal_factor is NULL) THEN
		s_error = 'Float in biota.number (' || NEW.number || ') is not allowed (missing data in sample.subsample_share or biota.subtotal_factor)';
	END IF;
		
	--------------------------------------------------------------------------------------------
	-- BioDjinn
	--------------------------------------------------------------------------------------------
			
	-- Bestimme Area (sampled_area, reference_area)
	IF rec_gear.category='trawl' then
		-- Trawl
		-- Teste ob Sampled Area berechenbar ist via Gear und Sample?
		if (rec_sample.sampling_distance is not NULL) or
			(rec_sample.sampling_speed is not NULL and rec_sample.time_start is not NULL and rec_sample.time_end is not NULL) or 
			(rec_sample.lat_start is not NULL and rec_sample.lat_end is not NULL and rec_sample.lon_start is not NULL and rec_sample.lon_end is not NULL) and
			rec_gear.width is not NULL THEN
			-- Sampled Area ist berechenbar - wird aber nicht berechnet
			b_gsa = true;
		END IF;
	ELSE
		-- Sensor/Trap/Grab/Video/Scrape/Rocks
		-- Hole Area aus Gear-Tabelle (gear.area * sample.replicates)
		IF (rec_gear.area is not NULL and rec_gear.area <> 0) THEN
				-- Sampled Area ist berechenbar und wird berechnet
			b_gsa = true;
			f_gsa = rec_gear.area * rec_sample.replicates;
			-- Teste ob Area aus Gear-Tabelle von sample.sampled_area abweicht
			IF (rec_sample.sampled_area is not NULL and rec_sample.sampled_area <> f_gsa) THEN
				s_error = 'BioDjinn says: Gear (' || rec_gear.name || ') sampling area (sample.replicates*gear.area -> '|| rec_sample.replicates || '*' || rec_gear.area || '=' || f_gsa ||') differing from given sampled area in sample (' || rec_sample.sampled_area || ')';
			END IF;
		END IF;
	END IF;
	
	-- Teste: ob Sampled/Reference-Area zu Standard Unit des Gears passt (both,area)
	if (rec_sample.sampled_area IS NOT NULL OR rec_sample.reference_area IS NOT NULL) AND rec_gear.smpunit NOT IN ('both','area') THEN
		s_error = 'BioDjinn says: Sampled/Reference-Area is given and gear standard unit is not set to area (it is set to: ' || rec_gear.smpunit || ')';
	END IF;
	
	-- Teste: ob Sampled/Reference-Vol zu Standard Unit des Gears passt (both,vol)
	if (rec_sample.sampled_vol is not NULL OR rec_sample.reference_vol is not null) AND rec_gear.smpunit not in ('both','vol') THEN
		s_error = 'BioDjinn says: Sampled/Reference-Volume is given and gear standard unit is not set to volume (it is set to: ' || rec_gear.smpunit || ')';
	END IF;
	
	-- Teste: SampledArea/Vol oder ReferenceArea/Vol und Gear Standard Unit = both -> Immer Warnung
	--if (rec_sample.sampled_area is NOT NULL or rec_sample.sampled_vol is NOT NULL) and rec_gear.smpunit = 'both' THEN
	--	s_warn = 'BioDjinn says: SampledArea/Volume is given and gear standard unit is set to both - is that right?';
	--END IF;
	--if (rec_sample.reference_area is NOT NULL or rec_sample.reference_vol is NOT NULL) AND rec_gear.smpunit = 'both' THEN
	--	s_warn = 'BioDjinn says: ReferenceArea/Volume is given and gear standard unit is set to both - is that right?';
	--END IF;
	
	-- Test: Mutual Exclusion: Number/Abundanz - PresenceAbsence
	IF NEW.number IS NOT NULL and NEW.presence_absence IS NOT NULL THEN
		s_error = 'BioDjinn says: Number (' || NEW.number || ') and Presence/Absence ('|| NEW.presence_absence ||') must not be set at the same time';
	END IF;
	IF NEW.abundance IS NOT NULL and NEW.presence_absence IS NOT NULL THEN
		s_error = 'BioDjinn says: Abundance (' || NEW.abundance || ') and Presence/Absence ('|| NEW.presence_absence ||') must not be set at the same time';
	END IF;

	-- Teste: Daten-Szenario
	-- 0a (1 Sample, 1 Replicate, Sampled Area berechenbar durch replicate * gear.area oder direkt durch sample.sampled_area oder Sampled Volume gegeben in sample.sampled_vol)
	IF ((rec_sample.sampled_area is NOT NULL or f_gsa is not null) OR rec_sample.sampled_vol is not NULL) AND
		(rec_sample.reference_area is NULL AND rec_sample.reference_vol is NULL) AND
		NEW.number is NOT NULL AND
		NEW.abundance is NULL AND
		NEW.presence_absence is NULL AND
		rec_sample.replicates = 1 THEN
			NEW.abundance = NULL; -- Verwerfen -> Kann aus SA und N berechnet werden
			s_szenario_grab = '0a';
	-- 0b (1 Sample, n Replicate, Sampled Area (in Summe) berechenbar durch replicate * gear.area oder direkt durch sample.sampled_area oder Sampled Volume gegeben in sample.sampled_vol)
	ELSIF ((rec_sample.sampled_area is NOT NULL or f_gsa is not null) or rec_sample.sampled_vol is not NULL) AND
		(rec_sample.reference_area is NULL AND rec_sample.reference_vol is NULL) AND
		NEW.number is NOT NULL AND
		NEW.abundance is NULL AND
		NEW.presence_absence is NULL AND
		rec_sample.replicates > 1 THEN
			NEW.abundance = NULL; -- Verwerfen -> Kann aus SA und N berechnet werden
			s_szenario_grab = '0b';
	-- 0c (1 Sample, ? Replikate, Reference Area/Vol, Abundance, Number ->  Sampled Area ist berechenbar (SA = (N*RA)/A) )
	ELSIF (rec_sample.sampled_area is NULL AND rec_sample.sampled_vol is NULL) AND 
		((rec_sample.reference_area is NOT NULL) OR (rec_sample.reference_vol is NOT NULL)) AND 
		NEW.number is NOT NULL AND 
		NEW.abundance is NOT NULL AND
		NEW.presence_absence is NULL THEN
			s_szenario_grab = '0c';			
			-- Teste: Fall 0c bei gegebener Reference Area, Gear!=Trawl, GSA berechenbar -> Gilt die Formel: GSA=(N*RA)/A
			-- Gear=Video/Sensor/Grab und GSA 
			IF (rec_sample.reference_area is NOT NULL) and (rec_gear.category <> 'trawl') and b_gsa is True THEN
				f_gsa = rec_gear.area * rec_sample.replicates;
				f_gsa_tmp = (NEW.number*rec_sample.reference_area)/NEW.abundance; 
				if f_gsa <> f_gsa_tmp THEN
					-- Problem: Vergleich zweier Floats -> das kann häufig falsch positiv zuschlagen.
					s_error = format('BioDjinn says: Problem testing gear calculated sampled_area GSA - equation GSA=(N*RA)/A does not apply: gear (%s),  gear category (%s), GSA (%s), (N*RA)/A (%s), number N (%s), abundance A (%s), reference_area RA (%s)', rec_gear.name, rec_gear.category, f_gsa, f_gsa_tmp, NEW.number, NEW.abundance, rec_sample.reference_area);
				END IF;
			END IF;	
	-- 1a (1 Sample, Abundance, Reference Area/Vol, keine Sampled Area/Vol, GSA (in unser Datenmodell reingequetscht)
	ELSIF (rec_sample.sampled_area is NULL  AND rec_sample.sampled_vol is NULL) AND 
		((rec_sample.reference_area is NOT NULL) OR (rec_sample.reference_vol is NOT NULL)) AND
		b_gsa AND
		NEW.number is NULL AND 
		NEW.abundance is NOT NULL AND
		NEW.presence_absence is NULL THEN
			s_szenario_grab = '1a';
	-- 1b (1 Sample, Abundance, Reference Area/Vol, keine Sampled Area, keine GSA (in unser Datenmodell reingequetscht)
	ELSIF (rec_sample.sampled_area is NULL  AND rec_sample.sampled_vol is NULL) AND 
		((rec_sample.reference_area is NOT NULL) OR (rec_sample.reference_vol is NOT NULL)) and 
		not b_gsa AND
		NEW.number is NULL AND 
		NEW.abundance is NOT NULL AND
		NEW.presence_absence is NULL THEN
		s_szenario_grab = '1b';
	-- 2 (1 Sample, Number)
	ELSIF rec_sample.sampled_area is NULL  AND rec_sample.sampled_vol is NULL AND 
		rec_sample.reference_area is NULL AND rec_sample.reference_vol is NULL and 
		NEW.number is NOT NULL and 
		NEW.abundance is NULL AND
		NEW.presence_absence is NULL THEN
			s_szenario_grab = '2';
	-- 3 (1 Sample, PresenceAbsence, Sampled Area/Vol)
	ELSIF ((rec_sample.sampled_area is NOT NULL or f_gsa is NOT NULL) OR (rec_sample.sampled_vol is not NULL)) AND 
		(rec_sample.reference_area is NULL AND rec_sample.reference_vol is NULL) and 
		NEW.number is NULL AND 
		NEW.abundance is NULL AND
		NEW.presence_absence is NOT NULL THEN
			s_szenario_grab = '3';	
	-- 4 (1 Sample, PresenceAbsence)
	ELSIF (rec_sample.sampled_area is NULL AND rec_sample.sampled_vol is NULL) AND 
		NEW.number is NULL and 
		NEW.presence_absence is NOT NULL THEN
			s_szenario_grab = '4';
	-- Kein Szenario erkannt -> Fehler
	ELSE
		if b_gsa = true then
			-- Sampled Area ist berechenbar (Trawl) wurde aber nicht ausgerechent
			s_error = format('BioDjinn says: Could not determine data szenario: replicates R (%s), number N (%s), abundance A (%s), gear calculated sampled_area GSA (is calcuable), sampled_area SA (%s), reference_area RA (%s) and presence/absence PA (%s)', 
							 rec_sample.replicates, NEW.number, NEW.abundance, rec_sample.sampled_area, rec_sample.reference_area, NEW.presence_absence);
		ELSE
			-- Sampled Area ist berechenbar und wurde bereits ausgerechnet
			s_error = format('BioDjinn says: Could not determine biota data szenario: replicates R (%s), number N (%s), abundance A (%s), gear calculated sampled_area GSA (%s), sampled_area SA (%s), reference_area RA (%s) and presence/absence PA (%s)', 
							 rec_sample.replicates, NEW.number, NEW.abundance, f_gsa, rec_sample.sampled_area, rec_sample.reference_area, NEW.presence_absence);
		END IF;
	END IF;
	
	--------------------------------------------------------------------------------------------
	-- Fehler 
	--------------------------------------------------------------------------------------------
	IF s_error is not NULL or s_warn is not NULL THEN
		-- Bestimme Ast für Meldungen
		EXECUTE format('select cru.name cruise, cru.ingest_id cruise_ing, sta.name station, sta.ingest_id station_ing, smp.name sample, smp.ingest_id sample_ing, subsmp.name subsample, subsmp.ingest_id subsample_ing, sub.name subset, sub.ingest_id subset_ing from %1$s.cruise cru, %1$s.station sta, %1$s.sample smp, %1$s.subsample subsmp, %1$s.subset sub where sta.cruise_id=cru.id and smp.station_id=sta.id and subsmp.sample_id=smp.id and sub.subsample_id=subsmp.id and sub.id=%2$s;',TG_TABLE_SCHEMA, i_subset_id) into rec_branch;
		s_branch_long = format('Cruise=%L (ingest=%s), Station=%L (ingest=%s), Sample=%L (ingest=%s), Subset=%L (ingest=%s)', rec_branch.cruise, rec_branch.cruise_ing, rec_branch.station, rec_branch.station_ing, rec_branch.sample, rec_branch.sample_ing, rec_branch.subset, rec_branch.subset_ing);
		s_branch = format('%s|%s|%s|%s|%s {%s}{%s}{%s}{%s}{%s}', rec_branch.cruise, rec_branch.station, rec_branch.sample, rec_branch.subsample, rec_branch.subset, rec_branch.cruise_ing, rec_branch.station_ing, rec_branch.sample_ing, rec_branch.subsample_ing, rec_branch.subset_ing);
	END IF;
	IF s_error is not NULL THEN
		-- Error
		RAISE EXCEPTION '%', s_error || ' -> ' || s_branch;
	END IF;
	IF s_warn is not NULL THEN
		-- Warnung
		RAISE WARNING '%', s_warn || ' -> ' || s_branch;
	END IF;
	
	--------------------------------------------------------------------------------------------
	-- Alles OK 
	--------------------------------------------------------------------------------------------
	-- Info
	IF b_debug THEN
		RAISE NOTICE '%', '* Information: new biota';
		RAISE NOTICE 'Gear category: %', rec_gear.category;
		RAISE NOTICE 'Gear name: %', rec_gear.name;
		RAISE NOTICE 'Number: %', NEW.number;
		RAISE NOTICE 'Abundance: %', NEW.abundance;
		RAISE NOTICE 'Presence/Absence: %', NEW.presence_absence;
		RAISE NOTICE 'Sample.replicates: %', rec_sample.replicates;
		-- Gear Sampled Area
		IF b_gsa=true THEN
			RAISE NOTICE '%', 'Gear Sampled Area GSA: is calcuable';
			IF f_gsa is not null then
				RAISE NOTICE 'Gear Sampled Area GSA: %', f_gsa;
			END IF;
		END IF;
		-- Sampled Area
		RAISE NOTICE 'Sampled Area SA: %', rec_sample.sampled_area;
		-- Reference Area
		RAISE NOTICE 'Reference area RA: %', rec_sample.reference_area;
		-- Data Szenario
		IF s_szenario_grab IS NOT NULL THEN
			RAISE NOTICE 'Data scenario type: %', s_szenario_grab;
		END IF;
	END IF; 	
	
	-- Antwort
	RETURN NEW;
			
END;$_$;


--
-- Name: new_gear_check_consistency(); Type: FUNCTION; Schema: cb_empty; Owner: -
--

CREATE FUNCTION cb_empty.new_gear_check_consistency() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
declare
	--
	 
begin
	-- Test: Sampling Unit
	if NEW.smpunit not in ('area', 'vol', 'both') then
		raise exception 'Unknown sampling unit: %', NEW.smpunit;
	end if;

	-- Test: Category
	if NEW.category not in ('grab', 'trawl', 'video', 'sensor', 'trap', 'scrape', 'rocks', 'dive', 'census', 'remote sensing') then
		raise exception 'Unknown gear category: %', NEW.category;
	end if;
	
	-- Test: Grab
	if NEW.category = 'grab' then
		IF NEW.meshsize is not null then
			raise exception 'Meshsize can not be set at gear with category: %', NEW.category;
		END IF;
	end if;
	
	-- Test: Trawl
	if NEW.category = 'trawl' then
		IF NEW.area is not null then
			raise exception '%', 'Area can not be set at gear with category: trawl';
		END IF;
	end if;
	
	-- Test: Census, Remote Sensing
	 if NEW.category in ('census', 'remote sensing') then
	 	IF NEW.meshsize is not null or NEW.length is not null or NEW.width is not null or NEW.height is not null or NEW.area is not null or NEW.mass is not null then
			raise exception 'Meshsize can not be set at gear with category: %', NEW.category;
		END IF;
	 end if;
	
	-- Test: Video
	if NEW.category = 'video' then
		IF NEW.meshsize is not null or NEW.length is not null or NEW.width is not null or NEW.height is not null then
			raise exception 'Meshsize can not be set at gear with category: %', NEW.category;
		END IF;
	end if;
	
	-- Test: Sensor
	if NEW.category = 'sensor' then
		IF NEW.meshsize is not null or NEW.length is not null or NEW.width is not null or NEW.height is not null then
			raise exception 'Meshsize can not be set at gear with category: %', NEW.category;
		END IF;
	end if;
	
	-- Test: Scrape
	if NEW.category = 'scrape' then
	end if;
	
	-- Test: Rocks
	if NEW.category = 'rocks' then
	end if;
	
	-- Test: Dive
	if NEW.category = 'dive' then
		IF NEW.meshsize is not null then
			raise exception 'Meshsize can not be set at gear with category: %', NEW.category;
		END IF;
	end if;
	
	-- Test: Trap
	if NEW.category = 'trap' then  
		IF NEW.meshsize is not null or NEW.length is not null or NEW.width is not null or NEW.height is not null then
			raise exception 'Meshsize can not be set at gear with category: %', NEW.category;
		END IF;
	end if;
	
	return NEW;
end;
$$;


--
-- Name: new_sample_check_consistency(); Type: FUNCTION; Schema: cb_empty; Owner: -
--

CREATE FUNCTION cb_empty.new_sample_check_consistency() RETURNS trigger
    LANGUAGE plpgsql
    AS $_$DECLARE
	arr_area_antwort text ARRAY[3]; -- Antwort des Area-Berechnbarkeitstest 
	rec_branch record; -- Vollständiger Cruise-Station-Sample-Ast des Tests 
	s_branch text; 
BEGIN
	-- Teste Area
	EXECUTE format('select %1$s."is_sample_ok"(%1$L, %2$L,%3$L)', TG_TABLE_SCHEMA, NEW.id, false) into arr_area_antwort;
	
	-- *** Alles lief gut 
	-- Area ist konsistent? (konsistent bestimmbar/nicht bestimmtt
	IF arr_area_antwort[0] is NULL THEN
		RETURN NEW;
	END IF;
	
	-- ** Es gab einen Fehler
	-- Bestimme Ast für Meldungen
	EXECUTE format('select cru.name cruise, cru.ingest_id cruise_ing, sta.name station, sta.ingest_id station_ing, smp.name sample, smp.ingest_id sample_ing from %1$s.cruise cru, %1$s.station sta, %1$s.sample smp where sta.cruise_id=cru.id and smp.station_id=sta.id and smp.id=%2$s;', TG_TABLE_SCHEMA, NEW.id) into rec_branch;
	s_branch = format('Cruise=%L (ingest=%s), Station=%L (ingest=%s), Sample=%L (ingest=%s)', rec_branch.cruise, rec_branch.cruise_ing, rec_branch.station, rec_branch.station_ing, rec_branch.sample, rec_branch.sample_ing);
	RAISE EXCEPTION 'Error while checking area: %', s_branch || ' - ' || arr_area_antwort[0];
	
END;$_$;


--
-- Name: new_subsample_check_consistency(); Type: FUNCTION; Schema: cb_empty; Owner: -
--

CREATE FUNCTION cb_empty.new_subsample_check_consistency() RETURNS trigger
    LANGUAGE plpgsql
    AS $_$DECLARE
	
	-- Basis
	rec_branch record; -- Vollständiger Cruise-Station-Sample-Subset-Ast des Tests 
	f_ratio_sum float;
	i_ratio_null integer;
	
	-- Answer
	s_error text; -- Fehlertext
	s_warn text; -- Warnungstext
	s_branch text;
	
BEGIN
	
	--------------------------------------------------------------------------------------------
	-- Test: Summe aller Ratios aller SubSamples pro Sample <= 1
	--------------------------------------------------------------------------------------------
	
	f_ratio_sum = 0;
	i_ratio_null = 0;
	
	-- Hole Summe aller Ratios aller SubSamples pro Sample
	EXECUTE format('select sum(ratio) from %1$s.subsample subsmp where subsmp.ratio is not NULL and subsmp.sample_id=%2$s',TG_TABLE_SCHEMA, NEW.sample_id) into f_ratio_sum;
	
	-- Sum Ratio > 1 -> Warnung	   
	IF f_ratio_sum > 1 THEN
		s_warn = format('Sum of all SubSample ratios per Sample >1: %1$s', f_ratio_sum);		
	END IF;
	
	-- Sum Ratio > 1 und es sind noch Ratios ohne Wert vorhanden -> Warnung
	IF f_ratio_sum = 1 THEN
		EXECUTE format('select count(id) from %1$s.subsample subsmp where subsmp.ratio is NULL and subsmp.sample_id=%2$s',TG_TABLE_SCHEMA, NEW.sample_id) into i_ratio_null;
		IF i_ratio_null > 0 THEN
			s_warn = format('Sum of all SubSample ratios per Sample = 1 and SubSamples with no ratio value are given: %1$sx', i_ratio_null);		
		END IF;
	END IF;
	
	--------------------------------------------------------------------------------------------
	-- Fehler/Warnung
	--------------------------------------------------------------------------------------------
	IF s_error is not NULL or s_warn is not NULL THEN
		-- Bestimme Ast für Meldungen
		EXECUTE format('select cru.name cruise, cru.ingest_id cruise_ing, sta.name station, sta.ingest_id station_ing, smp.name sample, smp.ingest_id sample_ing, subsmp.name subsample, subsmp.ingest_id subsample_ing from %1$s.cruise cru, %1$s.station sta, %1$s.sample smp, %1$s.subsample subsmp where sta.cruise_id=cru.id and smp.station_id=sta.id and subsmp.sample_id=smp.id and subsmp.id=%2$s;',TG_TABLE_SCHEMA, NEW.id) into rec_branch;
		s_branch = format('%s|%s|%s|%s {%s}{%s}{%s}{%s}', rec_branch.cruise, rec_branch.station, rec_branch.sample, rec_branch.subsample, rec_branch.cruise_ing, rec_branch.station_ing, rec_branch.sample_ing, rec_branch.subsample_ing);
	END IF;
	IF s_error is not NULL THEN
		-- Error
		RAISE EXCEPTION '%', s_error || ' -> ' || s_branch;
	END IF;
	IF s_warn is not NULL THEN
		-- Warnung
		RAISE WARNING '%', s_warn || ' -> ' || s_branch;
	END IF;
	
	-- Antwort
	RETURN NEW;
			
END;$_$;


--
-- Name: new_subset_check_consistency(); Type: FUNCTION; Schema: cb_empty; Owner: -
--

CREATE FUNCTION cb_empty.new_subset_check_consistency() RETURNS trigger
    LANGUAGE plpgsql
    AS $_$DECLARE
	-- Vollständiger Cruise-Station-Sample-SubSample-Subset-Ast des Tests
	rec_branch record; -- als record
	s_branch text; -- als string
	-- Triplett
	s_taxcover text;
	b_is_triplet_uniq boolean;
	-- Error Typen
	b_error boolean; 
	b_error_triplet_uniq boolean;
	
BEGIN
	-- *** Tests
	b_error=False;
	b_error_triplet_uniq=False;
	
	--- Test: Einmaligkeit Triplet pro Sample
	EXECUTE format('select %1$s.is_subset_triplet_uniq(%1$L, %2$s, %3$s, %4$L, %5$L, %6$L)', TG_TABLE_SCHEMA, NEW.subsample_id, NEW.taxcover_id, NEW.taxres, NEW.spatiallevel_bottom, NEW.spatiallevel_top) into b_is_triplet_uniq;

	IF b_is_triplet_uniq is false THEN
		b_error = True;
		b_error_triplet_uniq = True;
	END IF;
	
	-- *** Alles OK -> Zurück
	IF b_error is False THEN RETURN NEW; END IF;

	-- *** Es lief was schief
	-- Meldungen sind notwendig
	
	-- Bestimme Ast für Meldungen
	EXECUTE format('select cru.name cruise, sta.name station, smp.name sample, subsmp.name subsample 
				   from 
				   %1$s.cruise cru, 
				   %1$s.station sta, 
				   %1$s.sample smp, 
				   %1$s.subsample subsmp
				   where 
				   subsmp.id=%2$s and 
				   smp.id=subsmp.sample_id and 
				   sta.id=smp.station_id and 
				   cru.id=sta.cruise_id;',TG_TABLE_SCHEMA, NEW.subsample_id) into rec_branch;
	s_branch = format('%s|%s|%s|%s|%s', rec_branch.cruise, rec_branch.station, rec_branch.sample, rec_branch.subsample, NEW.name);

	-- Fehlermeldungen
	IF b_error_triplet_uniq is True THEN
		-- Triplet ist nicht uniq pro subsample
		RAISE EXCEPTION 'Subset: taxonomic coverage, taxonomic resolution, spatiallevel_top/bottom: Not uniq (hint: spatial level ranges might overlap and unset ranges are treated like maximum ranges): %', s_branch;
	END IF;
END;$_$;


SET default_tablespace = '';

SET default_table_access_method = heap;

--
-- Name: actionlog; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.actionlog (
    id integer NOT NULL,
    username text NOT NULL,
    action text NOT NULL,
    object text,
    date timestamp without time zone NOT NULL,
    host text NOT NULL,
    os text NOT NULL,
    log text,
    ip text NOT NULL
);


--
-- Name: actionlog_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.actionlog_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: actionlog_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.actionlog_id_seq OWNED BY cb_empty.actionlog.id;


--
-- Name: admin; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.admin (
    key text NOT NULL,
    value text NOT NULL,
    id integer NOT NULL
);


--
-- Name: admin_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.admin_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: admin_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.admin_id_seq OWNED BY cb_empty.admin.id;


--
-- Name: biogenic; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.biogenic (
    id integer NOT NULL,
    subset_id integer NOT NULL,
    ingest_id integer NOT NULL,
    c_org double precision,
    c_total double precision,
    loss_on_ignition double precision,
    nitrogen double precision,
    sulfur double precision,
    sed_afdw double precision,
    chl_a double precision,
    phaepigments double precision,
    fda double precision,
    protein double precision,
    lipids double precision,
    bacteria_cell_number integer,
    bacteria_cell_vol double precision,
    remark text
);


--
-- Name: biogenic_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.biogenic_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: biogenic_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.biogenic_id_seq OWNED BY cb_empty.biogenic.id;


--
-- Name: biota; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.biota (
    id integer NOT NULL,
    subset_id integer NOT NULL,
    taxon_id integer NOT NULL,
    lifestage_id integer,
    ingest_id integer NOT NULL,
    name text NOT NULL,
    number numeric,
    given_taxon_name text NOT NULL,
    biomass_wet double precision,
    biomass_dry double precision,
    biomass_afdm double precision,
    subtotal_factor text,
    remark text,
    specification text,
    abundance numeric,
    presence_absence integer,
    sieve integer,
    image text,
    CONSTRAINT biota_abundance_has_to_be_greater_or_equal_0 CHECK ((abundance >= (0)::numeric)),
    CONSTRAINT biota_presence_absence_has_to_be_0_or_1 CHECK ((presence_absence = ANY (ARRAY[0, 1]))),
    CONSTRAINT "number has to be: >=0" CHECK ((number >= (0)::numeric))
);


--
-- Name: biota_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.biota_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: biota_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.biota_id_seq OWNED BY cb_empty.biota.id;


--
-- Name: country; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.country (
    id integer NOT NULL,
    name text NOT NULL,
    gold boolean DEFAULT false NOT NULL
);


--
-- Name: country_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.country_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: country_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.country_id_seq OWNED BY cb_empty.country.id;


--
-- Name: crs; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.crs (
    id integer NOT NULL,
    name text DEFAULT 'WGS84'::character varying NOT NULL,
    srid integer,
    gold boolean DEFAULT false NOT NULL
);


--
-- Name: crs_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.crs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: crs_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.crs_id_seq OWNED BY cb_empty.crs.id;


--
-- Name: cruise; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.cruise (
    id integer NOT NULL,
    name text NOT NULL,
    ingest_id integer NOT NULL,
    crs_id integer NOT NULL,
    ship_id integer,
    positioningsystem_id integer,
    lead_person_id integer,
    date_start date,
    date_end date,
    region text,
    remark text
);


--
-- Name: cruise_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.cruise_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: cruise_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.cruise_id_seq OWNED BY cb_empty.cruise.id;


--
-- Name: dataset; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.dataset (
    id integer NOT NULL,
    name text NOT NULL,
    institution text NOT NULL,
    client text,
    reference_person text NOT NULL,
    owner_country_id integer NOT NULL,
    contact_person_id integer NOT NULL,
    realm text,
    doi text,
    description text,
    collection character(1) NOT NULL,
    access_level_data text DEFAULT 'confidential'::text NOT NULL,
    access_level_data_product text DEFAULT 'confidential'::text NOT NULL,
    gold boolean DEFAULT false NOT NULL,
    title text,
    abstract text,
    keywords text,
    version text NOT NULL,
    CONSTRAINT "dataset_rule_access_level_data has_to_be_confidential_free_or_r" CHECK (((access_level_data = 'confidential'::text) OR (access_level_data = 'free'::text) OR (access_level_data = 'request'::text))),
    CONSTRAINT dataset_rule_access_level_data_product_has_to_be_confidential_f CHECK ((access_level_data_product = ANY (ARRAY['confidential'::text, 'free'::text, 'request'::text]))),
    CONSTRAINT dataset_rule_collection_has_to_be_m_or_r CHECK (((collection = 'M'::character(1)) OR (collection = 'R'::character(1))))
);


--
-- Name: dataset_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.dataset_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: dataset_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.dataset_id_seq OWNED BY cb_empty.dataset.id;


--
-- Name: gear; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.gear (
    id integer NOT NULL,
    name text NOT NULL,
    type text,
    category text NOT NULL,
    width double precision,
    height double precision,
    length double precision,
    smpunit text NOT NULL,
    area double precision,
    mass double precision,
    meshsize double precision,
    description text,
    gold boolean DEFAULT false NOT NULL
);


--
-- Name: gear_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.gear_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: gear_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.gear_id_seq OWNED BY cb_empty.gear.id;


--
-- Name: imeasure_macro; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.imeasure_macro (
    id integer NOT NULL,
    biota_id integer NOT NULL,
    ingest_id integer NOT NULL,
    sex text,
    length_total double precision,
    length_st double precision,
    mass_total double precision,
    mass_sh double precision,
    remark text
);


--
-- Name: imeasure_meio; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.imeasure_meio (
    id integer NOT NULL,
    biota_id integer NOT NULL,
    ingest_id integer NOT NULL,
    sex text,
    length_total double precision,
    width double precision,
    lw_ratio double precision,
    mass_wet double precision,
    mass_dry double precision,
    mass_afdw double precision,
    trait_tail_shape text,
    trait_feeding_type text,
    trait_cp_val text,
    int_slide_no text,
    int_slide_ind_no text,
    image text,
    remark text
);


--
-- Name: imeasure_meio_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.imeasure_meio_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: imeasure_meio_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.imeasure_meio_id_seq OWNED BY cb_empty.imeasure_meio.id;


--
-- Name: indmeasure_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.indmeasure_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: indmeasure_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.indmeasure_id_seq OWNED BY cb_empty.imeasure_macro.id;


--
-- Name: ingest; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.ingest (
    id integer NOT NULL,
    name text DEFAULT 'anonymous'::character varying NOT NULL,
    created_on timestamp without time zone DEFAULT now() NOT NULL,
    workflow character varying NOT NULL,
    log character varying,
    rows integer,
    warn integer,
    info integer,
    file text
);


--
-- Name: ingest_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.ingest_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: ingest_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.ingest_id_seq OWNED BY cb_empty.ingest.id;


--
-- Name: lifestage; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.lifestage (
    id integer NOT NULL,
    name text NOT NULL,
    gold boolean DEFAULT false NOT NULL
);


--
-- Name: lifestage_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.lifestage_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: lifestage_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.lifestage_id_seq OWNED BY cb_empty.lifestage.id;


--
-- Name: noworms; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.noworms (
    id integer NOT NULL,
    scientificname text NOT NULL,
    target text
);


--
-- Name: noworms_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.noworms_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: noworms_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.noworms_id_seq OWNED BY cb_empty.noworms.id;


--
-- Name: person; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.person (
    id integer NOT NULL,
    name text NOT NULL,
    full_name text,
    affiliation text,
    email text,
    gold boolean DEFAULT false NOT NULL
);


--
-- Name: person_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.person_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: person_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.person_id_seq OWNED BY cb_empty.person.id;


--
-- Name: possys; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.possys (
    id integer NOT NULL,
    name text NOT NULL,
    gold boolean DEFAULT false NOT NULL
);


--
-- Name: positioningsystem_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.positioningsystem_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: positioningsystem_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.positioningsystem_id_seq OWNED BY cb_empty.possys.id;


--
-- Name: region; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.region (
    id integer NOT NULL,
    code character varying,
    name character varying,
    level character varying,
    geometry public.geometry
);


--
-- Name: region2sample; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.region2sample (
    sample_id integer NOT NULL,
    region_id integer NOT NULL
);


--
-- Name: region_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.region_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: region_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.region_id_seq OWNED BY cb_empty.region.id;


--
-- Name: sample; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.sample (
    id integer NOT NULL,
    name text NOT NULL,
    station_id integer NOT NULL,
    gear_id integer NOT NULL,
    ingest_id integer NOT NULL,
    responsible_person_id integer,
    sampled_area double precision,
    lon_start double precision NOT NULL,
    lat_start double precision NOT NULL,
    date_start date NOT NULL,
    time_start time without time zone,
    date_start_error integer DEFAULT 0,
    depth_start double precision,
    lon_end double precision,
    lat_end double precision,
    date_end date,
    time_end time without time zone,
    depth_end double precision,
    sample_mass real DEFAULT 0,
    subsample_mass real,
    subsample_share real,
    sampling_distance double precision,
    method_biomass text,
    method_biomass_dry text,
    method_biomass_afdm text,
    method_conservation text,
    temperature_surfacewater_start double precision,
    temperature_surfacewater_end double precision,
    salinity_surfacewater_start double precision,
    salinity_surfacewater_end double precision,
    remark text,
    reference_area double precision,
    replicates integer DEFAULT 1,
    sampling_speed double precision,
    sediment_properties text,
    sediment_smell text,
    grab_penetration_depth double precision,
    sampled_vol double precision,
    reference_vol double precision,
    "precision" integer DEFAULT 5 NOT NULL,
    CONSTRAINT "sample reference_area has to be: >0" CHECK (((reference_area <> NULL::double precision) AND (reference_area > (0)::double precision))),
    CONSTRAINT "sample replicates has to be: >0" CHECK ((replicates > 0)),
    CONSTRAINT "sample sampled_area hs to be: >0" CHECK (((sampled_area <> NULL::double precision) AND (sampled_area > (0)::double precision)))
);


--
-- Name: sample_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.sample_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: sample_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.sample_id_seq OWNED BY cb_empty.sample.id;


--
-- Name: sediment; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.sediment (
    id integer NOT NULL,
    subset_id integer NOT NULL,
    ingest_id integer NOT NULL,
    residue1 double precision,
    meshsize1 double precision,
    residue2 double precision,
    meshsize2 double precision,
    residue3 double precision,
    meshsize3 double precision,
    residue4 double precision,
    meshsize4 double precision,
    residue5 double precision,
    meshsize5 double precision,
    residue6 double precision,
    meshsize6 double precision,
    residue7 double precision,
    meshsize7 double precision,
    residue8 double precision,
    meshsize8 double precision,
    residue9 double precision,
    meshsize9 double precision,
    median double precision,
    gsd double precision,
    weight double precision,
    method_grainsize text,
    temperature double precision,
    oxygen_pd double precision,
    remark text,
    fuz_silt integer,
    fuz_vfsand integer,
    fuz_fsand integer,
    fuz_msand integer,
    fuz_csand integer,
    fuz_vcsand integer,
    fuz_gravel integer,
    fuz_stones integer,
    fuz_shell integer,
    fuz_clay integer,
    type text,
    water_content double precision,
    oxygen double precision
);


--
-- Name: sediment_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.sediment_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: sediment_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.sediment_id_seq OWNED BY cb_empty.sediment.id;


--
-- Name: ship; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.ship (
    id integer NOT NULL,
    name text NOT NULL,
    gold boolean DEFAULT false NOT NULL
);


--
-- Name: ship_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.ship_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: ship_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.ship_id_seq OWNED BY cb_empty.ship.id;


--
-- Name: station; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.station (
    id integer NOT NULL,
    name text NOT NULL,
    cruise_id integer NOT NULL,
    ingest_id integer NOT NULL,
    responsible_person_id integer,
    lon_start double precision,
    lat_start double precision,
    date_start date,
    time_start time without time zone,
    depth_start double precision,
    lon_end double precision,
    lat_end double precision,
    date_end date,
    time_end time without time zone,
    depth_end double precision,
    location text,
    lon_target double precision,
    lat_target double precision,
    replicates integer,
    temperature_air double precision,
    temperature_surfacewater double precision,
    temperature_bottomwater double precision,
    salinity_surfacewater double precision,
    salinity_bottomwater double precision,
    oxygen_bottomwater double precision,
    windspeed_beaufort double precision,
    wind_direction text,
    remark text,
    "precision" integer DEFAULT 5 NOT NULL
);


--
-- Name: station_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.station_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: station_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.station_id_seq OWNED BY cb_empty.station.id;


--
-- Name: subsample; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.subsample (
    id integer NOT NULL,
    ingest_id integer NOT NULL,
    sample_id integer NOT NULL,
    ratio double precision,
    name text NOT NULL
);


--
-- Name: subsample_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.subsample_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: subsample_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.subsample_id_seq OWNED BY cb_empty.subsample.id;


--
-- Name: subset; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.subset (
    id integer NOT NULL,
    name text NOT NULL,
    subsample_id integer NOT NULL,
    dataset_id integer NOT NULL,
    ingest_id integer NOT NULL,
    taxcover_id integer NOT NULL,
    taxres text NOT NULL,
    spatiallevel_bottom integer,
    spatiallevel_top integer,
    CONSTRAINT subset_rule_check_subset_taxonomic_resolution CHECK ((taxres = ANY (ARRAY['domain'::text, 'kingdom'::text, 'phylum'::text, 'class'::text, 'order'::text, 'family'::text, 'genus'::text, 'species'::text])))
);


--
-- Name: subset_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.subset_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: subset_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.subset_id_seq OWNED BY cb_empty.subset.id;


--
-- Name: taxcover; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.taxcover (
    id integer NOT NULL,
    name text NOT NULL,
    description text,
    gold boolean DEFAULT false NOT NULL
);


--
-- Name: taxcover_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.taxcover_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: taxcover_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.taxcover_id_seq OWNED BY cb_empty.taxcover.id;


--
-- Name: taxon; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.taxon (
    id integer NOT NULL,
    aid integer NOT NULL,
    scientificname text,
    valid_name text,
    aaid integer,
    aname text,
    status text,
    unacceptreason text,
    rank text,
    kingdom text,
    phylum text,
    class text,
    "order" text,
    family text,
    genus text,
    vaid integer,
    valid_authority text,
    citation text,
    url text,
    lsid text,
    ismarine integer,
    isbrackish integer,
    isfreshwater integer,
    isterrestrial integer,
    isextinct integer,
    is_colony boolean,
    authority text,
    match_type text,
    modified text
);


--
-- Name: taxon_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.taxon_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: taxon_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.taxon_id_seq OWNED BY cb_empty.taxon.id;


--
-- Name: tmpsmpnew; Type: VIEW; Schema: cb_empty; Owner: -
--

CREATE VIEW cb_empty.tmpsmpnew AS
 SELECT smp.id
   FROM cb_empty.sample smp
  WHERE (NOT (smp.id IN ( SELECT DISTINCT region2sample.sample_id
           FROM cb_empty.region2sample)));


--
-- Name: wormsaaid; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.wormsaaid (
    id integer NOT NULL,
    aid integer NOT NULL,
    scientificname text,
    accepted_scientificname text,
    aaid integer NOT NULL,
    aaid_scientificname text
);


--
-- Name: wormsaaid_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.wormsaaid_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: wormsaaid_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.wormsaaid_id_seq OWNED BY cb_empty.wormsaaid.id;


--
-- Name: wormsbase; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.wormsbase (
    id integer NOT NULL,
    aid integer NOT NULL,
    scientificname text,
    accepted_scientificname text,
    gold boolean DEFAULT false NOT NULL
);


--
-- Name: wormsbase_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.wormsbase_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: wormsbase_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.wormsbase_id_seq OWNED BY cb_empty.wormsbase.id;


--
-- Name: wormschange_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.wormschange_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: wormschange_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.wormschange_id_seq OWNED BY cb_empty.wormsaaid.aaid;


--
-- Name: wormschange; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.wormschange (
    id integer DEFAULT nextval('cb_empty.wormschange_id_seq'::regclass) NOT NULL,
    aid integer NOT NULL,
    scientificname text,
    accepted_scientificname text,
    treated text NOT NULL,
    workflow text,
    date_revision date,
    validity text,
    source text,
    comment text
);


--
-- Name: wormscolony; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.wormscolony (
    id integer NOT NULL,
    aid integer NOT NULL,
    scientificname text,
    accepted_scientificname text
);


--
-- Name: wormscolony_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.wormscolony_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: wormscolony_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.wormscolony_id_seq OWNED BY cb_empty.wormscolony.id;


--
-- Name: wormsprivate; Type: TABLE; Schema: cb_empty; Owner: -
--

CREATE TABLE cb_empty.wormsprivate (
    id integer NOT NULL,
    private_name text NOT NULL,
    aid integer NOT NULL,
    scientificname text,
    accepted_scientificname text
);


--
-- Name: wormsprivate_id_seq; Type: SEQUENCE; Schema: cb_empty; Owner: -
--

CREATE SEQUENCE cb_empty.wormsprivate_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: wormsprivate_id_seq; Type: SEQUENCE OWNED BY; Schema: cb_empty; Owner: -
--

ALTER SEQUENCE cb_empty.wormsprivate_id_seq OWNED BY cb_empty.wormsprivate.id;


--
-- Name: ws_subsets_view_noregions; Type: VIEW; Schema: cb_empty; Owner: -
--

CREATE VIEW cb_empty.ws_subsets_view_noregions AS
 SELECT smp.id,
    smp.name AS sample,
    smp.lat_start,
    smp.lon_start,
    (((smp.date_start || ' '::text) || COALESCE(smp.time_start, '00:00:00'::time without time zone)))::timestamp without time zone AS datetime,
    smp.date_start,
    smp.time_start,
    ssp.name AS subsample,
    cru.name AS cruise,
    cru.id AS cruise_id,
    sta.name AS station,
    sta.id AS station_id,
    sub.name AS subset,
    sub.id AS subset_id,
    sub.taxres AS taxonomic_resolution,
    dat.name AS dataset,
    dat.id AS dataset_id,
    smp."precision" AS spatial_precision,
    dat.access_level_data,
    dat.access_level_data_product,
    dat.institution,
    dat.reference_person,
    dat.doi,
    cov.name AS taxcover,
    cov.id AS taxcover_id,
    cov.name AS taxonomic_coverage,
    per.full_name AS contact_person,
    per.email AS contact_person_email,
    ctr.name AS owner_country,
    gea.name AS gear,
    gea.id AS gear_id,
    gea.category AS gear_category,
    ing.id AS ingest_id,
    ing.workflow AS ingest_workflow,
    ing.workflow AS type
   FROM ((((((((((cb_empty.subset sub
     LEFT JOIN cb_empty.subsample ssp ON ((ssp.id = sub.subsample_id)))
     LEFT JOIN cb_empty.sample smp ON ((smp.id = ssp.sample_id)))
     LEFT JOIN cb_empty.station sta ON ((sta.id = smp.station_id)))
     LEFT JOIN cb_empty.cruise cru ON ((cru.id = sta.cruise_id)))
     LEFT JOIN cb_empty.ingest ing ON ((ing.id = smp.ingest_id)))
     LEFT JOIN cb_empty.dataset dat ON ((dat.id = sub.dataset_id)))
     LEFT JOIN cb_empty.country ctr ON ((ctr.id = dat.owner_country_id)))
     LEFT JOIN cb_empty.gear gea ON ((gea.id = smp.gear_id)))
     LEFT JOIN cb_empty.person per ON ((per.id = dat.contact_person_id)))
     LEFT JOIN cb_empty.taxcover cov ON ((cov.id = sub.taxcover_id)))
  WHERE (dat.access_level_data = 'free'::text);


--
-- Name: ws_subsets_view; Type: MATERIALIZED VIEW; Schema: cb_empty; Owner: -
--

CREATE MATERIALIZED VIEW cb_empty.ws_subsets_view AS
 SELECT v.id,
    v.sample,
    v.lat_start,
    v.lon_start,
    v.datetime,
    v.date_start,
    v.time_start,
    v.subsample,
    v.cruise,
    v.cruise_id,
    v.station,
    v.station_id,
    v.subset,
    v.subset_id,
    v.taxonomic_resolution,
    v.dataset,
    v.dataset_id,
    v.spatial_precision,
    v.access_level_data,
    v.access_level_data_product,
    v.institution,
    v.reference_person,
    v.doi,
    v.taxcover,
    v.taxcover_id,
    v.taxonomic_coverage,
    v.contact_person,
    v.contact_person_email,
    v.owner_country,
    v.gear,
    v.gear_id,
    v.gear_category,
    v.ingest_id,
    v.ingest_workflow,
    v.type,
    ( SELECT reg.name AS region
           FROM (cb_empty.region reg
             LEFT JOIN cb_empty.region2sample reg2smp ON ((v.id = reg2smp.sample_id)))
          WHERE (reg2smp.region_id = reg.id)) AS region
   FROM cb_empty.ws_subsets_view_noregions v
  WITH NO DATA;


--
-- Name: ws_combinations_view; Type: MATERIALIZED VIEW; Schema: cb_empty; Owner: -
--

CREATE MATERIALIZED VIEW cb_empty.ws_combinations_view AS
 SELECT DISTINCT ws_subsets_view.ingest_workflow,
    ws_subsets_view.ingest_id,
    ws_subsets_view.gear_category,
    ws_subsets_view.gear_id,
    ws_subsets_view.dataset,
    ws_subsets_view.dataset_id,
    ws_subsets_view.taxcover,
    ws_subsets_view.taxcover_id,
    min(ws_subsets_view.date_start) AS min_date,
    max(ws_subsets_view.date_start) AS max_date,
    ws_subsets_view.region
   FROM cb_empty.ws_subsets_view
  GROUP BY ws_subsets_view.ingest_workflow, ws_subsets_view.ingest_id, ws_subsets_view.gear_category, ws_subsets_view.gear_id, ws_subsets_view.dataset, ws_subsets_view.dataset_id, ws_subsets_view.taxcover, ws_subsets_view.taxcover_id, ws_subsets_view.region
  WITH NO DATA;


--
-- Name: ws_statistics_view; Type: MATERIALIZED VIEW; Schema: cb_empty; Owner: -
--

CREATE MATERIALIZED VIEW cb_empty.ws_statistics_view AS
 SELECT 0 AS id,
    ( SELECT count(*) AS count
           FROM cb_empty.dataset) AS datasets,
    ( SELECT count(*) AS count
           FROM cb_empty.cruise) AS expeditions,
    ( SELECT count(*) AS count
           FROM cb_empty.station) AS stations,
    ( SELECT count(*) AS count
           FROM cb_empty.sample) AS samples,
    ( SELECT count(*) AS count
           FROM cb_empty.biota) AS biotas,
    ( SELECT count(*) AS count
           FROM cb_empty.sediment) AS sediments,
    ( SELECT count(DISTINCT biota.taxon_id) AS count
           FROM cb_empty.biota) AS taxa
  WITH NO DATA;


--
-- Name: actionlog id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.actionlog ALTER COLUMN id SET DEFAULT nextval('cb_empty.actionlog_id_seq'::regclass);


--
-- Name: admin id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.admin ALTER COLUMN id SET DEFAULT nextval('cb_empty.admin_id_seq'::regclass);


--
-- Name: biogenic id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.biogenic ALTER COLUMN id SET DEFAULT nextval('cb_empty.biogenic_id_seq'::regclass);


--
-- Name: biota id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.biota ALTER COLUMN id SET DEFAULT nextval('cb_empty.biota_id_seq'::regclass);


--
-- Name: country id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.country ALTER COLUMN id SET DEFAULT nextval('cb_empty.country_id_seq'::regclass);


--
-- Name: crs id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.crs ALTER COLUMN id SET DEFAULT nextval('cb_empty.crs_id_seq'::regclass);


--
-- Name: cruise id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.cruise ALTER COLUMN id SET DEFAULT nextval('cb_empty.cruise_id_seq'::regclass);


--
-- Name: dataset id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.dataset ALTER COLUMN id SET DEFAULT nextval('cb_empty.dataset_id_seq'::regclass);


--
-- Name: gear id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.gear ALTER COLUMN id SET DEFAULT nextval('cb_empty.gear_id_seq'::regclass);


--
-- Name: imeasure_macro id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.imeasure_macro ALTER COLUMN id SET DEFAULT nextval('cb_empty.indmeasure_id_seq'::regclass);


--
-- Name: imeasure_meio id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.imeasure_meio ALTER COLUMN id SET DEFAULT nextval('cb_empty.imeasure_meio_id_seq'::regclass);


--
-- Name: ingest id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.ingest ALTER COLUMN id SET DEFAULT nextval('cb_empty.ingest_id_seq'::regclass);


--
-- Name: lifestage id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.lifestage ALTER COLUMN id SET DEFAULT nextval('cb_empty.lifestage_id_seq'::regclass);


--
-- Name: noworms id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.noworms ALTER COLUMN id SET DEFAULT nextval('cb_empty.noworms_id_seq'::regclass);


--
-- Name: person id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.person ALTER COLUMN id SET DEFAULT nextval('cb_empty.person_id_seq'::regclass);


--
-- Name: possys id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.possys ALTER COLUMN id SET DEFAULT nextval('cb_empty.positioningsystem_id_seq'::regclass);


--
-- Name: region id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.region ALTER COLUMN id SET DEFAULT nextval('cb_empty.region_id_seq'::regclass);


--
-- Name: sample id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.sample ALTER COLUMN id SET DEFAULT nextval('cb_empty.sample_id_seq'::regclass);


--
-- Name: sediment id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.sediment ALTER COLUMN id SET DEFAULT nextval('cb_empty.sediment_id_seq'::regclass);


--
-- Name: ship id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.ship ALTER COLUMN id SET DEFAULT nextval('cb_empty.ship_id_seq'::regclass);


--
-- Name: station id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.station ALTER COLUMN id SET DEFAULT nextval('cb_empty.station_id_seq'::regclass);


--
-- Name: subsample id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.subsample ALTER COLUMN id SET DEFAULT nextval('cb_empty.subsample_id_seq'::regclass);


--
-- Name: subset id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.subset ALTER COLUMN id SET DEFAULT nextval('cb_empty.subset_id_seq'::regclass);


--
-- Name: taxcover id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.taxcover ALTER COLUMN id SET DEFAULT nextval('cb_empty.taxcover_id_seq'::regclass);


--
-- Name: taxon id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.taxon ALTER COLUMN id SET DEFAULT nextval('cb_empty.taxon_id_seq'::regclass);


--
-- Name: wormsaaid id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.wormsaaid ALTER COLUMN id SET DEFAULT nextval('cb_empty.wormsaaid_id_seq'::regclass);


--
-- Name: wormsaaid aaid; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.wormsaaid ALTER COLUMN aaid SET DEFAULT nextval('cb_empty.wormschange_id_seq'::regclass);


--
-- Name: wormsbase id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.wormsbase ALTER COLUMN id SET DEFAULT nextval('cb_empty.wormsbase_id_seq'::regclass);


--
-- Name: wormscolony id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.wormscolony ALTER COLUMN id SET DEFAULT nextval('cb_empty.wormscolony_id_seq'::regclass);


--
-- Name: wormsprivate id; Type: DEFAULT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.wormsprivate ALTER COLUMN id SET DEFAULT nextval('cb_empty.wormsprivate_id_seq'::regclass);


--
-- Name: actionlog actionlog_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.actionlog
    ADD CONSTRAINT actionlog_id_pk PRIMARY KEY (id);


--
-- Name: admin admin_rule_key_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.admin
    ADD CONSTRAINT admin_rule_key_uniq UNIQUE (key);


--
-- Name: biogenic biogenic_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.biogenic
    ADD CONSTRAINT biogenic_id_pk PRIMARY KEY (id);


--
-- Name: biota biota_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.biota
    ADD CONSTRAINT biota_id_pk PRIMARY KEY (id);


--
-- Name: country country_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.country
    ADD CONSTRAINT country_id_pk PRIMARY KEY (id);


--
-- Name: country country_rule_name_has_to_be_unique; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.country
    ADD CONSTRAINT country_rule_name_has_to_be_unique UNIQUE (name);


--
-- Name: crs crs_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.crs
    ADD CONSTRAINT crs_id_pk PRIMARY KEY (id);


--
-- Name: crs crs_rule_name_has_to_be_unique; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.crs
    ADD CONSTRAINT crs_rule_name_has_to_be_unique UNIQUE (name);


--
-- Name: cruise cruise_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.cruise
    ADD CONSTRAINT cruise_id_pk PRIMARY KEY (id);


--
-- Name: cruise cruise_name_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.cruise
    ADD CONSTRAINT cruise_name_uniq UNIQUE (name);


--
-- Name: sample cruise_station_sample_name_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.sample
    ADD CONSTRAINT cruise_station_sample_name_uniq UNIQUE (name, station_id);


--
-- Name: biota cruise_station_sample_subset_biota_name_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.biota
    ADD CONSTRAINT cruise_station_sample_subset_biota_name_uniq UNIQUE (subset_id, id);


--
-- Name: dataset dataset_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.dataset
    ADD CONSTRAINT dataset_id_pk PRIMARY KEY (id);


--
-- Name: dataset dataset_name_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.dataset
    ADD CONSTRAINT dataset_name_uniq UNIQUE (name);


--
-- Name: gear gear_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.gear
    ADD CONSTRAINT gear_id_pk PRIMARY KEY (id);


--
-- Name: gear gear_name_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.gear
    ADD CONSTRAINT gear_name_uniq UNIQUE (name);


--
-- Name: imeasure_macro imeasure_macro_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.imeasure_macro
    ADD CONSTRAINT imeasure_macro_pk PRIMARY KEY (id);


--
-- Name: imeasure_meio imeasure_meio_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.imeasure_meio
    ADD CONSTRAINT imeasure_meio_pk PRIMARY KEY (id);


--
-- Name: ingest ingest_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.ingest
    ADD CONSTRAINT ingest_id_pk PRIMARY KEY (id);


--
-- Name: lifestage lifestage_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.lifestage
    ADD CONSTRAINT lifestage_id_pk PRIMARY KEY (id);


--
-- Name: lifestage lifestage_rule_name_has_to_be_unique; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.lifestage
    ADD CONSTRAINT lifestage_rule_name_has_to_be_unique UNIQUE (name);


--
-- Name: noworms noworms_rule_scientificname_has_to_be_unique; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.noworms
    ADD CONSTRAINT noworms_rule_scientificname_has_to_be_unique UNIQUE (scientificname);


--
-- Name: person person_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.person
    ADD CONSTRAINT person_id_pk PRIMARY KEY (id);


--
-- Name: person person_rule_name_has_to_be_unique; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.person
    ADD CONSTRAINT person_rule_name_has_to_be_unique UNIQUE (name);


--
-- Name: possys possys_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.possys
    ADD CONSTRAINT possys_id_pk PRIMARY KEY (id);


--
-- Name: possys possys_rule_name_has_to_be_unique; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.possys
    ADD CONSTRAINT possys_rule_name_has_to_be_unique UNIQUE (name);


--
-- Name: sample precision has to be in: [1,5] (integer); Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.sample
    ADD CONSTRAINT "precision has to be in: [1,5] (integer)" CHECK ((("precision" > 0) AND ("precision" < 6))) NOT VALID;


--
-- Name: region region_code_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.region
    ADD CONSTRAINT region_code_uniq UNIQUE (code);


--
-- Name: region region_name_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.region
    ADD CONSTRAINT region_name_uniq UNIQUE (name);


--
-- Name: region region_pkey; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.region
    ADD CONSTRAINT region_pkey PRIMARY KEY (id);


--
-- Name: region2sample sample2region_rule_sample_id_region_id_must_be_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.region2sample
    ADD CONSTRAINT sample2region_rule_sample_id_region_id_must_be_uniq UNIQUE (sample_id, region_id);


--
-- Name: sample sample_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.sample
    ADD CONSTRAINT sample_id_pk PRIMARY KEY (id);


--
-- Name: sediment sediment_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.sediment
    ADD CONSTRAINT sediment_id_pk PRIMARY KEY (id);


--
-- Name: sediment sediment_rule_fuz_clay_has_to_be_1_2_3; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.sediment
    ADD CONSTRAINT sediment_rule_fuz_clay_has_to_be_1_2_3 CHECK ((fuz_clay = ANY (ARRAY[1, 2, 3, NULL::integer]))) NOT VALID;


--
-- Name: sediment sediment_rule_fuz_csand_has_to_be_1_2_3; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.sediment
    ADD CONSTRAINT sediment_rule_fuz_csand_has_to_be_1_2_3 CHECK ((fuz_csand = ANY (ARRAY[1, 2, 3, NULL::integer]))) NOT VALID;


--
-- Name: sediment sediment_rule_fuz_fsand_has_to_be_1_2_3; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.sediment
    ADD CONSTRAINT sediment_rule_fuz_fsand_has_to_be_1_2_3 CHECK ((fuz_fsand = ANY (ARRAY[1, 2, 3, NULL::integer]))) NOT VALID;


--
-- Name: sediment sediment_rule_fuz_gravel_has_to_be_1_2_3; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.sediment
    ADD CONSTRAINT sediment_rule_fuz_gravel_has_to_be_1_2_3 CHECK ((fuz_gravel = ANY (ARRAY[1, 2, 3, NULL::integer]))) NOT VALID;


--
-- Name: sediment sediment_rule_fuz_msand_has_to_be_1_2_3; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.sediment
    ADD CONSTRAINT sediment_rule_fuz_msand_has_to_be_1_2_3 CHECK ((fuz_msand = ANY (ARRAY[1, 2, 3, NULL::integer]))) NOT VALID;


--
-- Name: sediment sediment_rule_fuz_shell_has_to_be_1_2_3; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.sediment
    ADD CONSTRAINT sediment_rule_fuz_shell_has_to_be_1_2_3 CHECK ((fuz_shell = ANY (ARRAY[1, 2, 3, NULL::integer]))) NOT VALID;


--
-- Name: sediment sediment_rule_fuz_silt_has_to_be_1_2_3; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.sediment
    ADD CONSTRAINT sediment_rule_fuz_silt_has_to_be_1_2_3 CHECK ((fuz_silt = ANY (ARRAY[1, 2, 3, NULL::integer]))) NOT VALID;


--
-- Name: sediment sediment_rule_fuz_stones_has_to_be_1_2_3; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.sediment
    ADD CONSTRAINT sediment_rule_fuz_stones_has_to_be_1_2_3 CHECK ((fuz_stones = ANY (ARRAY[1, 2, 3, NULL::integer]))) NOT VALID;


--
-- Name: sediment sediment_rule_fuz_vcsand_has_to_be_1_2_3; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.sediment
    ADD CONSTRAINT sediment_rule_fuz_vcsand_has_to_be_1_2_3 CHECK ((fuz_vcsand = ANY (ARRAY[1, 2, 3, NULL::integer]))) NOT VALID;


--
-- Name: sediment sediment_rule_fuz_vfsand_has_to_be_1_2_3; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.sediment
    ADD CONSTRAINT sediment_rule_fuz_vfsand_has_to_be_1_2_3 CHECK ((fuz_vfsand = ANY (ARRAY[1, 2, 3, NULL::integer]))) NOT VALID;


--
-- Name: ship ship_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.ship
    ADD CONSTRAINT ship_id_pk PRIMARY KEY (id);


--
-- Name: ship ship_name_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.ship
    ADD CONSTRAINT ship_name_uniq UNIQUE (name);


--
-- Name: station station_cruise_name_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.station
    ADD CONSTRAINT station_cruise_name_uniq UNIQUE (name, cruise_id);


--
-- Name: station station_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.station
    ADD CONSTRAINT station_id_pk PRIMARY KEY (id);


--
-- Name: station station_rule_precision_has_to_be_in_1-5; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.station
    ADD CONSTRAINT "station_rule_precision_has_to_be_in_1-5" CHECK ((("precision" > 0) AND ("precision" < 6))) NOT VALID;


--
-- Name: subsample subsample_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.subsample
    ADD CONSTRAINT subsample_id_pk PRIMARY KEY (id);


--
-- Name: subsample subsample_rule_ratio_has_to_be_in_0-1; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.subsample
    ADD CONSTRAINT "subsample_rule_ratio_has_to_be_in_0-1" CHECK (((ratio > (0)::double precision) AND (ratio <= (1)::double precision))) NOT VALID;


--
-- Name: subset subset_cruise_station_sample_name_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.subset
    ADD CONSTRAINT subset_cruise_station_sample_name_uniq UNIQUE (subsample_id, name);


--
-- Name: subset subset_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.subset
    ADD CONSTRAINT subset_id_pk PRIMARY KEY (id);


--
-- Name: sediment subset_id_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.sediment
    ADD CONSTRAINT subset_id_uniq UNIQUE (subset_id);


--
-- Name: subset subset_rule_both_spatiallevels_must_be_set_or_unset; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.subset
    ADD CONSTRAINT subset_rule_both_spatiallevels_must_be_set_or_unset CHECK ((((spatiallevel_top IS NOT NULL) AND (spatiallevel_bottom IS NOT NULL)) OR ((spatiallevel_top IS NULL) AND (spatiallevel_bottom IS NULL)))) NOT VALID;


--
-- Name: subset subset_rule_spatiallevel_bottom_between_0_1000; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.subset
    ADD CONSTRAINT subset_rule_spatiallevel_bottom_between_0_1000 CHECK (((spatiallevel_bottom >= 0) AND (spatiallevel_bottom <= 1000))) NOT VALID;


--
-- Name: subset subset_rule_spatiallevel_bottom_greater_top; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.subset
    ADD CONSTRAINT subset_rule_spatiallevel_bottom_greater_top CHECK ((spatiallevel_bottom >= spatiallevel_top)) NOT VALID;


--
-- Name: subset subset_rule_spatiallevel_top_between_0_1000; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.subset
    ADD CONSTRAINT subset_rule_spatiallevel_top_between_0_1000 CHECK (((spatiallevel_top >= 0) AND (spatiallevel_top <= 1000))) NOT VALID;


--
-- Name: subset subset_rule_spatiallevels_must_differ; Type: CHECK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE cb_empty.subset
    ADD CONSTRAINT subset_rule_spatiallevels_must_differ CHECK ((spatiallevel_top <> spatiallevel_bottom)) NOT VALID;


--
-- Name: taxcover taxcover_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.taxcover
    ADD CONSTRAINT taxcover_id_pk PRIMARY KEY (id);


--
-- Name: taxon taxon_aphiaid_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.taxon
    ADD CONSTRAINT taxon_aphiaid_uniq UNIQUE (aid);


--
-- Name: taxon taxon_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.taxon
    ADD CONSTRAINT taxon_id_pk PRIMARY KEY (id);


--
-- Name: wormsaaid wormasaaid_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.wormsaaid
    ADD CONSTRAINT wormasaaid_id_pk PRIMARY KEY (id);


--
-- Name: wormsaaid wormsaaid_aphiaid_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.wormsaaid
    ADD CONSTRAINT wormsaaid_aphiaid_uniq UNIQUE (aid);


--
-- Name: wormsbase wormsbase_aphiaid_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.wormsbase
    ADD CONSTRAINT wormsbase_aphiaid_uniq UNIQUE (aid);


--
-- Name: wormsbase wormsbase_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.wormsbase
    ADD CONSTRAINT wormsbase_id_pk UNIQUE (id);


--
-- Name: wormsprivate wormsbase_private_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.wormsprivate
    ADD CONSTRAINT wormsbase_private_uniq UNIQUE (private_name);


--
-- Name: wormschange wormschange_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.wormschange
    ADD CONSTRAINT wormschange_id_pk UNIQUE (id);


--
-- Name: wormscolony wormscolony_aphiaid_uniq; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.wormscolony
    ADD CONSTRAINT wormscolony_aphiaid_uniq UNIQUE (aid);


--
-- Name: wormscolony wormscolony_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.wormscolony
    ADD CONSTRAINT wormscolony_id_pk PRIMARY KEY (id);


--
-- Name: wormsprivate wormsprivate_id_pk; Type: CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.wormsprivate
    ADD CONSTRAINT wormsprivate_id_pk PRIMARY KEY (id);


--
-- Name: biogenic_ingest_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX biogenic_ingest_idx ON cb_empty.biogenic USING btree (ingest_id);


--
-- Name: biogenic_subset_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX biogenic_subset_idx ON cb_empty.biogenic USING btree (subset_id);


--
-- Name: biota_ingest_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX biota_ingest_idx ON cb_empty.biota USING btree (ingest_id);


--
-- Name: biota_lifestage_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX biota_lifestage_idx ON cb_empty.biota USING btree (lifestage_id);


--
-- Name: biota_subset_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX biota_subset_idx ON cb_empty.biota USING btree (subset_id);


--
-- Name: biota_taxon_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX biota_taxon_idx ON cb_empty.biota USING btree (taxon_id);


--
-- Name: cruise_crs_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX cruise_crs_idx ON cb_empty.cruise USING btree (crs_id);


--
-- Name: cruise_ingest_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX cruise_ingest_idx ON cb_empty.cruise USING btree (ingest_id);


--
-- Name: cruise_person_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX cruise_person_idx ON cb_empty.cruise USING btree (lead_person_id);


--
-- Name: cruise_possys_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX cruise_possys_idx ON cb_empty.cruise USING btree (positioningsystem_id);


--
-- Name: cruise_ship_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX cruise_ship_idx ON cb_empty.cruise USING btree (ship_id);


--
-- Name: dataset_country_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX dataset_country_idx ON cb_empty.dataset USING btree (owner_country_id);


--
-- Name: dataset_person_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX dataset_person_idx ON cb_empty.dataset USING btree (contact_person_id);


--
-- Name: idx_imeasure_macro_biota; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX idx_imeasure_macro_biota ON cb_empty.imeasure_macro USING btree (biota_id);


--
-- Name: idx_imeasure_macro_ingest; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX idx_imeasure_macro_ingest ON cb_empty.imeasure_macro USING btree (ingest_id);


--
-- Name: imeasure_meio_biota_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX imeasure_meio_biota_idx ON cb_empty.imeasure_meio USING btree (biota_id);


--
-- Name: imeasure_meio_ingest_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX imeasure_meio_ingest_idx ON cb_empty.imeasure_meio USING btree (ingest_id);


--
-- Name: sample2region_region_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX sample2region_region_idx ON cb_empty.region2sample USING btree (region_id);


--
-- Name: sample2region_sample_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX sample2region_sample_idx ON cb_empty.region2sample USING btree (sample_id);


--
-- Name: sample_gear_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX sample_gear_idx ON cb_empty.sample USING btree (gear_id);


--
-- Name: sample_ingest_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX sample_ingest_idx ON cb_empty.sample USING btree (ingest_id);


--
-- Name: sample_person_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX sample_person_idx ON cb_empty.sample USING btree (responsible_person_id);


--
-- Name: sample_station_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX sample_station_idx ON cb_empty.sample USING btree (station_id);


--
-- Name: sediment_ingest_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX sediment_ingest_idx ON cb_empty.sediment USING btree (ingest_id);


--
-- Name: sediment_subset_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX sediment_subset_idx ON cb_empty.sediment USING btree (subset_id);


--
-- Name: station_cruise_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX station_cruise_idx ON cb_empty.station USING btree (cruise_id);


--
-- Name: station_ingest_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX station_ingest_idx ON cb_empty.station USING btree (ingest_id);


--
-- Name: station_person_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX station_person_idx ON cb_empty.station USING btree (responsible_person_id);


--
-- Name: subsample_ingest_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX subsample_ingest_idx ON cb_empty.subsample USING btree (ingest_id);


--
-- Name: subsample_sample_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX subsample_sample_idx ON cb_empty.subsample USING btree (sample_id);


--
-- Name: subset_dataset_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX subset_dataset_idx ON cb_empty.subset USING btree (dataset_id);


--
-- Name: subset_ingest_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX subset_ingest_idx ON cb_empty.subset USING btree (ingest_id);


--
-- Name: subset_subsample_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX subset_subsample_idx ON cb_empty.subset USING btree (subsample_id);


--
-- Name: subset_taxcover_idx; Type: INDEX; Schema: cb_empty; Owner: -
--

CREATE INDEX subset_taxcover_idx ON cb_empty.subset USING btree (taxcover_id);


--
-- Name: gear gear_check_consistency; Type: TRIGGER; Schema: cb_empty; Owner: -
--

CREATE TRIGGER gear_check_consistency BEFORE INSERT OR UPDATE ON cb_empty.gear FOR EACH ROW EXECUTE FUNCTION cb_empty.new_gear_check_consistency();


--
-- Name: biota trigger_check_consistency; Type: TRIGGER; Schema: cb_empty; Owner: -
--

CREATE TRIGGER trigger_check_consistency AFTER INSERT OR UPDATE ON cb_empty.biota FOR EACH ROW EXECUTE FUNCTION cb_empty.new_biota_check_consistency();


--
-- Name: sample trigger_check_consistency; Type: TRIGGER; Schema: cb_empty; Owner: -
--

CREATE TRIGGER trigger_check_consistency AFTER INSERT OR UPDATE ON cb_empty.sample FOR EACH ROW EXECUTE FUNCTION cb_empty.new_sample_check_consistency();


--
-- Name: subsample trigger_check_consistency; Type: TRIGGER; Schema: cb_empty; Owner: -
--

CREATE TRIGGER trigger_check_consistency AFTER INSERT OR UPDATE ON cb_empty.subsample FOR EACH ROW EXECUTE FUNCTION cb_empty.new_subsample_check_consistency();


--
-- Name: subset trigger_check_consistency; Type: TRIGGER; Schema: cb_empty; Owner: -
--

CREATE TRIGGER trigger_check_consistency BEFORE INSERT OR UPDATE ON cb_empty.subset FOR EACH ROW EXECUTE FUNCTION cb_empty.new_subset_check_consistency();


--
-- Name: biogenic biogenic_ingest_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.biogenic
    ADD CONSTRAINT biogenic_ingest_fk FOREIGN KEY (ingest_id) REFERENCES cb_empty.ingest(id) NOT VALID;


--
-- Name: biogenic biogenic_subset_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.biogenic
    ADD CONSTRAINT biogenic_subset_fk FOREIGN KEY (subset_id) REFERENCES cb_empty.subset(id) NOT VALID;


--
-- Name: biota biota_ingest_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.biota
    ADD CONSTRAINT biota_ingest_fk FOREIGN KEY (ingest_id) REFERENCES cb_empty.ingest(id);


--
-- Name: biota biota_lifestage_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.biota
    ADD CONSTRAINT biota_lifestage_fk FOREIGN KEY (lifestage_id) REFERENCES cb_empty.lifestage(id);


--
-- Name: biota biota_subset_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.biota
    ADD CONSTRAINT biota_subset_fk FOREIGN KEY (subset_id) REFERENCES cb_empty.subset(id);


--
-- Name: biota biota_taxon_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.biota
    ADD CONSTRAINT biota_taxon_fk FOREIGN KEY (taxon_id) REFERENCES cb_empty.taxon(id);


--
-- Name: cruise cruise_crs_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.cruise
    ADD CONSTRAINT cruise_crs_fk FOREIGN KEY (crs_id) REFERENCES cb_empty.crs(id);


--
-- Name: cruise cruise_ingest_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.cruise
    ADD CONSTRAINT cruise_ingest_fk FOREIGN KEY (ingest_id) REFERENCES cb_empty.ingest(id);


--
-- Name: cruise cruise_person_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.cruise
    ADD CONSTRAINT cruise_person_fk FOREIGN KEY (lead_person_id) REFERENCES cb_empty.person(id);


--
-- Name: cruise cruise_possys_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.cruise
    ADD CONSTRAINT cruise_possys_fk FOREIGN KEY (positioningsystem_id) REFERENCES cb_empty.possys(id);


--
-- Name: cruise cruise_ship_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.cruise
    ADD CONSTRAINT cruise_ship_fk FOREIGN KEY (ship_id) REFERENCES cb_empty.ship(id);


--
-- Name: dataset dataset_country_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.dataset
    ADD CONSTRAINT dataset_country_fk FOREIGN KEY (owner_country_id) REFERENCES cb_empty.country(id);


--
-- Name: dataset dataset_person_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.dataset
    ADD CONSTRAINT dataset_person_fk FOREIGN KEY (contact_person_id) REFERENCES cb_empty.person(id);


--
-- Name: imeasure_macro imeasure_macro_biota_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.imeasure_macro
    ADD CONSTRAINT imeasure_macro_biota_fk FOREIGN KEY (biota_id) REFERENCES cb_empty.biota(id);


--
-- Name: imeasure_macro imeasure_macro_ingest_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.imeasure_macro
    ADD CONSTRAINT imeasure_macro_ingest_fk FOREIGN KEY (ingest_id) REFERENCES cb_empty.ingest(id);


--
-- Name: imeasure_meio imeasure_meio_biota_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.imeasure_meio
    ADD CONSTRAINT imeasure_meio_biota_fk FOREIGN KEY (biota_id) REFERENCES cb_empty.biota(id) NOT VALID;


--
-- Name: imeasure_meio inmeasure_meio_ingest_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.imeasure_meio
    ADD CONSTRAINT inmeasure_meio_ingest_fk FOREIGN KEY (ingest_id) REFERENCES cb_empty.ingest(id) NOT VALID;


--
-- Name: region2sample sample2region_region_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.region2sample
    ADD CONSTRAINT sample2region_region_fk FOREIGN KEY (region_id) REFERENCES cb_empty.region(id) NOT VALID;


--
-- Name: region2sample sample2region_sample_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.region2sample
    ADD CONSTRAINT sample2region_sample_fk FOREIGN KEY (sample_id) REFERENCES cb_empty.sample(id) NOT VALID;


--
-- Name: sample sample_gear_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.sample
    ADD CONSTRAINT sample_gear_fk FOREIGN KEY (gear_id) REFERENCES cb_empty.gear(id);


--
-- Name: sample sample_ingest_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.sample
    ADD CONSTRAINT sample_ingest_fk FOREIGN KEY (ingest_id) REFERENCES cb_empty.ingest(id);


--
-- Name: sample sample_person_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.sample
    ADD CONSTRAINT sample_person_fk FOREIGN KEY (responsible_person_id) REFERENCES cb_empty.person(id);


--
-- Name: sediment sample_sediment_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.sediment
    ADD CONSTRAINT sample_sediment_fk FOREIGN KEY (ingest_id) REFERENCES cb_empty.ingest(id);


--
-- Name: sample sample_station_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.sample
    ADD CONSTRAINT sample_station_fk FOREIGN KEY (station_id) REFERENCES cb_empty.station(id);


--
-- Name: sediment sediment_subset_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.sediment
    ADD CONSTRAINT sediment_subset_fk FOREIGN KEY (subset_id) REFERENCES cb_empty.subset(id);


--
-- Name: station station_cruise_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.station
    ADD CONSTRAINT station_cruise_fk FOREIGN KEY (cruise_id) REFERENCES cb_empty.cruise(id);


--
-- Name: station station_ingest_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.station
    ADD CONSTRAINT station_ingest_fk FOREIGN KEY (ingest_id) REFERENCES cb_empty.ingest(id);


--
-- Name: station station_person_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.station
    ADD CONSTRAINT station_person_fk FOREIGN KEY (responsible_person_id) REFERENCES cb_empty.person(id);


--
-- Name: subsample subsample_cruise_station_sample_name_uniq; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.subsample
    ADD CONSTRAINT subsample_cruise_station_sample_name_uniq FOREIGN KEY (sample_id) REFERENCES cb_empty.sample(id) NOT VALID;


--
-- Name: subsample subsample_ingest_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.subsample
    ADD CONSTRAINT subsample_ingest_fk FOREIGN KEY (ingest_id) REFERENCES cb_empty.ingest(id) NOT VALID;


--
-- Name: subsample subsample_sample_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.subsample
    ADD CONSTRAINT subsample_sample_fk FOREIGN KEY (sample_id) REFERENCES cb_empty.sample(id);


--
-- Name: subset subset_dataset_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.subset
    ADD CONSTRAINT subset_dataset_fk FOREIGN KEY (dataset_id) REFERENCES cb_empty.dataset(id);


--
-- Name: subset subset_ingest_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.subset
    ADD CONSTRAINT subset_ingest_fk FOREIGN KEY (ingest_id) REFERENCES cb_empty.ingest(id);


--
-- Name: subset subset_taxcover_fk; Type: FK CONSTRAINT; Schema: cb_empty; Owner: -
--

ALTER TABLE ONLY cb_empty.subset
    ADD CONSTRAINT subset_taxcover_fk FOREIGN KEY (taxcover_id) REFERENCES cb_empty.taxcover(id);


--
-- PostgreSQL database dump complete
--

