-- INFO
-- for all tables materialized views are made (free and used)
-- no constrains are defined (e.g. pk and fk)
-- except
-- 1) region (too big because of shapes)
-- 2) taxon (hard to track because of acceptance chains and no special confident data)

-- drops
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_combinations_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_compact_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_statistics_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_regions_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_projects_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_region2sample_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_ingest_view;

DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_dataset_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_gear_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_taxcover_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_country_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_crs_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_person_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_person_cruiseleader_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_possys_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_lifestage_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_ship_view;

DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_biota_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_imeasure_meio_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_imeasure_macro_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_sediment_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_biogenic_view;

DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_cruise_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_station_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_sample_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_subsample_view;
DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_subset_view;

-- branch: subset
-- here the magic happens: filter only visible -> base vor all other views
CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_subset_view AS
    SELECT
    sub.*
    FROM SCHEMAPLACEHOLDER.subset sub
    LEFT JOIN SCHEMAPLACEHOLDER.dataset dat ON dat.id = sub.dataset_id
    WHERE dat.access_level_data = 'free'::text;

-- branch: cruise station subsample sample
CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_cruise_view AS
    SELECT
    DISTINCT cru.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
    LEFT JOIN SCHEMAPLACEHOLDER.subsample ssp ON ssp.id = sub.subsample_id
	LEFT JOIN SCHEMAPLACEHOLDER.sample smp ON smp.id = ssp.sample_id
	LEFT JOIN SCHEMAPLACEHOLDER.station sta ON sta.id = smp.station_id
	LEFT JOIN SCHEMAPLACEHOLDER.cruise cru ON cru.id = sta.cruise_id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_station_view AS
    SELECT
    DISTINCT sta.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
    LEFT JOIN SCHEMAPLACEHOLDER.subsample ssp ON ssp.id = sub.subsample_id
	LEFT JOIN SCHEMAPLACEHOLDER.sample smp ON smp.id = ssp.sample_id
	LEFT JOIN SCHEMAPLACEHOLDER.station sta ON sta.id = smp.station_id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_sample_view AS
    SELECT
    DISTINCT smp.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
    LEFT JOIN SCHEMAPLACEHOLDER.subsample ssp ON ssp.id = sub.subsample_id
	LEFT JOIN SCHEMAPLACEHOLDER.sample smp ON smp.id = ssp.sample_id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_subsample_view AS
    SELECT
    DISTINCT ssp.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
    LEFT JOIN SCHEMAPLACEHOLDER.subsample ssp ON ssp.id = sub.subsample_id;

-- leaf: biota imeasure sediment
-- leaf: biota imeasure sediment
CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_biota_view AS
    SELECT
    DISTINCT bio.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
	LEFT JOIN SCHEMAPLACEHOLDER.biota bio ON bio.subset_id = sub.id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_imeasure_meio_view AS
    SELECT
    DISTINCT ind.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
	LEFT JOIN SCHEMAPLACEHOLDER.biota bio ON bio.subset_id = sub.id
	LEFT JOIN SCHEMAPLACEHOLDER.imeasure_meio ind ON ind.biota_id = bio.id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_imeasure_macro_view AS
    SELECT
    DISTINCT ind.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
	LEFT JOIN SCHEMAPLACEHOLDER.biota bio ON bio.subset_id = sub.id
	LEFT JOIN SCHEMAPLACEHOLDER.imeasure_macro ind ON ind.biota_id = bio.id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_sediment_view AS
    SELECT
    DISTINCT sed.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
	LEFT JOIN SCHEMAPLACEHOLDER.sediment sed ON sed.subset_id = sub.id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_biogenic_view AS
    SELECT
    DISTINCT bgen.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
	LEFT JOIN SCHEMAPLACEHOLDER.biogenic bgen ON bgen.subset_id = sub.id;

-- lookup
CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_ship_view AS
    SELECT
    DISTINCT shp.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
    LEFT JOIN SCHEMAPLACEHOLDER.subsample ssp ON ssp.id = sub.subsample_id
	LEFT JOIN SCHEMAPLACEHOLDER.sample smp ON smp.id = ssp.sample_id
	LEFT JOIN SCHEMAPLACEHOLDER.station sta ON sta.id = smp.station_id
	LEFT JOIN SCHEMAPLACEHOLDER.cruise cru ON cru.id = sta.cruise_id
	LEFT JOIN SCHEMAPLACEHOLDER.ship shp ON cru.ship_id = shp.id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_crs_view AS
    SELECT
    DISTINCT crs.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
    LEFT JOIN SCHEMAPLACEHOLDER.subsample ssp ON ssp.id = sub.subsample_id
	LEFT JOIN SCHEMAPLACEHOLDER.sample smp ON smp.id = ssp.sample_id
	LEFT JOIN SCHEMAPLACEHOLDER.station sta ON sta.id = smp.station_id
	LEFT JOIN SCHEMAPLACEHOLDER.cruise cru ON cru.id = sta.cruise_id
	LEFT JOIN SCHEMAPLACEHOLDER.crs crs ON cru.crs_id = crs.id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_lifestage_view AS
	SELECT
    DISTINCT lif.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
	LEFT JOIN SCHEMAPLACEHOLDER.biota bio ON bio.subset_id = sub.id
	LEFT JOIN SCHEMAPLACEHOLDER.lifestage lif ON bio.lifestage_id = lif.id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_possys_view AS
    SELECT
    DISTINCT pos.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
    LEFT JOIN SCHEMAPLACEHOLDER.subsample ssp ON ssp.id = sub.subsample_id
	LEFT JOIN SCHEMAPLACEHOLDER.sample smp ON smp.id = ssp.sample_id
	LEFT JOIN SCHEMAPLACEHOLDER.station sta ON sta.id = smp.station_id
	LEFT JOIN SCHEMAPLACEHOLDER.cruise cru ON cru.id = sta.cruise_id
	LEFT JOIN SCHEMAPLACEHOLDER.possys pos ON cru.positioningsystem_id = pos.id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_country_view AS
    SELECT
    DISTINCT ctr.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
    LEFT JOIN SCHEMAPLACEHOLDER.subsample ssp ON ssp.id = sub.subsample_id
	LEFT JOIN SCHEMAPLACEHOLDER.sample smp ON smp.id = ssp.sample_id
	LEFT JOIN SCHEMAPLACEHOLDER.station sta ON sta.id = smp.station_id
	LEFT JOIN SCHEMAPLACEHOLDER.cruise cru ON cru.id = sta.cruise_id
    LEFT JOIN SCHEMAPLACEHOLDER.dataset dat ON dat.id = sub.dataset_id
	LEFT JOIN SCHEMAPLACEHOLDER.country ctr ON ctr.id = dat.owner_country_id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_person_view AS
    SELECT
    DISTINCT per.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
    LEFT JOIN SCHEMAPLACEHOLDER.subsample ssp ON ssp.id = sub.subsample_id
	LEFT JOIN SCHEMAPLACEHOLDER.sample smp ON smp.id = ssp.sample_id
	LEFT JOIN SCHEMAPLACEHOLDER.station sta ON sta.id = smp.station_id
	LEFT JOIN SCHEMAPLACEHOLDER.cruise cru ON cru.id = sta.cruise_id
	LEFT JOIN SCHEMAPLACEHOLDER.ingest ing ON ing.id = smp.ingest_id
    LEFT JOIN SCHEMAPLACEHOLDER.dataset dat ON dat.id = sub.dataset_id
	LEFT JOIN SCHEMAPLACEHOLDER.country ctr ON ctr.id = dat.owner_country_id
    LEFT JOIN SCHEMAPLACEHOLDER.gear gea ON gea.id = smp.gear_id
	LEFT JOIN SCHEMAPLACEHOLDER.person per ON (
		per.id = cru.lead_person_id OR
		per.id = sta.responsible_person_id OR
		per.id = smp.responsible_person_id OR
		per.id = dat.contact_person_id);

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_person_cruiseleader_view AS
    SELECT
    DISTINCT per.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
    LEFT JOIN SCHEMAPLACEHOLDER.subsample ssp ON ssp.id = sub.subsample_id
	LEFT JOIN SCHEMAPLACEHOLDER.sample smp ON smp.id = ssp.sample_id
	LEFT JOIN SCHEMAPLACEHOLDER.station sta ON sta.id = smp.station_id
	LEFT JOIN SCHEMAPLACEHOLDER.cruise cru ON cru.id = sta.cruise_id
	LEFT JOIN SCHEMAPLACEHOLDER.person per ON per.id = cru.lead_person_id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_dataset_view AS
    SELECT
    DISTINCT dat.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
    LEFT JOIN SCHEMAPLACEHOLDER.dataset dat ON dat.id = sub.dataset_id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_taxcover_view AS
    SELECT
    DISTINCT cov.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
	LEFT JOIN SCHEMAPLACEHOLDER.taxcover cov ON cov.id = sub.taxcover_id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_gear_view AS
    SELECT
    DISTINCT gea.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
    LEFT JOIN SCHEMAPLACEHOLDER.subsample ssp ON ssp.id = sub.subsample_id
	LEFT JOIN SCHEMAPLACEHOLDER.sample smp ON smp.id = ssp.sample_id
    LEFT JOIN SCHEMAPLACEHOLDER.gear gea ON gea.id = smp.gear_id;

-- special: compact combinations staistics projects region2sample
CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_compact_view AS
    SELECT
	cru.id AS cruise_id,
    cru.name AS cruise,
	sta.id AS station_id,
    sta.name AS station,
    smp.id AS sample_id,
    smp.name AS sample,
    sub.id AS subset_id,
    sub.name AS subset,
	ssp.id AS subsample_id,
	ssp.name AS subsample,
    smp.lat_start,
    smp.lon_start,
	((smp.date_start || ' '::text) || COALESCE(smp.time_start, '00:00:00'))::timestamp without time zone AS datetime,
	smp.date_start,
	smp.time_start,
	sub.taxres AS taxonomic_resolution,
    dat.name AS dataset,
    dat.id AS dataset_id,
    smp.precision AS spatial_precision,
    dat.access_level_data,
    dat.access_level_data_product,
	dat.institution,
    dat.reference_person,
    dat.doi,
    cov.name AS taxcover,
    cov.id AS taxcover_id,
    cov.name AS taxonomic_coverage,
	per.full_name AS contact_person,
	per.email AS contact_person_email,
	ctr.name AS owner_country,
	gea.name AS gear,
	gea.id AS gear_id,
    gea.category AS gear_category,
    ing.id AS ingest_id,
    ing.workflow AS ingest_workflow,
	reg.name AS region,
	reg.id AS region_id
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
    LEFT JOIN SCHEMAPLACEHOLDER.subsample ssp ON ssp.id = sub.subsample_id
	LEFT JOIN SCHEMAPLACEHOLDER.sample smp ON smp.id = ssp.sample_id
	LEFT JOIN SCHEMAPLACEHOLDER.station sta ON sta.id = smp.station_id
	LEFT JOIN SCHEMAPLACEHOLDER.cruise cru ON cru.id = sta.cruise_id
	LEFT JOIN SCHEMAPLACEHOLDER.ingest ing ON ing.id = smp.ingest_id
    LEFT JOIN SCHEMAPLACEHOLDER.dataset dat ON dat.id = sub.dataset_id
	LEFT JOIN SCHEMAPLACEHOLDER.country ctr ON ctr.id = dat.owner_country_id
    LEFT JOIN SCHEMAPLACEHOLDER.gear gea ON gea.id = smp.gear_id
	LEFT JOIN SCHEMAPLACEHOLDER.person per ON per.id = dat.contact_person_id
	LEFT JOIN SCHEMAPLACEHOLDER.taxcover cov ON cov.id = sub.taxcover_id
	LEFT JOIN SCHEMAPLACEHOLDER.region2sample reg2smp ON smp.id = reg2smp.sample_id
	LEFT JOIN SCHEMAPLACEHOLDER.region reg ON reg2smp.region_id = reg.id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_combinations_view AS
	SELECT DISTINCT ingest_workflow, ingest_id, gear_category, gear_id, dataset, dataset_id, taxcover, taxcover_id, min(date_start) AS min_date, max(date_start) AS max_date, region, region_id
	FROM SCHEMAPLACEHOLDER.ws_compact_view
	Group BY ingest_workflow, ingest_id, gear_category, gear_id, dataset, dataset_id, taxcover, taxcover_id, region, region_id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_statistics_view AS
    SELECT
        0 AS id,
        (SELECT COUNT(*) FROM SCHEMAPLACEHOLDER.ws_dataset_view) AS datasets,
        (SELECT COUNT(*) FROM SCHEMAPLACEHOLDER.ws_cruise_view) AS expeditions,
        (SELECT COUNT(*) FROM SCHEMAPLACEHOLDER.ws_station_view) AS stations,
        (SELECT COUNT(*) FROM SCHEMAPLACEHOLDER.ws_sample_view) AS samples,
        (SELECT COUNT(*) FROM SCHEMAPLACEHOLDER.ws_biota_view) AS biotas,
        (SELECT COUNT(*) FROM SCHEMAPLACEHOLDER.ws_sediment_view) AS sediments,
        (SELECT COUNT(DISTINCT taxon_id) FROM SCHEMAPLACEHOLDER.ws_biota_view) AS taxa;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_projects_view AS
    SELECT
    DISTINCT ing.workflow
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
	LEFT JOIN SCHEMAPLACEHOLDER.ingest ing ON ing.id = sub.ingest_id
    LEFT JOIN SCHEMAPLACEHOLDER.dataset dat ON dat.id = sub.dataset_id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_region2sample_view AS
    SELECT
    DISTINCT reg2smp.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
    LEFT JOIN SCHEMAPLACEHOLDER.subsample ssp ON ssp.id = sub.subsample_id
	LEFT JOIN SCHEMAPLACEHOLDER.sample smp ON smp.id = ssp.sample_id
	LEFT JOIN SCHEMAPLACEHOLDER.region2sample reg2smp ON smp.id = reg2smp.sample_id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_regions_view AS
    SELECT
    DISTINCT reg.id, reg.name, reg.code, reg.level
    FROM SCHEMAPLACEHOLDER.ws_region2sample_view reg2smp
	LEFT JOIN SCHEMAPLACEHOLDER.region reg ON reg.id = reg2smp.region_id;

CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_ingest_view AS
    SELECT
    DISTINCT ing.*
    FROM SCHEMAPLACEHOLDER.ws_subset_view sub
    LEFT JOIN SCHEMAPLACEHOLDER.subsample ssp ON ssp.id = sub.subsample_id
	LEFT JOIN SCHEMAPLACEHOLDER.sample smp ON smp.id = ssp.sample_id
	LEFT JOIN SCHEMAPLACEHOLDER.station sta ON sta.id = smp.station_id
	LEFT JOIN SCHEMAPLACEHOLDER.cruise cru ON cru.id = sta.cruise_id
	LEFT JOIN SCHEMAPLACEHOLDER.ingest ing ON (
		ing.id = cru.ingest_id OR
		ing.id = sta.ingest_id OR
		ing.id = smp.ingest_id OR
		ing.id = ssp.ingest_id OR
		ing.id = sub.ingest_id
	);

-- FK indexes

CREATE INDEX fki_view_combinations_ingest ON SCHEMAPLACEHOLDER.ws_combinations_view USING btree (ingest_id);
CREATE INDEX fki_view_combinations_gear ON SCHEMAPLACEHOLDER.ws_combinations_view USING btree (gear_id);
CREATE INDEX fki_view_combinations_dataset ON SCHEMAPLACEHOLDER.ws_combinations_view USING btree (dataset_id);
CREATE INDEX fki_view_combinations_taxcover ON SCHEMAPLACEHOLDER.ws_combinations_view USING btree (taxcover_id);
CREATE INDEX fki_view_combinations_region ON SCHEMAPLACEHOLDER.ws_combinations_view USING btree (region_id);

CREATE INDEX fki_view_compact_ingest ON SCHEMAPLACEHOLDER.ws_compact_view USING btree (ingest_id);
CREATE INDEX fki_view_compact_cruise ON SCHEMAPLACEHOLDER.ws_compact_view USING btree (cruise_id);
CREATE INDEX fki_view_compact_station ON SCHEMAPLACEHOLDER.ws_compact_view USING btree (station_id);
CREATE INDEX fki_view_compact_samnple ON SCHEMAPLACEHOLDER.ws_compact_view USING btree (sample_id);
CREATE INDEX fki_view_compact_subsample ON SCHEMAPLACEHOLDER.ws_compact_view USING btree (subsample_id);
CREATE INDEX fki_view_compact_subet ON SCHEMAPLACEHOLDER.ws_compact_view USING btree (subset_id);
CREATE INDEX fki_view_compact_gear ON SCHEMAPLACEHOLDER.ws_compact_view USING btree (gear_id);
CREATE INDEX fki_view_compact_dataset ON SCHEMAPLACEHOLDER.ws_compact_view USING btree (dataset_id);
CREATE INDEX fki_view_compact_taxcover ON SCHEMAPLACEHOLDER.ws_compact_view USING btree (taxcover_id);
CREATE INDEX fki_view_compact_region ON SCHEMAPLACEHOLDER.ws_compact_view USING btree (region_id);

CREATE INDEX fki_view_cruise_ingest ON SCHEMAPLACEHOLDER.ws_cruise_view USING btree (ingest_id);
CREATE INDEX fki_view_cruise_possys ON SCHEMAPLACEHOLDER.ws_cruise_view USING btree (positioningsystem_id);
CREATE INDEX fki_view_cruise_crs ON SCHEMAPLACEHOLDER.ws_cruise_view USING btree (crs_id);
CREATE INDEX fki_view_cruise_ship ON SCHEMAPLACEHOLDER.ws_cruise_view USING btree (ship_id);
CREATE INDEX fki_view_cruise_person ON SCHEMAPLACEHOLDER.ws_cruise_view USING btree (lead_person_id);

CREATE INDEX fki_view_station_ingest ON SCHEMAPLACEHOLDER.ws_station_view USING btree (ingest_id);
CREATE INDEX fki_view_station_cruise ON SCHEMAPLACEHOLDER.ws_station_view USING btree (cruise_id);
CREATE INDEX fki_view_station_person ON SCHEMAPLACEHOLDER.ws_station_view USING btree (responsible_person_id);

CREATE INDEX fki_view_sample_ingest ON SCHEMAPLACEHOLDER.ws_sample_view USING btree (ingest_id);
CREATE INDEX fki_view_sample_station ON SCHEMAPLACEHOLDER.ws_sample_view USING btree (station_id);
CREATE INDEX fki_view_sample_person ON SCHEMAPLACEHOLDER.ws_sample_view USING btree (responsible_person_id);
CREATE INDEX fki_view_sample_gear ON SCHEMAPLACEHOLDER.ws_sample_view USING btree (gear_id);

CREATE INDEX fki_view_subsample_ingest ON SCHEMAPLACEHOLDER.ws_subsample_view USING btree (ingest_id);
CREATE INDEX fki_view_subsample_sample ON SCHEMAPLACEHOLDER.ws_subsample_view USING btree (sample_id);

CREATE INDEX fki_view_subset_ingest ON SCHEMAPLACEHOLDER.ws_subset_view USING btree (ingest_id);
CREATE INDEX fki_view_subset_subsample ON SCHEMAPLACEHOLDER.ws_subset_view USING btree (subsample_id);
CREATE INDEX fki_view_subset_dataset ON SCHEMAPLACEHOLDER.ws_subset_view USING btree (dataset_id);
CREATE INDEX fki_view_subset_taxcover ON SCHEMAPLACEHOLDER.ws_subset_view USING btree (taxcover_id);

CREATE INDEX fki_view_biota_ingest ON SCHEMAPLACEHOLDER.ws_biota_view USING btree (ingest_id);
CREATE INDEX fki_view_biota_subset ON SCHEMAPLACEHOLDER.ws_biota_view USING btree (subset_id);
CREATE INDEX fki_view_biota_taxon ON SCHEMAPLACEHOLDER.ws_biota_view USING btree (taxon_id);
CREATE INDEX fki_view_biota_lifestage ON SCHEMAPLACEHOLDER.ws_biota_view USING btree (lifestage_id);

CREATE INDEX fki_view_imeasure_meio_ingest ON SCHEMAPLACEHOLDER.ws_imeasure_meio_view USING btree (ingest_id);
CREATE INDEX fki_view_imeasure_meio_biota ON SCHEMAPLACEHOLDER.ws_imeasure_meio_view USING btree (biota_id);

CREATE INDEX fki_view_imeasure_macro_ingest ON SCHEMAPLACEHOLDER.ws_imeasure_macro_view USING btree (ingest_id);
CREATE INDEX fki_view_imeasure_macro_biota ON SCHEMAPLACEHOLDER.ws_imeasure_macro_view USING btree (biota_id);

CREATE INDEX fki_view_sediment_ingest ON SCHEMAPLACEHOLDER.ws_sediment_view USING btree (ingest_id);
CREATE INDEX fki_view_sediment_subset ON SCHEMAPLACEHOLDER.ws_sediment_view USING btree (subset_id);

CREATE INDEX fki_view_biogenic_ingest ON SCHEMAPLACEHOLDER.ws_biogenic_view USING btree (ingest_id);
CREATE INDEX fki_view_biogenic_subset ON SCHEMAPLACEHOLDER.ws_biogenic_view USING btree (subset_id);

CREATE INDEX fki_view_dataset_person ON SCHEMAPLACEHOLDER.ws_dataset_view USING btree (contact_person_id);
CREATE INDEX fki_view_dataset_country ON SCHEMAPLACEHOLDER.ws_dataset_view USING btree (owner_country_id);

CREATE INDEX fki_view_sample2region_sample ON SCHEMAPLACEHOLDER.ws_region2sample_view USING btree (sample_id);
CREATE INDEX fki_view_sample2region_region ON SCHEMAPLACEHOLDER.ws_region2sample_view USING btree (region_id);

-- rights
-- Boss (and public)
-- specials
ALTER TABLE SCHEMAPLACEHOLDER.ws_compact_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_compact_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_compact_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_combinations_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_combinations_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_combinations_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_statistics_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_statistics_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_statistics_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_projects_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_projects_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_projects_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_region2sample_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_region2sample_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_region2sample_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_regions_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_regions_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_regions_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_ingest_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_ingest_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_ingest_view TO critterboss;

--- lookups
ALTER TABLE SCHEMAPLACEHOLDER.ws_dataset_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_dataset_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_dataset_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_gear_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_gear_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_gear_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_taxcover_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_taxcover_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_taxcover_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_crs_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_crs_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_crs_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_possys_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_possys_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_possys_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_person_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_person_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_person_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_person_cruiseleader_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_person_cruiseleader_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_person_cruiseleader_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_country_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_country_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_country_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_lifestage_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_lifestage_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_lifestage_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_ship_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_ship_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_ship_view TO critterboss;

-- branch
ALTER TABLE SCHEMAPLACEHOLDER.ws_cruise_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_cruise_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_cruise_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_station_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_station_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_station_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_sample_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_sample_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_sample_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_subsample_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_subsample_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_subsample_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_subset_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_subset_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_subset_view TO critterboss;

--- leaf
ALTER TABLE SCHEMAPLACEHOLDER.ws_biota_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_biota_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_biota_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_imeasure_meio_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_imeasure_meio_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_imeasure_meio_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_imeasure_macro_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_imeasure_macro_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_imeasure_macro_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_sediment_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_sediment_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_sediment_view TO critterboss;

ALTER TABLE SCHEMAPLACEHOLDER.ws_biogenic_view OWNER TO critterboss;
GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_biogenic_view TO public;
GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_biogenic_view TO critterboss;

-- Read
GRANT SELECT ON ALL TABLES IN SCHEMA SCHEMAPLACEHOLDER TO critterread;
GRANT USAGE ON SCHEMA SCHEMAPLACEHOLDER TO critterread;


-- SCRIBBLE ************************

--DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_taxa_view;
--DROP MATERIALIZED VIEW IF EXISTS SCHEMAPLACEHOLDER.ws_datasets_view;

---- GENERIC USED NAMES
--CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_datasets_view AS
--   	SELECT DISTINCT(dat.name) AS dataset
--    FROM ((((((((((SCHEMAPLACEHOLDER.subset sub
--    LEFT JOIN SCHEMAPLACEHOLDER.subsample ssp ON ((ssp.id = sub.subsample_id)))
--	LEFT JOIN SCHEMAPLACEHOLDER.sample smp ON ((smp.id = ssp.sample_id)))
--	LEFT JOIN SCHEMAPLACEHOLDER.station sta ON ((sta.id = smp.station_id)))
--	LEFT JOIN SCHEMAPLACEHOLDER.cruise cru ON ((cru.id = sta.cruise_id)))
--	LEFT JOIN SCHEMAPLACEHOLDER.ingest ing ON ((ing.id = smp.ingest_id)))
--    LEFT JOIN SCHEMAPLACEHOLDER.dataset dat ON ((dat.id = sub.dataset_id)))
--	LEFT JOIN SCHEMAPLACEHOLDER.country ctr ON ((ctr.id = dat.owner_country_id)))
--    LEFT JOIN SCHEMAPLACEHOLDER.gear gea ON ((gea.id = smp.gear_id)))
--	LEFT JOIN SCHEMAPLACEHOLDER.person per ON ((per.id = dat.contact_person_id)))
--	LEFT JOIN SCHEMAPLACEHOLDER.taxcover cov ON ((cov.id = sub.taxcover_id)))
--    WHERE (dat.access_level_data = 'free'::text)
--	ORDER BY dat.name;

--ALTER TABLE SCHEMAPLACEHOLDER.ws_taxa_view OWNER TO critterboss;
--GRANT SELECT ON TABLE SCHEMAPLACEHOLDER.ws_taxa_view TO public;
--GRANT ALL ON TABLE SCHEMAPLACEHOLDER.ws_taxa_view TO critterboss;


--CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_subsets_view AS
--SELECT
--v.*, (
--	SELECT reg.name AS "region"
--	FROM SCHEMAPLACEHOLDER.region reg
--	LEFT JOIN SCHEMAPLACEHOLDER.region2sample reg2smp ON v.id = reg2smp.sample_id
--	WHERE reg2smp.region_id = reg.id
--)
--FROM SCHEMAPLACEHOLDER.ws_subsets_view_noregions v;

---- used datasets
--CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_datasets_view AS
--   	SELECT DISTINCT(dat.name) AS dataset
--    FROM ((((((((((SCHEMAPLACEHOLDER.subset sub
--    LEFT JOIN SCHEMAPLACEHOLDER.subsample ssp ON ((ssp.id = sub.subsample_id)))
--	LEFT JOIN SCHEMAPLACEHOLDER.sample smp ON ((smp.id = ssp.sample_id)))
--	LEFT JOIN SCHEMAPLACEHOLDER.station sta ON ((sta.id = smp.station_id)))
--	LEFT JOIN SCHEMAPLACEHOLDER.cruise cru ON ((cru.id = sta.cruise_id)))
--    LEFT JOIN SCHEMAPLACEHOLDER.dataset dat ON ((dat.id = sub.dataset_id)))
--    WHERE (dat.access_level_data = 'free'::text)
--	ORDER BY dat.name;
--SELECT * FROM SCHEMAPLACEHOLDER.ws_datasets_view;

---- used taxa
--CREATE MATERIALIZED VIEW SCHEMAPLACEHOLDER.ws_taxa_view AS
--   	select distinct tax.aid, tax.aname
--	from SCHEMAPLACEHOLDER.biota bio
--	join SCHEMAPLACEHOLDER.taxon tax on bio.taxon_id=tax.id
--	order by tax.aname;